# Snarlygribbly's Poser Particles
# Copyright 2015 Royston Harwood

# Import libraries
import poser, wx, os.path, math, random, string, webbrowser, wx.lib.colourdb, ConfigParser
import wx.lib.agw.floatspin as FS, wx.lib.filebrowsebutton as FB
from wx.lib.embeddedimage import PyEmbeddedImage

# Global application constants
APPNAME = 'Particles 3+'
POSER_VERSION = int(float(poser.Version()))
POSER_BUILD = int(poser.AppVersion().rsplit('.',1)[1])
VERSION = '1.0'
BUILD = '033'
APPTITLE = APPNAME
APPCAPTION = APPTITLE
APPSIZE = (800,650)
USE_RANDOM = True

# Script configuration
UI_WIDTH = 800
UI_HEIGHT = 600

# PRT3 Constants
IPARM_TYPE = "PRT3_Type"
IPARM_SHAPE = "PRT3_Shape"
IPARM_COUNT = "PRT3_Count"

# SIMULATION
IPARM_EMISSIONANGLE = "PRT3_AngleOfEmission"
IPARM_EMISSION = "PRT3_RateOfEmission"
IPARM_RECYCLE = "PRT3_Recycle"
IPARM_LIFESPAN = "PRT3_Lifespan"
IPARM_START = "PRT3_Start"
IPARM_END = "PRT3_End"
IPARM_GRAVITY = "PRT3_Gravity"
IPARM_ATTRACTION = "PRT3_AttractionStrength"
IPARM_RANGE = "PRT3_AttractionRange"
IPARM_FALLOFF = "PRT3_AttractionFalloff"
IPARM_DAMPENING = "PRT3_Dampening"

# VELOCITY
IPARM_VELOCITYX = "PRT3_VelocityX"
IPARM_VELOCITYY = "PRT3_VelocityY"
IPARM_VELOCITYZ = "PRT3_VelocityZ"
IPARM_ACCELERATIONX = "PRT3_AccelerationX"
IPARM_ACCELERATIONY = "PRT3_AccelerationY"
IPARM_ACCELERATIONZ = "PRT3_AccelerationZ"
IPARM_VELOCITYVAR = "PRT3_VelocityVar"

# ORBIT
IPARM_ORBITX = "PRT3_OrbitX"
IPARM_ORBITY = "PRT3_OrbitY"
IPARM_ORBITZ = "PRT3_OrbitZ"
IPARM_ORBITAX = "PRT3_OrbitAX"
IPARM_ORBITAY = "PRT3_OrbitAY"
IPARM_ORBITAZ = "PRT3_OrbitAZ"
IPARM_ORBITVAR = "PRT3_OrbitVar"

# SPIN
IPARM_SPINX = "PRT3_SpinX"
IPARM_SPINY = "PRT3_SpinY"
IPARM_SPINZ = "PRT3_SpinZ"
IPARM_SPINAX = "PRT3_SpinAX"
IPARM_SPINAY = "PRT3_SpinAY"
IPARM_SPINAZ = "PRT3_SpinAZ"
IPARM_SPINVAR = "PRT3_SpinVar"

# GROW
IPARM_GROWX = "PRT3_GrowX"
IPARM_GROWY = "PRT3_GrowY"
IPARM_GROWZ = "PRT3_GrowZ"
IPARM_GROWAX = "PRT3_GrowAX"
IPARM_GROWAY = "PRT3_GrowAY"
IPARM_GROWAZ = "PRT3_GrowAZ"
IPARM_GROWVAR = "PRT3_GrowVar"

# PARTICLE TRANSFORM
IPARM_PSCALE = "PRT3_PScale"
IPARM_PXSCALE = "PRT3_PXScale"
IPARM_PYSCALE = "PRT3_PYScale"
IPARM_PZSCALE = "PRT3_PZScale"
IPARM_PXROTATE = "PRT3_PXRotate"
IPARM_PYROTATE = "PRT3_PYRotate"
IPARM_PZROTATE = "PRT3_PZRotate"
IPARM_PXTRAN = "PRT3_PXTrans"
IPARM_PYTRAN = "PRT3_PYTrans"
IPARM_PZTRAN = "PRT3_PZTrans"

# HIDDEN
IPARM_ID = "PRT3_SysId"
IPARM_LIFE = "PRT3_Life"
IPARM_SPAWNPNTX = "PRT3_SpawnPntX"
IPARM_SPAWNPNTY = "PRT3_SpawnPntY"
IPARM_SPAWNPNTZ = "PRT3_SpawnPntZ"
IPARM_SPAWNVECX = "PRT3_SpawnVecX"
IPARM_SPAWNVECY = "PRT3_SpawnVecY"
IPARM_SPAWNVECZ = "PRT3_SpawnVecZ"

# SCALE MODIFIERS
IPARM_XSCALEMOD = "PRT3_xScaleMod"
IPARM_YSCALEMOD = "PRT3_yScaleMod"
IPARM_ZSCALEMOD = "PRT3_zScaleMod"

PARM_ID = "PRT3_SysId"
PARM_TYPE = "Type"
PARM_SHAPE = "Shape"
PARM_COUNT = "Count"
PARM_EMISSION = "Rate of Emission"
PARM_RECYCLE = "Recycle Particles"
PARM_LIFESPAN = "Particle Lifespan"
PARM_START = "Start Frame"
PARM_END = "End Frame"
PARM_GRAVITY = "Gravity"
PARM_VARIANCE = "Variance"
PARM_VELOCITYX = "Velocity X"
PARM_VELOCITYY = "Velocity Y"
PARM_VELOCITYZ = "Velocity Z"
PARM_ACCELERATIONX = "Acceleration X"
PARM_ACCELERATIONY = "Acceleration Y"
PARM_ACCELERATIONZ = "Acceleration Z"
PARM_VELOCITYVAR = "Velocity Variance"
PARM_ORBITX = "Orbit X"
PARM_ORBITY = "Orbit Y"
PARM_ORBITZ = "Orbit Z"
PARM_ORBITAX = "Orbit Acceleration X"
PARM_ORBITAY = "Orbit Acceleration Y"
PARM_ORBITAZ = "Orbit Acceleration Z"
PARM_ORBITVAR = "Orbit Variance"
PARM_SPINX = "Spin X"
PARM_SPINY = "Spin Y"
PARM_SPINZ = "Spin Z"
PARM_SPINAX = "Spin Acceleration X"
PARM_SPINAY = "Spin Acceleration Y"
PARM_SPINAZ = "Spin Acceleration Z"
PARM_SPINVAR = "Spin Variance"
PARM_GROWX = "Grow X"
PARM_GROWY = "Grow Y"
PARM_GROWZ = "Grow Z"
PARM_GROWAX = "Grow Acceleration X"
PARM_GROWAY = "Grow Acceleration Y"
PARM_GROWAZ = "Grow Acceleration Z"
PARM_GROWVAR = "Grow Variance"
PARM_PSCALE = "Initial Scale"
PARM_PXSCALE = "Initial xScale"
PARM_PYSCALE = "Initial yScale"
PARM_PZSCALE = "Initial zScale"
PARM_PXROTATE = "Initial xRotate"
PARM_PYROTATE = "Initial yRotate"
PARM_PZROTATE = "Initial zRotate"
PARM_PXTRAN = "Initial xOffset"
PARM_PYTRAN = "Initial yOffset"
PARM_PZTRAN = "Initial zOffset"
PARM_XSCALEMOD = "PRT3_xScaleMod"
PARM_YSCALEMOD = "PRT3_yScaleMod"
PARM_ZSCALEMOD = "PRT3_zScaleMod"
PARM_LIFE = "Life"
PARM_SPAWNPNTX = "Spawn Point X"
PARM_SPAWNPNTY = "Spawn Point Y"
PARM_SPAWNPNTZ = "Spawn Point Z"
PARM_SPAWNVECX = "Spawn Vector X"
PARM_SPAWNVECY = "Spawn Vector Y"
PARM_SPAWNVECZ = "Spawn Vector Z"
PARM_EMISSIONANGLE = "Angle of Emission"
PARM_ATTRACTION = "Strength"
PARM_RANGE = "Range"
PARM_FALLOFF = "Falloff"
PARM_DAMPENING = "Dampening"

SYSTEM_TYPE = 0;
EMITTER_TYPE = 1;
GROUP_TYPE = 2;
PARTICLE_TYPE = 3;
ATTRACTOR_TYPE = 4;
DEFLECTOR_TYPE = 5;

SHAPE_CUSTOM = 0
SHAPE_BALL = 1
SHAPE_TORUS = 2
SHAPE_PILL = 3
SHAPE_SPRITE = 4
SHAPE_CONE = 5

DIR_HOME = os.path.dirname(poser.AppLocation())
DIR_PRT3 = os.path.dirname(poser.ScriptLocation())
DIR_DATA = os.path.join(DIR_PRT3, "data")
DIR_RESOURCES = os.path.join(DIR_PRT3, "resources")
DIR_WACROS = os.path.join(DIR_PRT3, "wacros")
DIR_TEMP = poser.TempLocation()
DIR_PRESETS = os.path.join(DIR_HOME, "Runtime", "libraries", "Pose", "Particles 3 Presets")

def FindOurFolder():
	libs = poser.Libraries()
	target = u'Runtime/Geometries/Particles3+'
	found = False
	for lib in libs:
		lib = lib.replace('\\', '/')
		path = '/'.join((lib, target))
		if os.path.exists(path):
			found = True
			break								
	return (None, path)[found]

x = FindOurFolder()
if x:
	DIR_RESOURCES = u':Runtime:Geometries:Particles3+'
	GEOM_EMITTER = DIR_RESOURCES + ":emitter.obj"
	GEOM_ATTRACTOR = DIR_RESOURCES + ":attractor.obj"
	GEOM_DEFLECTOR = DIR_RESOURCES + ":deflector.obj"
	GEOM_BALL = DIR_RESOURCES + ":ball.obj"
	GEOM_TORUS = DIR_RESOURCES + ":torus.obj"
	GEOM_PILL = DIR_RESOURCES + ":pill.obj"
	GEOM_SPRITE = DIR_RESOURCES + ":sprite.obj"
	GEOM_CONE = DIR_RESOURCES + ":cone.obj"
else:
	GEOM_EMITTER = os.path.join(DIR_RESOURCES, "emitter.obj")
	GEOM_ATTRACTOR = os.path.join(DIR_RESOURCES, "attractor.obj")
	GEOM_DEFLECTOR = os.path.join(DIR_RESOURCES, "deflector.obj")
	GEOM_BALL = os.path.join(DIR_RESOURCES, "ball.obj")
	GEOM_TORUS = os.path.join(DIR_RESOURCES, "torus.obj")
	GEOM_PILL = os.path.join(DIR_RESOURCES, "pill.obj")
	GEOM_SPRITE = os.path.join(DIR_RESOURCES, "sprite.obj")
	GEOM_CONE = os.path.join(DIR_RESOURCES, "cone.obj")

# prt3Math
def ifelse(condition, true_value, false_value):
		if condition:
				return true_value
		else:
				return false_value
def degreesToRadians(degrees):
		return degrees*math.pi/180
def radiansToDegrees(radians):
		return radians * 180 / math.pi
class PwVector(object):
		__slots__ = ("x", "y", "z")
		def __init__(self, *args):
				if len(args) > 1:
						self.x, self.y, self.z = args[0], args[1], args[2]
				else:
						matrix = args[0]
						self.x, self.y, self.z = matrix.get(0,0), matrix.get(1,0), matrix.get(2,0)
		def __add__(self, other):
				if isinstance(other, PwVector):
						return PwVector(self.x + other.x, self.y + other.y, self.z + other.z)
				else:
						return PwVector(self.x + other, self.y + other, self.z + other)
		def __sub__(self, other):
				return self + -other
		def __mul__(self, other):
				return PwVector(self.x * other, self.y * other, self.z * other)
		def __rmul__(self, other):
				return self * other
		def __div__(self, other):
				return PwVector(self.x / other, self.y / other, self.z / other)
		def __rdiv__(self, other):
				return self / other
		def __neg__(self):
				return PwVector(-self.x, -self.y, -self.y)
		def __str__(self):
				return "Vector " + str( (self.x, self.y, self.z) )
		def __len__(self):
				return math.sqrt(self.x*self.x + self.y*self.y + self.z*self.z)
		def length(self):
				return math.sqrt(self.x*self.x + self.y*self.y + self.z*self.z)
		def normalize(self):
				lensq = self.x*self.x + self.y*self.y + self.z*self.z
				if( lensq == 0 ):
						return self
				else:
						recip = 1/self.length()
						return PwVector(self.x*recip, self.y*recip, self.z*recip)
		def dot(self, other):
				return self.x*other.x + self.y*other.y + self.z*other.z
		def cross(self, other):
				return PwVector(self.y*other.z - other.y*self.z, self.z*other.x - other.z*self.x, self.x*other.y - other.x*self.y)
		def matrix(self):
				return PwMatrix( [self.x], [self.y], [self.z], [0] )		
class PwPoint(object):
		__slots__ = ("x", "y", "z")
		def __init__(self, *args):
				if len(args) > 1:
						self.x, self.y, self.z = args[0], args[1], args[2]
				elif isinstance(args[0], PwMatrix):
						matrix = args[0]
						self.x, self.y, self.z = matrix.get(0,0), matrix.get(1,0), matrix.get(2,0)
				else:
						self.x, self.y, self.z = args[0]
		def __mul__(self, other):
				return PwPoint(self.x * other, self.y * other, self.z * other)
		def __rmul__(self, other):
				return self * other
		def __add__(self, other):
				return PwPoint(self.x+other.x, self.y+other.y, self.z+other.z)
		def __sub__(self, other):
				return PwVector(self.x-other.x, self.y-other.y, self.z-other.z)
		def __str__(self):
				return "Point " + str( (self.x, self.y, self.z) )
		def dist(self, other):
				x = self.x - other.x
				y = self.y - other.y
				z = self.z - other.z
				return math.sqrt(x*x + y*y + z*z)
		def matrix(self):
				return PwMatrix( [self.x], [self.y], [self.z], [1] )
		def blend(self, other, v):
				w = 1.0 - v
				return v * self + w * other
def PwPlane(object):
		def __init__(self, p0, p1, p2):
				self.origin = P
				radius = size/2
				p0 = P + PwVector(radius, radius, 0)
				p1 = P + PwVector(-radius, radius, 0)
				p2 = P + PwVector(-radius, -radius, 0)
				p3 = P + PwVector(radius, -radius, 0)
				self.A = p0.y*(p1.z-p2.z) + p1.y*(p2.z-p0.z) + p2.y*(p0.z-p1.z)
				self.B = p0.z*(p1.x-p2.x) + p1.z*(p2.x-p0.x) + p2.z*(p0.x-p1.x)
				self.C = p0.x*(p1.y-p2.y) + p0.x*(p2.y-p0.y) + p2.x*(p0.y-p1.y)
				self.D = -p0.x*(p1.y*p2.z - p2.y*p1.z)  +\
						 -p1.x*(p2.y*p0.z - p0.y*p2.z) +\
						 -p2.x*(p0.y*p1.z - p1.y*p0.z)
				self.N = PwVector(A,B,C)
class PwMatrix(object):
		__slots__ = ("matrix")
		def __init__(self, *args):
				self.matrix = []
				for row in args:
						self.append( row )
		def __mul__(self, other):
				if isinstance(other, PwMatrix):
						matrix = PwMatrix()
						rows = len(self.matrix)
						cols = len(other.matrix[0])
						for i in range(rows):
								row = []
								for j in range(cols):
										col = 0
										for k in range( len(self.matrix[i]) ):
												col = col + self.matrix[i][k] * other.matrix[k][j]
										row.append(col)
								matrix.append(row)
						return matrix
				elif isinstance(other, PwPoint):
						return PwPoint( self * other.matrix() )
				elif isinstance(other, PwVector):
						return PwVector( self * other.matrix() )
				else:
						matrix = PwMatrix()
						for row in self.matrix:
								newrow = []
								for col in row:
										newrow.append(col * other)
								matrix.append(newrow)
						return matrix
		def __rmul__(self, other):
				if isinstance(other, float) or isinstance(other, int):
						return self * other
		def __add__(self, other):
				matrix = PwMatrix()
				for i in range(len(self.matrix)):
						row = []
						for j in range(len(self.matrix[0])):
								row.append(self.matrix[i][j] + other.matrix[i][j])
						matrix.append(row)
				return matrix
		def __sub__(self, other):
				return self + -other
		def __neg__(self):
				matrix = PwMatrix()
				for row in self.matrix:
						newrow = []
						for col in row:
								newrow.append(-col)
						matrix.append(newrow)
				return matrix
		def __str__(self):
				out = "%i x %i Matrix" % (len(self.matrix), len(self.matrix[0]))
				for row in self.matrix:
						out += "\n" + str(row)
				return out
		def append(self, row):
				self.matrix.append(row)
		def get(self, row, col):
				return self.matrix[row][col]
		def define(self, row, col, v):
				self.matrix[row][col] = v
		def columns(self):
				return len(self.matrix[0])
		def rows(self):
				return len(self.matrix)
		def det(self):
				rows = len(self.matrix)
				if rows == 1:
						return self.get(0,0)
				firstrow = self.matrix[0]
				cols = len(firstrow)
				tally = 0
				for i in range(cols):
						term = firstrow[i]
						submatrix = PwMatrix()
						for row in range(rows):
								if row == 0:
										continue
								subrow = []
								for col in range(cols):
										if col == i:
												continue
										else:
												subrow.append( self.get(row,col) )
								submatrix.append(subrow)
						result = term * submatrix.det()
						if i == 0:
								tally = tally + result
						else:
								tally = tally - result
				return tally
		def det2(self):
				return self.get(0,0)*self.get(1,1) - self.get(0,1)*self.get(1,0)
		def det3(self):
				a, b, c = self.matrix[0]
				d, e, f = self.matrix[1]
				g, h, i = self.matrix[2]
				return a*(e*i-f*h) - b*(d*i-f*g) - c*(d*h-e*g)
		def inverse(self):
				rows = len(self.matrix)
				cols = len(self.matrix[0])
				inv = PwMatrix()
				for r in range(rows):
						newrow = []
						for c in range(cols):
								submatrix = PwMatrix()
								for i in range(rows):
										subrow = []
										if i == r:
												continue
										for j in range(cols):
												if j == c:
														continue
												subrow.append( self.get(i,j) )
										submatrix.append(subrow)
								newrow.append( submatrix.det() )
						inv.append(newrow)
				return inv
		def inverse2(self):
				a, b = self.matrix[0]
				c, d = self.matrix[1]
				return 1.0/(a*d-b*c) * PwMatrix([ d, -b],
												[-c,  a])
		def inverse3(self):
				a,b,c = self.matrix[0]
				d,e,f = self.matrix[1]
				g,h,i = self.matrix[2]
				anti = PwMatrix([ e*i-f*h,      c*h-b*i,        b*f-c*e ],
								[ f*g-d*i,      a*i-c*g,        c*d-a*f ],
								[ d*h-e*g,      b*g-a*h,        a*e-b*d ])
				return 1.0/self.det3() * anti
		def transpose(self):
				matrix = PwMatrix()
				for r in range(self.rows()):
						row = []
						for c in range(self.columns()):
								row.append( self.get(c,r) )
						matrix.append(row)
				return matrix
def translation(tx, ty, tz):
		return PwMatrix([1, 0, 0, tx],
						[0, 1, 0, ty],
						[0, 0, 1, tz],
						[0, 0, 0, 1 ] )
def rotation(axis, degrees):
		if degrees == 0:
				return IDENTITY4x4
		theta = degrees * math.pi / 180
		c = math.cos(theta)
		s = math.sin(theta)
		t = 1 - c
		x, y, z = axis.x, axis.y, axis.z
		return PwMatrix([t*x*x+c,       t*x*y-s*z,      t*x*z+s*y,      0],
						[t*x*y+s*z,     t*y*y+c,        t*y*z-s*x,      0],
						[t*x*z-s*y,     t*y*z+s*x,      t*z*z+c,        0],
						[0,             0,              0,              1])
def scale(x, y, z):
		return PwMatrix([x,0,0,0],
						[0,y,0,0],
						[0,0,z,0],
						[0,0,0,1])
IDENTITY4x4 = PwMatrix( [1,0,0,0],
						[0,1,0,0],
						[0,0,1,0],
						[0,0,0,1] )
IDENTITY3X3 = PwMatrix([1,0,0],
					   [0,1,0],
					   [0,0,1])
IDENTITY2X2 = PwMatrix([1,0],
					   [0,1])
ORIGIN = PwPoint(0,0,0)
XAXIS = PwVector(1,0,0)
YAXIS = PwVector(0,1,0)
ZAXIS = PwVector(0,0,1)
class PwQuaternion(object):
	def __init__(self, w, x=None, y=None, z=None):
		if w == XAXIS:
			self.w, self.x, self.y, self.z = math.cos(x)/2, math.sin(x)/2, 0, 0
		elif w == YAXIS:
			self.w, self.x, self.y, self.z = math.cos(x)/2, 0, math.sin(x)/2, 0
		elif w == ZAXIS:
			self.w, self.x, self.y, self.z = math.cos(x)/2, 0, 0, math.sin(x)/2
		elif isinstance(w, PwVector):
			self.w, self.x, self.y, self.z = 0, w.x, w.y, w.z
		elif isinstance(x, PwVector):
			self.w, self.x, self.y, self.z = w, x.x, x.y, x.z
		elif isinstance(w, tuple):
			self.w, self.x, self.y, self.z = w
		elif isinstance(x, tuple):
			self.w = w
			self.x, self.y, self.z = x
		else:
			self.w, self.x, self.y, self.z = w, x, y, z
	def __add__(self, other):
		return PwQuaternion(self.w + other.w, self.x + other.x, self.y + other.y, self.z + other.z)
	def __mul__(self, other):
		if isinstance(other, PwQuaternion):
			v1 = other.vector()
			v2 = self.vector()
			return PwQuaternion(self.w*other.w - v1.dot(v2),
								other.w*v2 + self.w*v1 + v2.cross(v1))
		elif isinstance(other, PwVector):
				pMult = 1.0 - self.w*self.w
				vMult = 2.0 * (self.x*other.x + self.y*other.y + self.z*other.z)
				crossMult = 2.0 * self.w
				return PwVector(pMult*other.x + vMult*self.x + crossMult*(self.y*other.z - self.z*other.y),
								pMult*other.y + vMult*self.y + crossMult*(self.z*other.x - self.x*other.z),
								pMult*other.z + vMult*self.z + crossMult*(self.x*other.y - self.y*other.x))
		elif isinstance(other, PwPoint):
				pMult = 1.0 - self.w*self.w
				vMult = 2.0 * (self.x*other.x + self.y*other.y + self.z*other.z)
				crossMult = 2.0 * self.w
				return PwPoint(pMult*other.x + vMult*self.x + crossMult*(self.y*other.z - self.z*other.y),
								pMult*other.y + vMult*self.y + crossMult*(self.z*other.x - self.x*other.z),
								pMult*other.z + vMult*self.z + crossMult*(self.x*other.y - self.y*other.x))
		else:
			return PwQuaternion(self.w * other, self.x * other, self.y * other, self.z * other)
	def __rmul__(self, other):
		if isinstance(other, PwQuaternion):
			pass
		else:
			return self * other
	def __div__(self, other):
		return PwQuaternion(self.w/other, self.x/other, self.y/other, self.z/other)
	def __neg__(self):
		return PwQuaternion(-self.w, -self.x, -self.y, -self.z)
	def length(self):
		return math.sqrt(self.w*self.w + self.x*self.x + self.y*self.y + self.z*self.z)
	def __str__(self):
		return repr((self.w, self.x, self.y, self.z))
	def normalize(self):
		self = self / self.length()
		return self
	def dot(self, other):
		return self.w*other.w + self.x*other.x + self.y*other.y + self.z*other.z
	def concat(self, other):
		v1 = other.vector()
		v2 = self.vector()
		return PwQuaternion(other.w*self.w - v1.dot(v2),
							other.w*v2 + self.w*v1 + v2.cross(v1) ).normalize()
	def reciprocal(self):
		return 1.0/self.length()**2 * PwQuaternion(self.w, -self.vector())
	def vector(self):
		return PwVector(self.x, self.y, self.z)
	def rotate(self, other):
		pmult = 1.0 - self.w*self.w
		vmult = 2.0 * (self.x*other.x + self.y*other.y + self.z*other.z)
		crossmult = 2.0 * self.w
		return PwVector( pmult*other.x + vmult*self.x + crossmult*(self.y*other.z - self.z*other.y),
						 pmult*other.y + vmult*self.y + crossmult*(self.z*other.x - self.x*other.z),
						 pmult*other.z + vmult*self.z + crossmult*(self.x*other.y - self.y*other.x) )
	def euler(self):
		return PwVector( radiansToDegrees(math.atan( 2*(self.w*self.x + self.y*self.z) / (1 - 2*(self.x*self.x + self.y*self.y)) )),
						 radiansToDegrees(math.asin( 2*(self.w*self.y + self.z*self.x) )),
						 radiansToDegrees(math.atan( 2*(self.w*self.z + self.x*self.y) / (1 - 2*(self.y*self.y + self.z*self.z)) )) )       
	def matrix(self):
		self.normalize()
		x2, y2, z2 = self.x + self.x, self.y + self.y, self.z + self.z
		xx, xy, xz = self.x * x2, self.x * y2, self.x * z2
		yy, yz, zz = self.y * y2, self.y * z2, self.z * z2
		wx, wy, wz = self.w * x2, self.w * y2, self.w * z2
		return PwMatrix([1.0 - yy - zz, xy - wz,        xz + wy,        0],
						[xy + wz,       1.0 - xx - zz,  yz - wx,        0],
						[xz - wy,       yz + wx,        1.0 - xx - yy,  0],
						[0,             0,              0,              1])												
class PwSpace(object):
		__slots__ = ("rotation", "translation", "transformation")
		def __init__(self, xtrans, ytrans, ztrans, xrot, yrot, zrot):
				self.translation = translation(xtrans, ytrans, ztrans)
				self.rotation = rotation(XAXIS, xrot) * rotation(YAXIS, yrot) * rotation(ZAXIS, zrot)
				self.transformation = self.translation * self.rotation
		def translate(self, point):
				return self.translation * point
		def rotate(self, point):
				return self.rotation * point
		def transform(self, point):
				return self.transformation * point

# prt3Core
def sanitizeName(name):
		parts = name.split("_")
		if len(parts) > 0:
				name = parts[0]
				suffix = parts[-1].split(":")[0]
		allowed = string.ascii_letters + "-"
		cleaned = ""
		for c in name:
				if allowed.find(c) != -1:
						cleaned = cleaned + c
		return cleaned + "_" + suffix
def createSystem(name, groups, emitter=None):
		system = PwSystem(name)
		for gname, gcount, gshape in groups:
				system.addParticleGroup(gname, gcount, gshape)
		cr2, pp2 = system.write(DIR_TEMP)
		poser.Scene().LoadLibraryFigure(cr2)
		if not emitter:
				poser.Scene().LoadLibraryFigure(pp2)
		else:
				body = poser.Scene().CurrentFigure().ParentActor()
				body.Parameter(PARM_ID).SetValue(emitter)
		system.assignTo(poser.Scene().CurrentFigure(), conformAssigned=True)
		return system
def buildSystemFrom(fig):
		system = PwSystem(fig.Name())
		group = fig.ParentActor()
		particles = group.Children()
		for p in particles:
				if p.IsBodyPart():
						continue
				else:
						particles.remove(p)
		system.addParticleGroup(group.Name(), len(particles), group.Parameter(PARM_SHAPE).Value())
		try:
			system.assignTo(fig)
			return system
		except:
			return None
class PwMover(object):
		def __init__(self, move=PwVector(0,0,0), accel=PwVector(0,0,0), var=0):
				self.movement = move
				self.acceleration = accel
				self.variance = var
		def __mul__(self, other):
				self.movement = self.movement * other
				self.acceleration = self.acceleration * other
				return self
		def setMovement(self, vector):
				self.movement = vector
		def setAcceleration(self, vector):
				self.acceleration = vector
		def setVariance(self, v):
				self.variance = v
				var = (random.random() - 0.5) * (v / 2)
				self.movement.x = self.movement.x + var
				self.movement.y = self.movement.y + var
				self.movement.z = self.movement.z + var
		def define(self, move, accel, var):
				self.setMovement(move)
				self.setAcceleration(accel)
				self.setVariance(var)
		def get(self, life):
				x = life * self.movement.x * (1.0 + self.acceleration.x) ** life
				y = life * self.movement.y * (1.0 + self.acceleration.y) ** life
				z = life * self.movement.z * (1.0 + self.acceleration.z) ** life
				return PwVector(x, y, z)
		def getUnmultiplied(self, life):
				x = self.movement.x * (1.0 + self.acceleration.x) ** life
				y = self.movement.y * (1.0 + self.acceleration.y) ** life
				z = self.movement.z * (1.0 + self.acceleration.z) ** life
				return PwVector(x, y, z)
class PwPoserObject(object):
		def __init__(self):
				pass
				self.source = None
		def assignTo(self, obj):
				self.source = obj
		def getObject(self):
				return self.source
		def Name(self):
				return self.source.Name()
class PwPoserParameters(object):
		def __init__(self, owner, actor):
				for iname in owner.channels:
						channel = owner.getChannel(iname)
						self.__setattr__(channel.iname, actor.Parameter(channel.name))
				self.xtran = actor.ParameterByCode(poser.kParmCodeXTRAN)
				self.ytran = actor.ParameterByCode(poser.kParmCodeYTRAN)
				self.ztran = actor.ParameterByCode(poser.kParmCodeZTRAN)
				self.xrot = actor.ParameterByCode(poser.kParmCodeXROT)
				self.yrot = actor.ParameterByCode(poser.kParmCodeYROT)
				self.zrot = actor.ParameterByCode(poser.kParmCodeZROT)
				self.scale = actor.ParameterByCode(poser.kParmCodeASCALE)
				self.xscale = actor.ParameterByCode(poser.kParmCodeXSCALE)
				self.yscale = actor.ParameterByCode(poser.kParmCodeYSCALE)
				self.zscale = actor.ParameterByCode(poser.kParmCodeZSCALE)
		def setTran(self, x, y, z):
				for parm, v in [("xtran", x), ("ytran", y), ("ztran", z)]:
						self.key(parm, v)
		def getTran(self):
				return PwPoint( self.get("xtran"), self.get("ytran"), self.get("ztran") )
		def setRot(self, x, y, z):
				for parm, v in [("xrot", x), ("yrot", y), ("zrot", z)]:
						self.key(parm, v)
		def getRot(self):
				return PwVector( self.get("xrot"), self.get("yrot"), self.get("zrot") )
		def setScale(self, a, x, y, z):
				for parm, v in [("scale", a), ("xscale", x), ("yscale", y), ("zscale", z)]:
						self.key(parm, v)
		def getScale(self):
				return (self.get("scale"), self.get("xscale"), self.get("yscale"), self.get("zscale"))
		def setRateOfEmission(self, v):
				self.key(IPARM_EMISSION, v)
		def getRateOfEmission(self):
				return self.get(IPARM_EMISSION)
		def setEmissionAngle(self, v):
				self.key(IPARM_EMISSIONANGLE, v)
		def getEmissionAngle(self):
				return self.get(IPARM_EMISSIONANGLE)
		def setRecycle(self, v):
				self.key(IPARM_RECYCLE, ifelse(v, 1.0, 0))
		def getRecycle(self):
				return ifelse(self.get(IPARM_RECYCLE), 1.0, 0)
		def setLifespan(self, v):
				self.key(IPARM_LIFESPAN, v)
		def getLifespan(self):
				return self.get(IPARM_LIFESPAN)
		def setStart(self, v):
				self.key(IPARM_START, v)
		def getStart(self):
				return self.get(IPARM_START)
		def setEnd(self, v):
				self.key(IPARM_END, v)
		def getEnd(self):
				return self.get(IPARM_END)
		def setGravity(self, v):
				self.key(IPARM_GRAVITY, v * 100)
		def getGravity(self):
				return self.get(IPARM_GRAVITY)/100
		def setAttraction(self, v):
				self.key(IPARM_ATTRACTION, v)
		def getAttraction(self):
				return self.get(IPARM_ATTRACTION)
		def setDampening(self, v):
				self.key(IPARM_DAMPENING, v)
		def getDampening(self):
				return self.get(IPARM_DAMPENING)
		def setVelocity(self, x, y, z):
				for parm, v in [(IPARM_VELOCITYX, x), (IPARM_VELOCITYY, y), (IPARM_VELOCITYZ, z)]:
						self.key(parm, v)
		def getVelocity(self):
				return PwVector(self.get(IPARM_VELOCITYX), self.get(IPARM_VELOCITYY), self.get(IPARM_VELOCITYZ))
		def setAcceleration(self, x, y, z):
				for parm, v in [(IPARM_ACCELERATIONX, x), (IPARM_ACCELERATIONY, y), (IPARM_ACCELERATIONZ, z)]:
						self.key(parm, v)
		def getAcceleration(self):
				return PwVector(self.get(IPARM_ACCELERATIONX),self.get(IPARM_ACCELERATIONY), self.get(IPARM_ACCELERATIONZ))
		def setVelocityVariance(self, v):
				self.key(IPARM_VELOCITYVAR, v)
		def getVelocityVariance(self):
				return self.get(IPARM_VELOCITYVAR)
		def setOrbit(self, x, y, z):
				for parm, v in [(IPARM_ORBITX, x), (IPARM_ORBITY, y), (IPARM_ORBITZ, z)]:
						self.key(parm, v)
		def getOrbit(self):
				return PwVector(self.get(IPARM_ORBITX), self.get(IPARM_ORBITY), self.get(IPARM_ORBITZ))
		def setOrbitAcceleration(self, x, y, z):
				for parm, v in [(IPARM_ORBITAX, x), (IPARM_ORBITAY, y), (IPARM_ORBITAZ, z)]:
						self.key(parm, v)
		def getOrbitAcceleration(self):
				return PwVector(self.get(IPARM_ORBITAX), self.get(IPARM_ORBITAY), self.get(IPARM_ORBITAZ))
		def setOrbitVariance(self, v):
				self.key(IPARM_ORBITVAR, v)
		def getOrbitVariance(self):
				return self.get(IPARM_ORBITVAR)
		def setSpin(self, x, y, z):
				for parm, v in [(IPARM_SPINX, x), (IPARM_SPINY, y), (IPARM_SPINZ, z)]:
						self.key(parm, v)
		def getSpin(self):
				return PwVector(self.get(IPARM_SPINX), self.get(IPARM_SPINY), self.get(IPARM_SPINZ))
		def setSpinAcceleration(self, x, y, z):
				for parm, v in [(IPARM_SPINAX, x), (IPARM_SPINAY, y), (IPARM_SPINAZ, z)]:
						self.key(parm, v)
		def getSpinAcceleration(self):
				return PwVector(self.get(IPARM_SPINAX), self.get(IPARM_SPINAY), self.get(IPARM_SPINAZ))
		def setSpinVariance(self, v):
				self.key(IPARM_SPINVAR, v)
		def getSpinVariance(self):
				return self.get(IPARM_SPINVAR)
		def setGrow(self, x, y, z):
				for parm, v in [(IPARM_GROWX, x), (IPARM_GROWY, y), (IPARM_GROWZ, z)]:
						self.key(parm, v*100)
		def getGrow(self):
				return PwVector(self.get(IPARM_GROWX)/100, self.get(IPARM_GROWY)/100, self.get(IPARM_GROWZ)/100)
		def setGrowAcceleration(self, x, y, z):
				for parm, v in [(IPARM_GROWAX, x), (IPARM_GROWAY, y), (IPARM_GROWAZ, z)]:
						self.key(parm, v*100)
		def getGrowAcceleration(self):
				return PwVector(self.get(IPARM_GROWAX)/100, self.get(IPARM_GROWAY)/100, self.get(IPARM_GROWAZ)/100)
		def setGrowVariance(self, v):
				self.key(IPARM_GROWVAR, v)
		def getGrowVariance(self):
				return self.get(IPARM_GROWVAR)/100
		def setLife(self, v):
				self.key(IPARM_LIFE, v)
		def getLife(self):
				return self.get(IPARM_LIFE)
		def setSpawnPoint(self, x, y, z):
				for parm, v in [(IPARM_SPAWNPNTX, x), (IPARM_SPAWNPNTY, y), (IPARM_SPAWNPNTZ, z)]:
						self.key(parm, v)
		def getSpawnPoint(self):
				return PwPoint( self.get(IPARM_SPAWNPNTX), self.get(IPARM_SPAWNPNTY), self.get(IPARM_SPAWNPNTZ) )
		def setSpawnVector(self, x, y, z):
				for parm, v in [(IPARM_SPAWNVECX, x), (IPARM_SPAWNVECY, y), (IPARM_SPAWNVECZ, z)]:
						self.key(parm, v)
		def getSpawnVector(self):
				return PwVector( self.get(IPARM_SPAWNVECX), self.get(IPARM_SPAWNVECY), self.get(IPARM_SPAWNVECZ) )
		def setScaleMod(self, x, y, z):
				for parm, v in [(IPARM_XSCALEMOD, x), (IPARM_YSCALEMOD, y), (IPARM_ZSCALEMOD, z)]:
						self.key(parm, v)
		def getScaleMod(self):
				return PwVector( self.get(IPARM_XSCALEMOD), self.get(IPARM_YSCALEMOD), self.get(IPARM_ZSCALEMOD) )
		def setType(self, v):
				self.key(IPARM_TYPE, v)
		def getType(self):
				return self.get(IPARM_TYPE)
		def setShape(self, v):
				self.key(IPARM_SHAPE, v)
		def getShape(self):
				return self.get(IPARM_SHAPE)
		def setCount(self, v):
				self.key(IPARM_COUNT, v)
		def getCount(self):
				return self.get(IPARM_COUNT)
		def getParticleOffset(self):
				return PwPoint( self.get(IPARM_PXTRAN), self.get(IPARM_PYTRAN), self.get(IPARM_PZTRAN) )
		def getParticleScale(self):
				return self.get(IPARM_PSCALE)/100, self.get(IPARM_PXSCALE)/100, self.get(IPARM_PYSCALE)/100, self.get(IPARM_PXSCALE)/100
		def get(self, parm):
				return self.__getattribute__(parm).Value()
		def key(self, parm, v):
				self.__getattribute__(parm).SetValue(v)
				self.__getattribute__(parm).AddKeyFrame()
class PwPoserActor(PwPoserObject):
		def __init__(self):
				PwPoserObject.__init__(self)
				self.channels = []
		def assignTo(self, obj, conformAssigned=False):
				PwPoserObject.assignTo(self, obj)
				self.parameters = PwPoserParameters(self, obj)
				if conformAssigned:
						self.conform(self.source)
		def setType(self, kind):
				self.setChannel(IPARM_TYPE, kind)
		def addChannel( self, iname, name, value, hidden=False, kind="valueParm" ):
				self.__setattr__( iname, PwParameter( kind, iname, name, value, hidden ) )
				self.channels.append( iname )
		def getChannel( self, iname ):
				return self.__getattribute__( iname )
		def setChannel( self, iname, v ):
				self.__getattribute__(iname).value = v
		def getObjectSpace(self):
				tup = self.source.WorldMatrix()
				matrix = PwMatrix()
				for row in tup:
						newrow = []
						for col in row:
								newrow.append(col)
						matrix.append(newrow)
				return matrix.transpose()
		def getObjectRotation(self):
				return PwQuaternion(self.source.WorldQuaternion()).euler()
class PwERC(object):
		def __init__(self, figure, group, channel, ratio):
				self.figure = figure
				self.group = group
				self.channel = channel
				self.ratio = ratio
		def write(self):
				return "\n\t\t\tvalueOpDeltaAdd" +\
					   "\n\t\t\t\t%s" % self.figure +\
					   "\n\t\t\t\t%s" % self.group +\
					   "\n\t\t\t\t%s" % self.channel +\
					   "\n\t\t\tdeltaAddDelta %f" % self.ratio
class PwParameter(PwPoserObject):
		def __init__(self, kind, iname, name, value, hidden):
				self.kind = kind
				self.iname = iname
				self.name = name
				self.value = value
				self.hidden = hidden
				self.links = []
		def addLink(self, figure, group, channel, ratio):
				self.links.append( PwERC(figure, group, channel, ratio) )
		def write(self):
				links = ""
				for link in self.links:
						links = links + link.write()
				if self.iname in ('xtran', 'ytran', 'ztran'):
					return "\n\t\t%s %s" % (self.kind, self.iname) +\
					   "\n\t\t\t{" +\
					   "\n\t\t\tname %s" % self.name +\
					   "\n\t\t\tinitValue %f" % self.value +\
					   "\n\t\t\thidden %i" % self.hidden +\
					   "\n\t\t\tmin -1000" +\
					   "\n\t\t\tmax 1000" +\
					   "\n\t\t\ttrackingScale 0.001" +\
					   "\n\t\t\tkeys" +\
					   "\n\t\t\t\t{" +\
					   "\n\t\t\t\tstatic 0" +\
					   "\n\t\t\t\tk 0 %f" % self.value +\
					   "\n\t\t\ttrackingScaleMult 1" +\
					   "\n\t\t\t\t}" +\
					   links +\
					   "\n\t\t\t}"
				else:
					return "\n\t\t%s %s" % (self.kind, self.iname) +\
					   "\n\t\t\t{" +\
					   "\n\t\t\tname %s" % self.name +\
					   "\n\t\t\tinitValue %f" % self.value +\
					   "\n\t\t\thidden %i" % self.hidden +\
					   "\n\t\t\tmin -1000" +\
					   "\n\t\t\tmax 1000" +\
					   "\n\t\t\ttrackingScale 1.000" +\
					   "\n\t\t\tkeys" +\
					   "\n\t\t\t\t{" +\
					   "\n\t\t\t\tstatic 0" +\
					   "\n\t\t\t\tk 0 %f" % self.value +\
					   "\n\t\t\t\t}" +\
					   links +\
					   "\n\t\t\t}"
class PwMaterial(PwPoserObject):
		def __init__(self, sName, tKd, tKa):
				self.name = sName
				self.kd = tKd
				self.ka = tKa
		def write(self):
				return "\n\tmaterial %s" % self.name +\
					   "\n\t\t{" +\
					   "\n\t\tKdColor %f %f %f %f" % self.kd +\
					   "\n\t\tKaColor %f %f %f %f" % self.ka +\
					   "\n\t\tKsColor 0 0 0 1" +\
					   "\n\t\t}"
class PwElement(PwPoserActor):
		def __init__(self, oParent, sName):
				PwPoserActor.__init__(self)
				self.prop = False
				self.name = sName
				self.iname = sanitizeName(sName)
				self.geom = ""
				self.on = "on"
				self.bend = 0
				self.dynamicsLock = 1
				self.hidden = 0
				self.addToMenu = 1
				self.castsShadow = 0
				self.includeInDepthCue = 1
				self.parent = oParent
				self.groups = []
				self.addGroup( "Transform", ["scale", "xScale", "yScale", "zScale", "rotateX", "rotateY", "rotateZ", "translateX", "translateY", "translateZ"] )
				self.addChannel( IPARM_TYPE, PARM_TYPE, 0.0, hidden=True )
				self.addChannel( "originX", "GetStringRes(1028,44)", 0.0, kind="xOffsetA", hidden=True )
				self.addChannel( "originY", "GetStringRes(1028,45)", 0.0, kind="yOffsetA", hidden=True )
				self.addChannel( "originZ", "GetStringRes(1028,46)", 0.0, kind="zOffsetA", hidden=True )
				self.addChannel( "xOffB", "xOffB", 0.0, kind="xOffsetB", hidden=True )
				self.addChannel( "yOffB", "yOffB", 0.0, kind="yOffsetB", hidden=True )
				self.addChannel( "zOffB", "zOffB", 0.001, kind="zOffsetB", hidden=True )
				self.origin = (0.0, 0.0, 0)
				self.endPoint = (0.0, 0.0, 1)
				self.orientation = (0.0, 0.0, 0.0)
				self.displayOrigin = 0
				self.displayMode = "USEPARENT"
				self.customMaterial = 0
				self.materials = []
				self.addMaterial("Preview", (1,1,1,1), (0,0,0,1))
				self.locked = 0
				self.visibleInRender = 1
		def addTranslations( self, hidden=False ):
				self.addChannel( "xtran", "GetStringRes(1028,12)", 0.0, kind="translateX", hidden=hidden )
				self.addChannel( "ytran", "GetStringRes(1028,13)", 0.0, kind="translateY", hidden=hidden )
				self.addChannel( "ztran", "GetStringRes(1028,14)", 0.0, kind="translateZ", hidden=hidden )
		def addRotations( self, hidden=False ):
				self.addChannel( "xRotate", "xrot", 0.0, kind="rotateX", hidden=hidden )
				self.addChannel( "yRotate", "yrot", 0.0, kind="rotateY", hidden=hidden )
				self.addChannel( "zRotate", "zrot", 0.0, kind="rotateZ", hidden=hidden )
		def addScales( self, hidden=False ):
				self.addChannel( "scale", "GetStringRes(1028,5)", 1.0, kind="propagatingScale", hidden=hidden )
				self.addChannel( "xScale", "GetStringRes(1028,6)", 1.0, kind="propagatingScaleX", hidden=hidden )
				self.addChannel( "yScale", "GetStringRes(1028,7)", 1.0, kind="propagatingScaleY", hidden=hidden )
				self.addChannel( "zScale", "GetStringRes(1028,8)", 1.0, kind="propagatingScaleZ", hidden=hidden )
		def addControlToScales(self, figure, group, aScaleCtrl, xScaleCtrl, yScaleCtrl, zScaleCtrl, ratio):
				"""
				for parm, ctrl in [(self.xScale, xScaleCtrl),
								   (self.yScale, yScaleCtrl),
								   (self.zScale, zScaleCtrl)]:
						parm.addLink( figure, group, ctrl, ratio )
				"""
				for parm in [self.xScale, self.yScale, self.zScale]:
						parm.addLink( figure, group, aScaleCtrl, ratio )
		def addModifierToScales(self, figure, group, xScaleMod, yScaleMod, zScaleMod, ratio):
				for parm, ctrl in [(self.xScale, xScaleMod), (self.yScale, yScaleMod), (self.zScale, zScaleMod)]:
						parm.addLink( figure, group, ctrl, ratio )
		def addControlToRots(self, figure, group, xRotCtrl, yRotCtrl, zRotCtrl, ratio):
				for parm, ctrl in [(self.xRotate, xRotCtrl),
								   (self.yRotate, yRotCtrl),
								   (self.zRotate, zRotCtrl)]:
						parm.addLink( figure, group, ctrl, ratio )
		def addControlToTrans(self, figure, group, xTranCtrl, yTranCtrl, zTranCtrl, ratio):
				for parm, ctrl in [(self.xtran, xTranCtrl),
								   (self.ytran, yTranCtrl),
								   (self.ztran, zTranCtrl)]:
						parm.addLink( figure, group, ctrl, ratio )
		def addGroup( self, name, channels ):
				self.groups.append( [name, channels] )
		def setGeometry( self, geompath ):
				self.geom = geompath
		def addMaterial(self, material, kd, ka):
				self.materials.append( PwMaterial(material, kd, ka) )
		def writeGeometry( self ):
				if self.geom:
						geomline = "\tstorageOffset 0 0 0\n\tobjFileGeom 0 0 %s" % self.geom
				else:
						geomline = ""
				return "\n%s %s%s\n\t{\n%s\n\t}" % (ifelse(self.prop, "prop", "actor"), self.iname, ifelse(self.prop, "", ":1"), geomline)
		def writeActor( self ):
				channels = "\t\tgroups\n\t\t\t{"
				for group, parms in self.groups:
						channels = channels + "\n\t\t\tgroupNode %s\n\t\t\t\t{" % group
						for parm in parms:
								channels = channels + "\n\t\t\t\tparmNode %s" % parm
						channels = channels + "\n\t\t\t\t}"
				channels = channels + "\n\t\t\t}"
				for channel in self.channels:
						channels = channels + self.__getattribute__(channel).write()
				materials = self.writeMaterials()
				return "\n%s %s%s" % (ifelse(self.prop, "prop", "actor"), self.iname, ifelse(self.prop, "", ":1")) +\
						"\n\t{" +\
						"\n\tname %s" % self.name +\
						"\n\t%s" % self.on +\
						"\n\tbend %i" % self.bend +\
						"\n\tdynamicsLock %i" % self.dynamicsLock +\
						"\n\thidden %i" % self.hidden +\
						"\n\taddToMenu %i" % self.addToMenu +\
						"\n\tcastsShadow %i" % self.castsShadow +\
						"\n\tincludeInDepthCue %i" % self.includeInDepthCue +\
						"\n\tparent %s%s" % (self.parent.iname, ifelse(self.parent.prop, "", ":1")) +\
						"\n\tchannels" +\
						"\n\t\t{" +\
						"\n%s" % channels +\
						"\n\t\t}" +\
						"\n\tendPoint %f %f %f" % self.endPoint +\
						"\n\torigin %f %f %f" % self.origin +\
						"\n\torientation %i %i %i" % self.orientation +\
						"\n\tdisplayOrigin %i" % self.displayOrigin +\
						"\n\tdisplayMode %s" % self.displayMode +\
						"\n\tcustomMaterial %i" % self.customMaterial +\
						materials +\
						"\n\tlocked %s" % self.locked +\
						"\n\tvisibleInRender %i" % self.visibleInRender +\
						"\n\t}"
		def writeAddChild(self):
				return "\n\taddChild %s%s" % (self.iname, ifelse(self.prop,"",":1")) +\
					   "\n\t\t%s%s" % (self.parent.iname, ifelse(self.parent.prop, "", ":1") )
		def writeRoot(self):
				return "\n\troot %s%s" % (self.iname, ifelse(self.prop,"",":1"))
		def writeMaterials(self):
				materials = ""
				for m in self.materials:
						materials = materials + m.write()
				return materials
UNIVERSE = PwElement(None, "UNIVERSE")
class PwSystem(PwPoserActor):
		def __init__(self, sName):
				PwPoserActor.__init__(self)
				self.name = sName
				self.iname = sanitizeName(sName)
				self.id = float( int(random.random() * 1000000) )
				self.particlegroups = []
				self.emitter = PwEmitter(self)
				self.resFile = ""
		def assignTo(self, fig, conformAssigned=True):
				PwPoserObject.assignTo(self, fig)
				body = self.particlegroups[0]
				body.assignTo(fig.ParentActor())
				#body.parameters.get(PARM_ID)
				for p in body.particles:
						p.assignTo(fig.Actor(p.name))
				# Synch emitter with system
				self.emitter.id = self.id
				self.id = fig.ParentActor().Parameter(PARM_ID).Value()
				for prop in poser.Scene().Actors():
						ME.StatusBar("Processing prop %s" % prop.Name())
						if prop.IsProp():
								parm = prop.Parameter(PARM_ID)
								if parm and parm.Value() == self.id:
										self.emitter.assignTo(prop)
										break
				if not self.emitter.source:
					ErrorBox("No emitter was found in the scene for " + self.name)
					raise MyError(1)
				if conformAssigned:
						body.parameters.setShape(body.shape)
						body.parameters.setCount(body.numParticles())
		def addParticleGroup(self, sName, nParticles, nShape):
				oGroup = PwParticleGroup( self, sName, nParticles, nShape )
				self.particlegroups.append( oGroup )
		def numGroups( self ):
				return len( self.particlegroups )
		def write(self, outdir):
				outcr2 = os.path.join(outdir, "temp.cr2")
				outpp2 = os.path.join(outdir, "temp.pp2")
				geoms = ""
				actors = ""
				groupedChildren = ""
				figureMaterials = ""
				for g in self.particlegroups:
						geoms = geoms + g.writeGeometry()
						actors = actors + g.writeActor()
						groupedChildren = groupedChildren + g.writeRoot()
						figureMaterials = g.writeMaterials()
						for p in g.particles:
								geoms = geoms + p.writeGeometry()
								actors = actors + p.writeActor()
								groupedChildren = groupedChildren + p.writeAddChild()
				out =  "{" +\
					   "\nversion" +\
					   "\n\t{" +\
					   "\n\tnumber 6.0" +\
					   "\n\t}" +\
					   "\nfigureResFile \"%s\"" % self.resFile +\
					   geoms +\
					   "\nfigureResFile \"%s\"\n" % self.resFile +\
					   actors +\
					   "\n\nfigure" +\
					   "\n\t{" +\
					   "\n\tname %s" % self.name +\
					   groupedChildren +\
					   "\n\tdefaultPick BODY:1" +\
					   "\n\tdisplayOn 1" +\
					   "\n\tallowsBending 0" +\
					   "\n\tfigureType 1318" +\
					   "\n\torigFigureType 1318" +\
					   "\n\tcanonType 8" +\
					   "\n\tconforming 0" +\
					   figureMaterials +\
					   "\n\tdisplayMode USEPARENT" +\
					   "\n\tlocked 0" +\
					   "\n\t}" +\
					   "\n\tsetGeomHandlerOffset 0 0 0" +\
					   "\n}"
				f = open(outcr2, "w")
				f.write(out)
				f.close()
				out = "{" +\
					  "\nversion" +\
					  "\n\t{" +\
					  "\n\tnumber 6.0" +\
					  "\n\t}" +\
					  self.emitter.writeGeometry() +\
					  self.emitter.writeActor() +\
					  "\n\ndoc" +\
					  "\n\t{" +\
					  "\n\taddActor %s" % self.emitter.iname +\
					  "\n\t}"
				f = open(outpp2, "w")
				f.write(out)
				f.close()
				return outcr2, outpp2
class PwEmitter(PwElement):
		def __init__(self, oParent):
				PwElement.__init__(self, oParent, oParent.name + " Emitter")
				self.setType(EMITTER_TYPE)
				self.parent = UNIVERSE
				self.system = oParent
				self.iname = sanitizeName(self.system.name) + "Emitter"
				self.geom = GEOM_EMITTER
				self.addScales(hidden=True)
				self.addRotations()
				self.addTranslations()
				self.addGroup( "Simulation", [IPARM_EMISSIONANGLE] )
				self.addChannel( IPARM_EMISSIONANGLE, PARM_EMISSIONANGLE, 60.0 )
				self.addChannel( IPARM_ID, PARM_ID, self.system.id )
				self.customMaterial = 1
				self.addMaterial( "ArrowX", (1,0,0,0.5), (1,0,0,0.5) )
				self.addMaterial( "ArrowY", (0,1,0,0.5), (0,1,0,0.5) )
				self.addMaterial( "ArrowZ", (0,0,1,0.5), (0,0,1,0.5) )
				self.addMaterial( "EmitterBase", (1,1,1,0.5), (1,1,1,0.5) )
				self.prop = True
				self.visibleInRender = 0
		def getSpawnPoint(self):
				return PwPoint( self.source.WorldDisplacement() )
		def getSpawnRotation(self):
				if self.source:
						def getVectorRotation(obj):
								rotation = PwVector(0,0,0)
								parm = obj.ParameterByCode(poser.kParmCodeXROT)
								if parm: rotation.x = parm.Value()
								parm = obj.ParameterByCode(poser.kParmCodeYROT)
								if parm: rotation.y = parm.Value()
								parm = obj.ParameterByCode(poser.kParmCodeZROT)
								if parm: rotation.z = parm.Value()
								parent = obj.Parent()
								if parent and parent.InternalName() != "UNIVERSE":
										rotation += getVectorRotation(parent)
								return rotation
						rotation = getVectorRotation(self.source)
						return rotation
				return None
		def getSpawnVector(self):
				if USE_RANDOM == True:
					x_angle = self.parameters.getEmissionAngle()/2 * ifelse(random.random() >= 0.5, 1.0, -1.0) * random.random()
					y_angle = random.random()*360
				else:
					x_angle = self.parameters.getEmissionAngle()
					y_angle = 0
				x_rot = rotation(XAXIS, x_angle)
				y_rot = rotation(YAXIS, y_angle)
				vector = PwQuaternion(self.source.WorldQuaternion()).matrix() * (y_rot * x_rot * YAXIS)
				#vector = PwQuaternion(self.source.WorldQuaternion()) * (y_rot * x_rot * YAXIS)
				return y_rot, x_rot, PwVector(x_angle, y_angle, 0), vector
class PwParticle(PwElement):
		def __init__(self, oParent, sName):
				PwElement.__init__(self, oParent, sName)
				self.setType(PARTICLE_TYPE)
				self.addScales(hidden=True)
				self.addRotations(hidden=True)
				self.addTranslations(hidden=True)
				self.addChannel( IPARM_LIFE, PARM_LIFE, 0.0, hidden=False )
				self.customMaterial = 1
				self.kill()
		def addScales(self, hidden=False):
				PwElement.addScales(self, hidden=hidden)
				self.scale.value = 0.0
		def spawn(self, emitter, group):
				#self.localPoint = group.parameters.getParticleOffset() # PwPoint(0,0,0)
				self.localRotation = PwVector(0,0,0)
				self.localScale = PwVector(0,0,0)
				self.spawnPoint = emitter.getSpawnPoint()
				self.spawnYrot, self.spawnXrot, self.spawnAngle, self.spawnVector = emitter.getSpawnVector()
				self.spawnSpace = emitter.getObjectSpace()
				#* translation(self.spawnPoint.x,
				#							   self.spawnPoint.y,
				#							   self.spawnPoint.z)
				##self.spawnSpace = self.spawnYrot * self.spawnXrot * emitter.getObjectSpace()
				###self.spawnSpace = emitter.getObjectSpace() * self.spawnYrot * self.spawnXrot
				self.localPoint = self.spawnYrot * self.spawnXrot * group.parameters.getParticleOffset() # PwPoint(0,0,0)
				###self.localPoint = group.parameters.getParticleOffset()
				#self.spawnRotation = emitter.getSpawnRotation() #emitter.getObjectRotation()
				"""
				self.spawnYrot = rotation(YAXIS, 0)
				self.spawnXrot = rotation(XAXIS, 0)
				self.spawnAngle = PwVector(0,1,0)
				"""
				self.spawnVector = PwQuaternion(emitter.source.WorldQuaternion()).matrix() * PwVector(0,1,0)
				#self.spawnVector = PwQuaternion(emitter.source.WorldQuaternion()) * PwVector(0,1,0)
				self.spawnRotation = emitter.getObjectRotation()
				self.velocity.define( group.parameters.getVelocity(),
									  group.parameters.getAcceleration(),
									  group.parameters.getVelocityVariance() )
				#self.velocity.define( group.parameters.getVelocity()/poser.Scene().FramesPerSecond(),
				#					   group.parameters.getAcceleration(),
				#					   group.parameters.getVelocityVariance() )
				self.orbit.define( group.parameters.getOrbit(),
								   group.parameters.getOrbitAcceleration(),
								   group.parameters.getOrbitVariance() )
				self.spin.define( group.parameters.getSpin(),
								  group.parameters.getSpinAcceleration(),
								  group.parameters.getSpinVariance() )
				self.grow.define( group.parameters.getGrow(),
								  group.parameters.getGrowAcceleration(),
								  group.parameters.getGrowVariance() )
				self.parameters.setLife(1.0)
				self.parameters.setScale(1.0, 0.0, 0.0, 0.0)
		def kill(self):
				self.spawnPoint = None
				self.spawnVector = None
				self.spawnSpace = None
				self.velocity = PwMover()
				self.orbit = PwMover()
				self.spin = PwMover()
				self.grow = PwMover()
				if self.source:
						self.parameters.setLife(-1.0)
						self.parameters.setScale(0.0, 0.0, 0.0, 0.0)
						self.parameters.setRot(0.0, 0.0, 0.0)
						self.parameters.setTran(0.0,0.0,0.0)
		def key(self):
				self.source.AddKeyFrame()
		def unkey(self, frame):
				self.source.DeleteKeyFrame(int(frame))
class PwParticleGroup(PwElement):
		def __init__(self, oParent, sName, nParticles, nShape):
				PwElement.__init__(self, oParent, sName)
				self.setType(GROUP_TYPE)
				self.system = oParent
				self.parent = UNIVERSE
				self.name = "GetStringRes(1024,1)"
				self.iname = "BODY"
				self.on = "off"
				#self.addScales()
				#self.addRotations()
				#self.addTranslations()
				self.addChannel( IPARM_ID, PARM_ID, oParent.id )
				self.particles = []
				self.addScales(hidden=True)
				self.addRotations(hidden=True)
				self.addTranslations(hidden=True)
				self.addGroup( "Particle Transform", [IPARM_PSCALE, IPARM_PXSCALE, IPARM_PYSCALE, IPARM_PZSCALE, IPARM_PXROTATE, IPARM_PYROTATE, IPARM_PZROTATE, IPARM_PXTRAN, IPARM_PYTRAN, IPARM_PZTRAN] )
				self.addGroup( "Simulation", [IPARM_EMISSION, IPARM_RECYCLE, IPARM_LIFESPAN, IPARM_START, IPARM_END, IPARM_GRAVITY] )
				self.addGroup( "Velocity", [IPARM_VELOCITYX, IPARM_VELOCITYY, IPARM_VELOCITYZ, IPARM_ACCELERATIONX, IPARM_ACCELERATIONY, IPARM_ACCELERATIONZ, IPARM_VELOCITYVAR] )
				self.addGroup( "Orbit", [IPARM_ORBITX, IPARM_ORBITY, IPARM_ORBITZ, IPARM_ORBITAX, IPARM_ORBITAY, IPARM_ORBITAZ, IPARM_ORBITVAR] )
				self.addGroup( "Spin", [IPARM_SPINX, IPARM_SPINY, IPARM_SPINZ, IPARM_SPINAX, IPARM_SPINAY, IPARM_SPINAZ, IPARM_SPINVAR] )
				self.addGroup( "Grow", [IPARM_GROWX, IPARM_GROWY, IPARM_GROWZ, IPARM_GROWAX, IPARM_GROWAY, IPARM_GROWAZ, IPARM_GROWVAR] )
				self.addChannel( IPARM_COUNT, PARM_COUNT, nParticles, hidden=True )
				self.addChannel( IPARM_SHAPE, PARM_SHAPE, 0.0, hidden=True )
				# Particle Transform
				self.addChannel( IPARM_PSCALE, PARM_PSCALE, 100.0 )
				#self.addChannel( IPARM_PXSCALE, PARM_PXSCALE, 100.0 )
				#self.addChannel( IPARM_PYSCALE, PARM_PYSCALE, 100.0 )
				#self.addChannel( IPARM_PZSCALE, PARM_PZSCALE, 100.0 )
				self.addChannel( IPARM_PXROTATE, PARM_PXROTATE, 0.0 )
				self.addChannel( IPARM_PYROTATE, PARM_PYROTATE, 0.0 )
				self.addChannel( IPARM_PZROTATE, PARM_PZROTATE, 0.0 )
				self.addChannel( IPARM_PXTRAN, PARM_PXTRAN, 0.0 )
				self.addChannel( IPARM_PYTRAN, PARM_PYTRAN, 0.0 )
				self.addChannel( IPARM_PZTRAN, PARM_PZTRAN, 0.0 )
				# Simulation
				self.addChannel( IPARM_EMISSION, PARM_EMISSION, 1.0 )
				self.addChannel( IPARM_RECYCLE, PARM_RECYCLE, 1.0 )
				self.addChannel( IPARM_LIFESPAN, PARM_LIFESPAN, 30.0 )
				self.addChannel( IPARM_START, PARM_START, 1.0 )
				self.addChannel( IPARM_END, PARM_END, -1.0 )
				self.addChannel( IPARM_GRAVITY, PARM_GRAVITY, 1.0 )
				# Velocity
				self.addChannel( IPARM_VELOCITYX, PARM_VELOCITYX, 0.0 )
				self.addChannel( IPARM_VELOCITYY, PARM_VELOCITYY, 1.0 )
				self.addChannel( IPARM_VELOCITYZ, PARM_VELOCITYZ, 0.0 )
				self.addChannel( IPARM_ACCELERATIONX, PARM_ACCELERATIONX, 0.0 )
				self.addChannel( IPARM_ACCELERATIONY, PARM_ACCELERATIONY, -0.1 )
				self.addChannel( IPARM_ACCELERATIONZ, PARM_ACCELERATIONZ, 0.0 )
				self.addChannel( IPARM_VELOCITYVAR, PARM_VELOCITYVAR, 0.0 )
				# Orbit
				self.addChannel( IPARM_ORBITX, PARM_ORBITX, 0.0 )
				self.addChannel( IPARM_ORBITY, PARM_ORBITY, 0.0 )
				self.addChannel( IPARM_ORBITZ, PARM_ORBITZ, 0.0 )
				self.addChannel( IPARM_ORBITAX, PARM_ORBITAX, 0.0 )
				self.addChannel( IPARM_ORBITAY, PARM_ORBITAY, 0.0 )
				self.addChannel( IPARM_ORBITAZ, PARM_ORBITAZ, 0.0 )
				self.addChannel( IPARM_ORBITVAR, PARM_ORBITVAR, 0.0 )
				# Spin
				self.addChannel( IPARM_SPINX, PARM_SPINX, 0.0 )
				self.addChannel( IPARM_SPINY, PARM_SPINY, 0.0 )
				self.addChannel( IPARM_SPINZ, PARM_SPINZ, 0.0 )
				self.addChannel( IPARM_SPINAX, PARM_SPINAX, 0.0 )
				self.addChannel( IPARM_SPINAY, PARM_SPINAY, 0.0 )
				self.addChannel( IPARM_SPINAZ, PARM_SPINAZ, 0.0 )
				self.addChannel( IPARM_SPINVAR, PARM_SPINVAR, 0.0 )
				# Grow
				self.addChannel( IPARM_GROWX, PARM_GROWX, 0.0 )
				self.addChannel( IPARM_GROWY, PARM_GROWY, 0.0 )
				self.addChannel( IPARM_GROWZ, PARM_GROWZ, 0.0 )
				self.addChannel( IPARM_GROWAX, PARM_GROWAX, 0.0 )
				self.addChannel( IPARM_GROWAY, PARM_GROWAY, 0.0 )
				self.addChannel( IPARM_GROWAZ, PARM_GROWAZ, 0.0 )
				self.addChannel( IPARM_GROWVAR, PARM_GROWVAR, 0.0 )

				for i in range(int(nParticles)):
						self.addParticle(i)
						ME.StatusBar("Particle %s added to system %s" % (i, sName))
						wx.Yield()
				self.setShape(nShape)
				if isinstance(nShape, unicode) or isinstance(nShape, str):
						self.shape = SHAPE_CUSTOM
				else:
						self.shape = nShape
		def addParticle(self, n):
				particle = PwParticle(self, "Particle_%i" % n )
				particle.addControlToScales( self.system.name, self.iname, IPARM_PSCALE, IPARM_PXSCALE, IPARM_PYSCALE, IPARM_PZSCALE, 0.01 )
				particle.addControlToRots( self.system.name, self.iname, IPARM_PXROTATE, IPARM_PYROTATE, IPARM_PZROTATE, 1.0 )
				particle.addChannel( IPARM_XSCALEMOD, PARM_XSCALEMOD, 0.0, hidden=True )
				particle.addChannel( IPARM_YSCALEMOD, PARM_YSCALEMOD, 0.0, hidden=True )
				particle.addChannel( IPARM_ZSCALEMOD, PARM_ZSCALEMOD, 0.0, hidden=True )
				particle.addModifierToScales( self.system.name, particle.iname, IPARM_XSCALEMOD, IPARM_YSCALEMOD, IPARM_ZSCALEMOD, 1.0 )
				self.particles.append( particle )
		def numParticles(self):
				return len( self.particles )
		def setShape(self, shape):
				geompath = shape
				if shape == SHAPE_BALL:
						geompath = GEOM_BALL
				elif shape == SHAPE_TORUS:
						geompath = GEOM_TORUS
				elif shape == SHAPE_PILL:
						geompath = GEOM_PILL
				elif shape == SHAPE_SPRITE:
						geompath = GEOM_SPRITE
				elif shape == SHAPE_CONE:
						geompath = GEOM_CONE
				for p in self.particles:
						p.setGeometry(geompath)
		def killAll(self):
				for p in self.particles:
						p.kill()
		def keyAll(self):
				for p in self.particles:
						p.key()
		def unkeyAll(self, frame):
				for p in self.particles:
						p.unkey(frame)
class PwAttractor(PwPoserActor):
		def __init__(self, obj):
				PwPoserActor.__init__(self)
				self.source = obj
				self.strength = self.source.Parameter(PARM_ATTRACTION)
				self.range = self.source.Parameter(PARM_RANGE)
				self.falloff = self.source.Parameter(PARM_FALLOFF)
		def getOrigin(self):
				return PwPoint(self.source.WorldDisplacement())
		def getStrength(self):
				return self.strength.Value()
		def setStrength(self, v):
				self.strength.SetValue(v)
		def getRange(self):
				return self.range.Value() * 0.190
		def setRange(self, v):
				self.range.SetValue(v)
		def getFalloff(self):
				return self.falloff.Value()
		def setFalloff(self, v):
				self.falloff.SetValue(v)
class PwDeflector(PwPoserActor):
		def __init__(self, obj):
				PwPoserActor.__init__(self)
				self.source = obj
				self.dampening = self.source.Parameter(PARM_DAMPENING)
				self.scale = self.source.ParameterByCode(poser.kParmCodeASCALE)
		def getDampening(self):
				return self.dampening.Value()
		def setDampening(self, v):
				self.dampening.SetValue(v)
		def getSize(self):
				return self.scale.Value()
		def setSize(self, v):
				self.scale.SetValue(v)
def getSystems():
		systems = []
		for system in poser.Scene().Figures():
				if isParticleFigure(system):
						s = buildSystemFrom(system)
						if s: systems.append(s)
		return systems
def getEmitters():
		emitters = []
		for actor in poser.Scene().Actors():
				if isEmitter(actor):
						emitters.append( actor )
		return emitters
def getAttractors():
		attractors = []
		for actor in poser.Scene().Actors():
				if isAttractor(actor):
						attractors.append( PwAttractor(actor) )
		return attractors
def getDeflectors():
		deflectors = []
		for actor in poser.Scene().Actors():
				if isDeflector(actor):
						deflectors.append( PwDeflector(actor) )
		return deflectors
def isParticleFigure(figure):
		parm = figure.ParentActor().Parameter(PARM_TYPE)
		if parm:
				if parm.Value() == GROUP_TYPE or parm.Value() == SYSTEM_TYPE:
						return True
		return False
def isEmitter(actor):
		parm = actor.Parameter(PARM_TYPE)
		if parm:
				if parm.Value() == EMITTER_TYPE:
						return 1
		return 0
def isParticle(actor):
		parm = actor.Parameter(PARM_TYPE)
		if parm:
				if parm.Value() == PARTICLE_TYPE:
						return 1
		return 0
def isAttractor(actor):
		parm = actor.Parameter(PARM_TYPE)
		if parm:
				if parm.Value() == ATTRACTOR_TYPE:
						return 1
		return 0
def isDeflector(actor):
		parm = actor.Parameter(PARM_TYPE)
		if parm:
				if parm.Value() == DEFLECTOR_TYPE:
						return 1
		return 0
def debug():
		system = createSystem("Particle Figure", [("Cones", 60, SHAPE_CONE)])

# prtSim
def ageParticleGroup(emitter, group, end=False):
	e = emitter.parameters
	g = group.parameters
	recycle = g.getRecycle()
	lifespan = g.getLifespan()
	# Array to hold particles that can be spawned
	available = []
	for particle in group.particles:
		p = particle.parameters
		particle.key()
		# Get the particle's age (in frames)
		age = p.getLife() + 1
		if age > lifespan:
			# Kill the particle if it has lived long enough
			# and then add it to the available list if
			# recycling is enabled
			if recycle:
				particle.kill()
				available.append(particle)
			else:
				p.setScale(0.0,0.0,0.0,0.0)
		elif age == 0:
			# Particle is not yet spawned, and therefore
			# should be added to available list
			available.append(particle)
		else:
			# Record the particle's new age
			p.setLife(age)
			# Particle is already spawned and needs to be updated
			updateParticle(particle, emitter, group)
	spawnThisMany = ifelse(end, 0, g.getRateOfEmission())
	# If there are particles available to spawn, spawn them until
	# the needs of the emitter have been satisfied or until
	# you run out of particles.
	for particle in available:
		if spawnThisMany == 0:
			break
		else:
			spawnThisMany = spawnThisMany - 1
			particle.spawn(emitter, group)
			updateParticle(particle, emitter, group)
def updateParticle(particle, emitter, group):
	p = particle.parameters
	e = emitter.parameters
	g = group.parameters
	age = p.getLife()
	fps = poser.Scene().FramesPerSecond()
	origin = getWorldPoint(particle)
	# Set the point
	newPoint = particle.spawnSpace.inverse() * particle.localPoint
	# Move the point
	newPoint = newPoint + particle.spawnYrot * particle.spawnXrot * particle.velocity.getUnmultiplied(age) / fps
	# Orbit the point around the local-space spawn point (0,0,0)
	orbit = particle.orbit.getUnmultiplied(age)
	orbitx, orbity, orbitz = orbit.x, orbit.y, orbit.z
	newPoint = rotation(ZAXIS, orbitz) * rotation(YAXIS, orbity) * rotation(XAXIS, orbitx) * newPoint
	# Transform the point into world space
	newPoint = particle.spawnSpace * newPoint
	# Figure out attractor influence
	for attractor in g_aATTRACTORS:
		#print "Old", newPoint
		newPoint = attract(attractor, newPoint)
		#print "New", newPoint
	# Store the local point value
	particle.localPoint = translation(-particle.spawnPoint.x,
									  -particle.spawnPoint.y,
									  -particle.spawnPoint.z) * newPoint
	#print "New local", particle.localPoint
	# Gravity
	gravity = 0.5 * (3.739 * g.getGravity() / fps) * (age*age)
	# Move the particle
	p.setTran(newPoint.x, newPoint.y - gravity, newPoint.z)
	# Deflect
	"""
	destination = getWorldPoint(particle)
	for deflector in g_aDEFLECTORS:
		deflect(deflector, particle, origin, destination)
	"""
	# Calculate spin
	newRot = particle.localRotation + particle.spin.getUnmultiplied(age)
	particle.localRotation = newRot
	# Orient the particle along its spawn vector
	newRot = particle.spawnAngle + newRot + particle.spawnRotation
	# Assign rotation values
	p.setRot(newRot.x, newRot.y, newRot.z)
	# Calculate grow
	#particle.localScale = particle.localScale + particle.grow.get(age)
	particle.localScale = particle.localScale + particle.grow.getUnmultiplied(age)
	p.setScaleMod(particle.localScale.x, particle.localScale.y, particle.localScale.z)
def attract(attractor, point):
	distance = attractor.getOrigin().dist(point)
	attrRange = attractor.getRange()
	attrStrength = attractor.getStrength() / 10
	if distance > attrRange or attrRange == 0 or attrStrength == 0 or distance == 0:
		return point
	else:
		"""
		attrFalloff = attractor.getFalloff()
		sway = distance/attrRange * attrStrength * (attrFalloff) ** (distance/attrRange)
		#print distance, sway
		return sway * attractor.getOrigin() + point
		"""
		#return attractor.getOrigin().blend(point, attrStrength * (1.0 - distance/attrRange))
		attrVec = (attractor.getOrigin() - point).normalize()
		proximity = distance / attrRange
		return point + attrVec * attrStrength * proximity * ((1.0 - attractor.getFalloff()) ** proximity)
def deflect(deflector, particle, p0, p1):
	worldSpace = deflector.getObjectSpace()
	deflectorPos = getWorldPoint(deflector)
	# Transform the points into local space
	P0 = worldSpace.inverse() * p0
	P1 = worldSpace.inverse() * p1
	# Intersect the segment created by the points with the local plane
	u = (P1 - P0).normalize()
	w = (P0 - PwPoint(0,0,0)).normalize()
	Pn = PwVector(0,1,0)
	D = Pn.dot(u)
	N = -Pn.dot(w)
	# Test if segment is on the plane
	if math.fabs(D) < 0.0001:
		return False
	# Test if there is no intersection
	sI = N / D
	if sI < 0 or sI > 1:
		return False
	# Otherwise, find point of intersection
	I0 = P0 + sI * u
	# If the point of intersection is out of bounds, stop
	radius = 0.382 * deflector.getSize()
	if I0.x > radius or I0.x < -radius or I0.z > radius or I0.z < -radius:
		return False
	# Get the distance left to travel
	remainder = P0.dist(P1) - P0.dist(I0)
	# Get the dampening
	dampening = 1.0 - deflector.getDampening()
	# Find the reflected vector
	R = (u - 2 * u.dot(Pn) * Pn).normalize()
	# Move the point along the reflected vector
	I1 = I0 + dampening * remainder * R
	# Transform the point back into world space
	I1 = worldSpace * I1
	# Return the dampening, revised spawning vector and the new point
	#particle.spawnPoint = I1
	#particle.spawnVector = (IDENTITY4x4, IDENTITY4x4, R, worldSpace * R)
	#particle.spawnSpace = worldSpace.inverse()
	particle.parameters.setTran(I1.x, I1.y, I1.z)
	particle.velocity = particle.velocity * dampening
	particle.orbit = particle.orbit * dampening
	particle.spin = particle.spin * dampening
	particle.spawnPoint = translation(-deflectorPos.x, -deflectorPos.y, -deflectorPos.z) * worldSpace * I0
	particle.spawnVector = (particle.spawnVector[0], particle.spawnVector[1], particle.spawnVector[2], R)
	return True
def getWorldPoint(obj):
	return PwPoint(obj.source.WorldDisplacement())
def simulateFrame(system):
	# Go through particle groups
	for group in system.particlegroups:
		# Get number of particles to spawn this frame
		# Advance the age of the particles in the system
		ageParticleGroup(system.emitter, group)
def simulate(system, inplace=0, update=None):
	
	if system:
	
		# Initialise
		global g_aATTRACTORS, g_aDEFLECTORS
		g_aATTRACTORS = getAttractors()
		g_aDEFLECTORS = getDeflectors()
		scene = poser.Scene()
		
		for group in system.particlegroups:
			
			for frame in range(scene.NumFrames()):
				group.unkeyAll(frame)
				
			# Fresh start
			scene.SetFrame(0)
			group.killAll()
			
			if group.parameters.getShape() == SHAPE_SPRITE:
				for particle in group.particles:
					particle.source.PointAt(scene.CurrentCamera())
					
			# Loop over particle range
			start = group.parameters.getStart() - 1
			end = group.parameters.getEnd() - 1
			if end < 0 or end < start:
				end = scene.NumFrames() - 1
				
			# Kill all particles before the simulation range
			if start >= 0:
				current = 0
				while current <= start:
					scene.SetFrame(int(current))
					group.killAll()
					current = current + 1
					
			# Simulate each frame inside the simulation range
			current = int(start)
			maxinplace = inplace
			
			while current >= start and current <= end:
				
				if update: update("Simulating %s - Frame %i" % (system.name, current + 1))
				scene.SetFrame(int(current))
				
				while inplace > 0:
					if update: update("Simulating %s - In Place (Step %i of %i)" % (system.name, maxinplace - inplace, maxinplace))
					ageParticleGroup(system.emitter, group)
					inplace -= 1
					wx.Yield()
					
				ageParticleGroup(system.emitter, group)
				current += 1
				wx.Yield()
				
			# Kill all particles after the simulation range
			if end >= 0 and end > start:
				current = end + 1
				while current < scene.NumFrames():
					scene.SetFrame(int(current))
					ageParticleGroup(system.emitter, group, end=True)
					current = current + 1

# prt3Preset
class PwPoserObject2(object):
	def __init__(self, obj):
		self.source = obj
	def write(self):
		pass
	def Name(self):
		return self.source.Name()
class PwPoserParameter(PwPoserObject2):
	def write(self):
		return "\n\t\tvalueParm %s" % self.source.InternalName() +\
			   "\n\t\t\t{" +\
			   "\n\t\t\tkeys" +\
			   "\n\t\t\t\t{" +\
			   "\n\t\t\t\tk 0 %f" % self.source.Value() +\
			   "\n\t\t\t\t}" +\
			   "\n\t\t\t}"
	def writeNameValue(self):
		return "\n\t%s %i" % (self.source.InternalName(), int(self.source.Value()))
class PwPoserActor2(PwPoserObject2):
	def __init__(self, obj, autobuild=0):
		PwPoserObject2.__init__(self, obj)
		self.channels = []
		if autobuild:
			for parm in obj.Parameters():
				self.addChannel(parm)
	def addParameter(self, name):
		parm = self.source.Parameter(name)
		if parm:
			self.addChannel(parm)
		else:
			ME.StatusBar("Warning! Skipping parameter '%s'..." % name)
	def addChannel(self, parm):
		self.channels.append( PwPoserParameter(parm) )
	def removeChannel(self, name):
		for i in range( len(self.channels) ):
			if self.channels[i].Name() == name:
				self.channels.remove(i)
				break
	def write(self):
		parms = ""
		for channel in self.channels:
			parms = parms + channel.write()
		return "\n\nactor %s" % self.source.InternalName() +\
			   "\n\t{" +\
			   "\n\tchannels" +\
			   "\n\t\t{" +\
			   parms +\
			   "\n\t\t}" +\
			   "\n\t}"
class PwSystemData(PwPoserActor2):
	def __init__(self, obj):
		PwPoserActor2.__init__(self, obj)
		self.properties = []
		name = self.source.ItsFigure().Name()
		self.addStringProperty("name", name)
		parm = self.source.Parameter(PARM_COUNT)
		if parm:
			self.addIntProperty("count", parm.Value())
		shapeGeom = ""
		shape = int(obj.Parameter(PARM_SHAPE).Value())
		if shape != SHAPE_CUSTOM:
			shapeGeom = {  SHAPE_BALL : GEOM_BALL,
						   SHAPE_TORUS: GEOM_TORUS,
						   SHAPE_PILL : GEOM_PILL,
						   SHAPE_SPRITE: GEOM_SPRITE,
						   SHAPE_CONE : GEOM_CONE }[shape]
		else:
			shapeGeom = SelectFile(title="Select the custom geometry file", type=wx.OPEN, folder="", filetype="*.obj")
		self.addStringProperty("shape", shapeGeom)
	def addIntProperty(self, name, value):
		self.properties.append( (name, int(value)) )
	def addStringProperty(self, name, value):
		self.properties.append( (name, "\"" + value + "\"") )
	def write(self):
		parms = ""
		for channel in self.channels:
			parms = parms + channel.writeNameValue()
		props = ""
		for prop, val in self.properties:
			props = props + "\n\t%s %s" % (prop, val)
		return "\n\nPRT3_Group" +\
			   "\n\t{" +\
			   parms +\
			   props +\
			   "\n\t}"
class PwPose(object):
	def __init__(self):
		self.actors = []
		self.figure = []
	def addActor(self, actor):
		self.actors.append(actor)
	def addToFigure(self, element):
		self.figure.append(element)
	def write(self):
		actors = ""
		for actor in self.actors:
			actors = actors + actor.write()
		figure = ""
		for element in self.figure:
			figure = figure + element.write()
		return "{" +\
			   "\nversion" +\
			   "\n\t{" +\
			   "\n\tnumber 3.0" +\
			   "\n\t}" +\
			   actors +\
			   "\n\nfigure" +\
			   "\n\t{" +\
			   figure +\
			   "\n\t}" +\
			   "\n}"
def savePreset(figure, outpath):
	body = figure.ParentActor()
	matData = PwPoserMaterial(figure.Materials()[0])
	sysData = PwSystemData(body)
	bodyData = PwPoserActor2(body)
	bodyParms = (PARM_EMISSION,
				 PARM_RECYCLE,
				 PARM_LIFESPAN,
				 PARM_START,
				 PARM_END,
				 PARM_GRAVITY,
				 PARM_VELOCITYX, PARM_VELOCITYY, PARM_VELOCITYZ,
				 PARM_ACCELERATIONX, PARM_ACCELERATIONY, PARM_ACCELERATIONZ,
				 PARM_VELOCITYVAR,
				 PARM_ORBITX, PARM_ORBITY, PARM_ORBITZ,
				 PARM_ORBITAX, PARM_ORBITAY, PARM_ORBITAZ,
				 PARM_ORBITVAR,
				 PARM_SPINX, PARM_SPINY, PARM_SPINZ,
				 PARM_SPINAX, PARM_SPINAY, PARM_SPINAZ,
				 PARM_SPINVAR,
				 PARM_GROWX, PARM_GROWY, PARM_GROWZ,
				 PARM_GROWAX, PARM_GROWAY, PARM_GROWAZ,
				 PARM_GROWVAR,
				 PARM_PSCALE,
				 PARM_PXROTATE, PARM_PYROTATE, PARM_PZROTATE,
				 PARM_PXTRAN, PARM_PYTRAN, PARM_PZTRAN )
	for parm in bodyParms:
		bodyData.addParameter(parm)
	pose = PwPose()
	pose.addActor(sysData)
	pose.addActor(bodyData)
	pose.addToFigure(matData)
	output = pose.write()
	fi = open(outpath, 'w')
	fi.write(output)
	fi.close()
class PwSystemCollector(object):
	def __init__(self):
		self.options = {}
	def collect(self, line):
		cleaned = self.clean(line)
		if cleaned == "}":
			return 0
		elif cleaned == "{":
			return 1
		elif not cleaned:
			return 1
		else:
			name, value = self.format(line)
			self.options[name] = value
			return 1
	def format(self, line):
		divided = line.split()
		name = divided[0]
		value = string.join(divided[1:], " ")
		if value[0] == "\"":
			value = value[1:-1]
		else:
			value = float(value)
		return name, value
	def clean(self, line):
		return line.strip()
	def get(self):
		return self.options
def readPreset(inpath):
	fi = open(inpath)
	fir = fi.readlines()
	fi.close()
	sysCollector = PwSystemCollector()
	i = 0
	sysStart = -1
	for line in fir:
		cleaned = line.strip()
		if cleaned == "PRT3_Group":
			sysStart = i + 2
			break
		i = i + 1
	if sysStart > -1:
		i = sysStart
		line = fir[i]
		while sysCollector.collect(line):
			i = i + 1
			line = fir[i]
	return sysCollector.get()

#prt3Node
class PwPoserMaterial(object):
	def __init__(self, obj):
		self.name = obj.Name()
		self.KdColor = obj.DiffuseColor()
		self.KaColor = obj.AmbientColor()
		self.KsColor = obj.SpecularColor()
		self.tMax = obj.TransparencyMax()
		self.textureMap = obj.TextureMapFileName()
		if not self.textureMap:
			self.textureMap = "NO_MAP"
		else:
			self.textureMap = "\"" + self.textureMap + "\""
		self.transparencyMap = obj.TransparencyMapFileName()
		if not self.transparencyMap:
			self.transparencyMap = "NO_MAP"
		else:
			self.transparencyMap = "\"" + self.transparencyMap + "\""
		self.shaderTree = PwPoserShaderTree(obj.ShaderTree())
	def Name(self):
		return self.name
	def write(self):
		stData = ""
		for node in self.shaderTree.Nodes():
			ipData = ""
			for ip in node.Inputs():
				ipfile = ""
				value = ip.Value()
				if isinstance(value, str):
					ipfile = value
					value = (1, 1, 1)
				elif isinstance(value, float):
					value = (value, 0, 0)
				elif not value:
					value = (0, 0, 0)
				innode = ip.InNode()
				if not innode:
					innode = "NO_NODE"
				ipData = ipData + "\n\t\t\t\tnodeInput \"%s\"" % ip.InternalName() +\
						 "\n\t\t\t\t\t{" +\
						 "\n\t\t\t\t\tname \"%s\"" % ip.Name() +\
						 "\n\t\t\t\t\tvalue %f %f %f" % value +\
						 "\n\t\t\t\t\tparmR NO_PARM" +\
						 "\n\t\t\t\t\tparmG NO_PARM" +\
						 "\n\t\t\t\t\tparmB NO_PARM" +\
						 "\n\t\t\t\t\tnode %s" % innode +\
						 "\n\t\t\t\t\tfile \"%s\"" % ipfile +\
						 "\n\t\t\t\t\t}"
			stData = stData + "\n\t\t\tnode \"%s\" \"%s\"" % (node.Type(), node.InternalName()) +\
					 "\n\t\t\t\t{" +\
					 "\n\t\t\t\tname \"%s\"" % node.Name() +\
					 "\n\t\t\t\tpos %i %i" % node.Location() +\
					 "\n\t\t\t\tshowPreview %i" % node.PreviewCollapsed() +\
					 "\n\t\t\t\tinputsCollapsed %s" % node.InputsCollapsed() +\
					 ipData +\
					 "\n\t\t\t\t}"
		return "\n\tmaterial %s" % self.Name() +\
			   "\n\t\t{" +\
			   "\n\t\tKdColor %f %f %f 1" % self.KdColor +\
			   "\n\t\tKaColor %f %f %f 1" % self.KaColor +\
			   "\n\t\tKsColor %f %f %f 1" % self.KsColor +\
			   "\n\t\ttMax %f" % self.tMax +\
			   "\n\t\ttextureMap %s" % self.textureMap +\
			   "\n\t\ttransparencyMap %s" % self.transparencyMap +\
			   "\n\t\tshaderTree" +\
			   "\n\t\t\t{" +\
			   stData +\
			   "\n\t\t\t}" +\
			   "\n\t\t}"
class PwPoserShaderTree(object):
	def __init__(self, st):
		self.nodes = []
		for node in st.Nodes():
			self.nodes.append( PwPoserNode(node) )
	def Nodes(self):
		return self.nodes
class PwPoserNode(object):
	def __init__(self, node):
		self.inputs = []
		for ip in node.Inputs():
			self.inputs.append( PwPoserInput(ip) )
		self.inputscollapsed = node.InputsCollapsed()
		self.internalname = node.InternalName()
		self.location = node.Location()
		self.name = node.Name()
		self.previewcollapsed = node.PreviewCollapsed()
		self.type = node.Type()
	def Inputs(self):
		return self.inputs
	def InputsCollapsed(self):
		return self.inputscollapsed
	def InternalName(self):
		return self.internalname
	def Location(self):
		return self.location
	def Name(self):
		return self.name
	def PreviewCollapsed(self):
		return self.previewcollapsed
	def Type(self):
		return self.type
class PwPoserInput(object):
	def __init__(self, ip):
		self.animated = ip.Animated()
		self.canbeanimated = ip.CanBeAnimated()
		self.innode = ip.InNode()
		if self.innode:
			self.innode = self.innode.InternalName()
		self.internalname = ip.InternalName()
		self.itsnode = ip.ItsNode().InternalName()
		self.name = ip.Name()
		self.parameters = ip.Parameters()
		self.type = ip.Type()
		self.value = ip.Value()
	def Animated(self):
		return self.animated
	def CanBeAnimated(self):
		return self.canbeanimated
	def InNode(self):
		return self.innode
	def InternalName(self):
		return self.internalname
	def ItsNode(self):
		return self.itsnode
	def Name(self):
		return self.name
	def Parameters(self):
		return self.parameters
	def Type(self):
		return self.type
	def Value(self):
		return self.value
def collectShader(source):
	return PwPoserShaderTree(source.ShaderTree())
def createFromStored(stored, material):
	equivs = {}
	st = material.ShaderTree()
	for node in st.Nodes():
		if node.Type() == poser.kNodeTypeCodePOSERSURFACE:
			continue
		else:
			node.Delete()
	for node in stored.Nodes():
		newNode = None
		if node.Type() == poser.kNodeTypeCodePOSERSURFACE:
			newNode = st.Node(0)
		else:
			newNode = st.CreateNode(node.Type())
		equivs[ node.InternalName() ] = newNode.InternalName()
		newNode.SetName(node.Name())
		newNode.SetLocation(node.Location()[0], node.Location()[1])
		newNode.SetInputsCollapsed(node.InputsCollapsed())
		newNode.SetPreviewCollapsed(node.PreviewCollapsed())
		for ip in node.Inputs():
			newInput = newNode.InputByInternalName(ip.InternalName())
			#newInput.SetAnimated( ip.Animated() )
			newInput.SetName( ip.Name() )
			kind = ip.Type()
			value = ip.Value()
			if kind == poser.kNodeInputCodeCOLOR or kind == poser.kNodeInputCodeVECTOR:
				r, g, b = value
				newInput.SetColor( r, g, b )
			elif kind == poser.kNodeInputCodeSTRING:
				if value == None:
					value = ""
				newInput.SetString( value )
			else:
				newInput.SetFloat( value )
	for node in stored.Nodes():
		for ip in node.Inputs():
			inNode = ip.InNode()
			if inNode:
				outNode = st.NodeByInternalName( equivs[inNode] )
				inNode = st.NodeByInternalName( equivs[ip.ItsNode()] )
				outNode.ConnectToInput( inNode.InputByInternalName(ip.InternalName()) )
	root = st.Node(0)
	r, g, b = root.InputByInternalName("Diffuse_Color").Value()
	material.SetDiffuseColor(r, g, b)
	r, g, b = root.InputByInternalName("Ambient_Color").Value()
	material.SetAmbientColor(r, g, b)
	r = root.InputByInternalName("Transparency_Max").Value()
	material.SetTransparencyMax(r)	
def findLifeNode(actor):
	nodes = actor.Materials()[0].ShaderTree().Nodes()
	lifeNode = None
	for node in nodes:
		if node.Name() == "PRT3_Life":
			lifeNode = node
			break
	return lifeNode
def configureLifeNode(actor, lifeNode):
	if lifeNode:
		kind = actor.Parameter(PARM_TYPE).Value()
		life = 0.0
		if kind == PARTICLE_TYPE:
			life = actor.Parameter(PARM_LIFE).Value()
		lifeNode.InputByInternalName("Value_1").SetFloat(life)
		body = actor.ItsFigure().ParentActor()
		lifespan = body.Parameter(PARM_LIFESPAN).Value()
		lifeNode.InputByInternalName("Value_2").SetFloat(lifespan)
		lifeNode.InputByInternalName("Math_Argument").SetFloat(4)
def addLifeNode(actor, kind=GROUP_TYPE):
	lifeNode = findLifeNode(actor)
	if not lifeNode:
		try:
			material = actor.Materials()[0]
		except:
			material = None
		if material:
			st = material.ShaderTree()
			lifeNode = st.CreateNode(poser.kNodeTypeCodeMATH)
			lifeNode.SetName("PRT3_Life")
			lifeNode.SetInputsCollapsed(1)
			lifeNode.InputByInternalName("Value_1").SetAnimated(1)
			material.ShaderTree().UpdatePreview()
		else:
			lifenode = None
	configureLifeNode(actor, lifeNode)
	return lifeNode
def updateLifeNode(actor):
	lifeNode = findLifeNode(actor)
	if lifeNode:
		configureLifeNode(actor, lifeNode)
def findRandomNodes(actor):
	material = actor.Materials()[0]
	st = material.ShaderTree()
	randNodes = []
	for node in st.Nodes():
		if node.Name() == "PRT3_Random":
			randNodes.append(node)
	return randNodes
def addRandomNode(actor):
	material = actor.Materials()[0]
	st = material.ShaderTree()
	node = st.CreateNode(poser.kNodeTypeCodeSIMPLECOLOR)
	node.SetName("PRT3_Random")
	node.SetInputsCollapsed(1)
	node.SetLocation(160,60)
	import random
	node.Input(0).SetColor( random.random(), random.random(), random.random() )
	return node
def updateRandomNodes(actor, copy=0):
	for node in findRandomNodes(actor):
		kind = actor.Parameter(PARM_TYPE).Value()
		life = 0.0
		if kind == PARTICLE_TYPE:
			life = actor.Parameter(PARM_LIFE).Value()
		if life <= 0 or copy:
			import random
			node.Input(0).SetColor( random.random(), random.random(), random.random() )
		else:
			r, g, b = node.Input(0).Value()
			node.Input(0).SetColor(r, g, b)
def addColorBy(actor, node):
	st = actor.Materials()[0].ShaderTree()
	ramp = st.CreateNode(poser.kNodeTypeCodeCOLORRAMP)
	ramp.SetLocation(160,60)
	node.ConnectToInput(ramp.InputByInternalName("Input"))
def addColorByLife(actor):
	lifeNode = findLifeNode(actor)
	if not lifeNode:
		parm = actor.Parameter(PARM_TYPE)
		lifeNode = addLifeNode(actor, parm.Value())
	addColorBy(actor, lifeNode)
def addColorByRandom(actor):
	randNode = addRandomNode(actor)
	addColorBy(actor, randNode)
def updatePrt3Nodes(actor, copy=0):
	updateLifeNode(actor)
	updateRandomNodes(actor, copy=copy)
def setToAnimate(actor):
	try:
		material = actor.Materials()[0]
	except:
		material = None
	if material:
		st = material.ShaderTree()
		randomNodes = findRandomNodes(actor)
		for randomNode in randomNodes:
			randomNode.Input(0).SetAnimated(1)
		lifeNode = findLifeNode(actor)
		if lifeNode:
			lifeNode.InputByInternalName("Value_1").SetAnimated(1)
		st.UpdatePreview()
def nodesimulate(figure, update):
	actlist = [x for x in figure.Actors() if ValidActor(x)]
	for bodypart in actlist:
		setToAnimate(bodypart)
		update("Bodypart %s set to animated" % bodypart.Name())
	scene = poser.Scene()
	for frame in range(scene.NumFrames()):
		update("Working on frame %s" % frame)
		scene.SetFrame(int(frame))
		for bodypart in actlist:
			update("Updating nodes for %s" % bodypart.Name())
			updatePrt3Nodes(bodypart)

# Utility functions
def ErrorBox(message, title=None):
	if title == None: title = '%s says oops!' % APPNAME
	msg = wx.MessageDialog(None, message, caption=title, style=wx.OK|wx.STAY_ON_TOP)
	msg.ShowModal()
def FindGeometry(self):
	return SelectFile(folder=self.config.get('PRT3', 'GeomFolder'), filetype='*.obj')
def SelectFile(title="Choose a file", type=wx.OPEN, folder="", filetype="*.pz2"):
	
	fdlg = wx.FileDialog(None,
		title,
		defaultDir=folder,
		defaultFile='',
		wildcard=filetype,
		style=type,
		pos=wx.DefaultPosition)
		
	if fdlg.ShowModal() == wx.ID_OK:			# User chose a file	
		path = fdlg.GetPath()					# get the path to the file	
	else:
		path = None
		
	fdlg.Destroy()
	
	return path
def GetConfig(cp, item, type=None):
	if type == 'boolean':
		try:
			v = cp.getboolean('PRT3', item)
		except:
			v = cp.get('PRT3', item)
	elif type == 'float':
		try:
			v = cp.getfloat('PRT3', item)
		except:
			v = cp.get('PRT3', item)
	elif type == 'int':
		try:
			v = cp.getint('PRT3', item)
		except:
			v = cp.get('PRT3', item)			
	else:
		v = cp.get('PRT3', item)
	return v
def ValidActor(actor):
	if actor:
		if actor.IsBodyPart() and not actor.Name().lower() in [
			'focusdistancecontrol',
			 'goalcenterofmass',
			 'centerofmass',
			 'innermat',
			 'nullmat',
			 'outermat']:
			parm = actor.Parameter(PARM_TYPE)
			if parm and (parm.Value() == PARTICLE_TYPE or parm.Value() == GROUP_TYPE):
				return True
	return False
	
# The Colour class is used to manipulate colour values
class Colour(object):
	def __init__(self, col=(255,255,255), poser=False):
		self.r, self.g, self.b = self.ConvertColour(col, adj = False)
		if poser:
			self.r *= 255
			self.g *= 255
			self.b *= 255
		
	def GetColour(self, ctl):
		wxcol = ctl.GetColour()
		return ConvertColour(wxcol)
		
	def ConvertColour(self, wxcol, adj = True):
		if isinstance(wxcol, wx.Colour):
			r, g, b = wxcol.Get()
		elif isinstance(wxcol, tuple):
			r, g, b = wxcol
		else:
			try:
				col = wx.Colour(0,0,0)
				col.SetFromName(wxcol)
				r, g, b = col.Get()
			except:
				tmp = wxcol[3:]
				r, g, b = eval(tmp)
		if adj:
			return (r/255.0, g/255.0, b/255.0)
		else:
			return (r,g,b)
		
	def RGBToPoserCol(self, rgb):
		return tuple([x/255.0 for x in rgb])
	
	def HTML(self):
		return self.RGBToHTMLColor(self.RGB())
		
	def HTMLColorToRGB(self, colorstring):
		""" convert #RRGGBB to an (R, G, B) tuple """
		colorstring = colorstring.strip()
		if colorstring[0] == '#': colorstring = colorstring[1:]
		if len(colorstring) != 6:
			raise ValueError, "input #%s is not in #RRGGBB format" % colorstring
		r, g, b = colorstring[:2], colorstring[2:4], colorstring[4:]
		r, g, b = [int(n, 16) for n in (r, g, b)]
		return (r, g, b)
	
	def POSERCOL(self):
		return self.RGBToPoserCol(self.RGB())
		
	def PoserColToHTML(self, col_tuple):
		rgb = tuple([int(x*255) for x in col_tuple])
		return RGBToHTMLColor(rgb)
		
	def PoserColToRGB(self, col_tuple):
		rgb = tuple([int(x*255) for x in col_tuple])
		return rgb

	def SetColour(self, rgb):
		self.r, self.g, self.b = self.ConvertColour(rgb, adj = False)
	
	def RGB(self):
		return (self.r, self.g, self.b)
		
	def RGBToHTMLColor(self, rgb_tuple):
		hexcolor = '#%02x%02x%02x' % rgb_tuple
		return hexcolor

# Classes for custom controls
class EZControl(object):
	def __init__(self, ui, parent, parms, width ):
	
		LABEL, TYPE, RANGE, INIT, TIP, INCREMENT = range(6)
		
		# Object parameters
		self.ui = ui
		self.parent = parent
		self.label = parms[LABEL]
		self.range = parms[RANGE]
		self.initValue = parms[INIT]
		self.tooltip = parms[TIP]
		self.labelwidth = width
		self.font = ui.GetFont()
		if parms[INCREMENT]:
			self.increment = parms[INCREMENT]
		else:
			self.increment = 1
		self.ProcessRange()
		self.BuildControl()
		
	def ProcessRange(self):		
		try:
			self.minValue = self.range[0]
			self.maxValue = self.range[1]
		except:
			self.minValue = None
			self.maxValue = None		
	
	def AddPresetCheckbox(self):
		self.save = wx.CheckBox( self.parent, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, 0 )
		self.save.SetValue(True)
		self.sizer.Add( self.save, 0, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5 )
	
	def AddToSizer(self):
		self.sizer.Add( self.control, 0, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5 )
	
	def BindEvents(self):
		pass
		
	def BuildControl(self):
		
		# Sizer
		self.sizer = wx.BoxSizer(wx.HORIZONTAL)
		
		# Add checkbox for saving presets
		#self.AddPresetCheckbox()
		
		# The control's label
		self.ctllabel = wx.StaticText( self.parent, wx.ID_ANY, self.label, wx.DefaultPosition, wx.DefaultSize, wx.ALIGN_RIGHT )
		self.ctllabel.SetMinSize( wx.Size( self.labelwidth,-1))
		self.sizer.Add( self.ctllabel, 0, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5 )
		
		# The main control
		try:
			self.control = self.CreateMainControl()
			if self.control:
				self.control.SetToolTipString(self.tooltip)
				self.AddToSizer()
				self.BindEvents()
		except:
			self.control = None
			ME.StatusBar('Unable to build control %s' % self.label)

	def Increment(self):
		return self.category
		
	def ConsumeKey( self, event ):
		pass
		
	def CreateMainControl(self):
		# wxTextCtrl is the base control
		ctl = wx.TextCtrl( self.parent, wx.ID_ANY, str(self.initValue), wx.DefaultPosition, wx.DefaultSize, 0 )
		return ctl
	
	def Height(self):
		return self.control.GetSize()[1]
		
	def Max(self):
		return self.maxValue
	
	def Min(self):
		return self.minValue
		
	def Reset(self):
		if not self.initValue is None:
			self.SetValue(self.initValue)
	
	def Save(self):
		if self.save is None:
			return False
		else:
			return self.save.IsChecked()
		
	def SetValue(self, value):
		self.control.SetValue(str(value))

	def SetValue2(self, value):
		self.SetValue(value)
		
	def Sizer(self):
		return self.sizer
		
	def Successful(self):
		return (True, False)[self.control is None]
	
	def UpdateSave(self, category):
		if self.save:
			self.save.SetValue(category == 'All')
			
	def Value(self):
		return self.control.GetValue()

	def Value2(self):
		return self.Value()	
class EZCtlBool(EZControl):		
	def CreateMainControl(self):
		ctl = wx.CheckBox( self.parent, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, 0 )
		ctl.SetValue(self.initValue)
		return ctl
	
	def SetValue(self, value):
		self.control.SetValue((False,True)[value=='True'] or value==True)
class EZCtlButton(EZControl):
	def AddPresetCheckbox(self):
		self.save = None
		self.sizer.AddSpacer( ( 25, 0), 0, wx.EXPAND, 5 )
		
	def CreateMainControl(self):
		# wxTextCtrl is the base control
		ctl = wx.Button( self.parent, wx.ID_ANY, self.label, wx.DefaultPosition, wx.DefaultSize, 0 )
		self.ctllabel.SetLabel('')
		ctl.Bind(wx.EVT_BUTTON, self.initValue)
		return ctl
		
	def SetValue(self, value):
		pass

	def SetValue2(self, value):
		pass

	def Value(self):
		return None

	def Value2(self):
		return None
class EZCtlInt(EZControl):
	def CreateMainControl(self):
		ctl = wx.SpinCtrl( self.parent, wx.ID_ANY, "", wx.DefaultPosition, wx.DefaultSize, wx.SP_ARROW_KEYS, self.minValue, self.maxValue, self.initValue )
		ctl.Bind( wx.EVT_KEY_DOWN, self.ConsumeKey )
		ctl.Bind( wx.EVT_TEXT, self.ConsumeKey )
		return ctl
	def SetValue(self, value):
		self.control.SetValue(int(value))
class EZCtlColour(EZControl):
	def CreateMainControl(self):
		initValue = tuple([int(x) for x in self.initValue])
		ctl = wx.ColourPickerCtrl( self.parent, wx.ID_ANY, self.initValue, wx.DefaultPosition, wx.DefaultSize, wx.CLRP_DEFAULT_STYLE )
		return ctl
		
	def SetValue(self, col):
		colour = Colour(col).RGB()
		self.control.SetColour(colour)
		
	def SetValue2(self, col):
		self.SetValue(col)
		
	def Value(self):
		col = Colour(self.control.GetColour())
		return col.RGB()
	
	def Value2(self):
		col = self.control.GetColour()
		if isinstance(col, wx.Colour): col = col.GetAsString()
		return col		
class EZCtlColourP(EZCtlColour):
	def CreateMainControl(self):
		initValue = tuple([int(x*255) for x in self.initValue])
		ctl = wx.ColourPickerCtrl( self.parent, wx.ID_ANY, initValue, wx.DefaultPosition, wx.DefaultSize, wx.CLRP_DEFAULT_STYLE )
		return ctl

	def SetValue(self, col):
		colour = Colour(col, poser=True).RGB()
		self.control.SetColour(colour)
		
	def SetValue2(self, col):
		colour = Colour(col).RGB()
		self.control.SetColour(colour)	
		
	def Value(self):
		col = Colour(self.control.GetColour())
		return col.POSERCOL()
class EZCtlFloat(EZControl):	
	def CreateMainControl(self):
		ctl = FS.FloatSpin(self.parent, wx.ID_ANY, pos=wx.DefaultPosition, size=wx.DefaultSize, style=0, value=self.initValue, min_val=self.minValue, max_val=self.maxValue, increment=self.increment, digits=3)
		ctl.GetTextCtrl().SetToolTipString(self.tooltip)
		return ctl
		
	def SetValue(self, value):
		self.control.SetValue(float(value))
class EZCtlChoice(EZControl):
	def CreateMainControl(self):
		ctl = wx.Choice( self.parent, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, self.initValue, 0 )
		ctl.SetSelection( 0 )
		return ctl

	def Reset(self):
		self.control.SetSelection(0)
			
	def SetValue(self, value):
		self.control.SetSelection(value)

	def SetValue2(self, value):
		self.control.SetSelection(int(value))
		
	def Value(self):
		return self.control.GetSelection()
class EZCtlSelect(EZControl):
	def CreateMainControl(self):
		ctl = wx.Choice( self.parent, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, self.range, 0 )
		start = ctl.FindString(self.initValue)
		ctl.SetSelection( max(0, start ))
		return ctl
		
	def ProcessRange(self):
		self.minValue = None
		self.maxValue = None

	def Reset(self):
		start = self.control.FindString(self.initValue)
		self.control.SetSelection( max(0, start ))

	def SetValue(self, value):
		start = self.control.FindString(value)
		self.control.SetSelection( max(0, start ))

	def SetValue2(self, value):
		self.SetValue(value)
		
	def Value(self):
		return self.control.GetString(self.control.GetSelection())
class EZCtlISelect(EZCtlSelect):
		
	def Value(self):
		return self.control.GetSelection()
		
	def SetValue2(self, value):
		self.control.SetSelection( int(value))
class EZCtlFile(EZControl):
	def CreateMainControl(self):
		
		try:
			start = ('', self.initValue)[os.path.exists(self.initValue)]
		except:
			start = ''
			
		ctl = FB.FileBrowseButton( self.parent, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, labelText='', startDirectory=start)
			
		return ctl
		
	def AddToSizer(self):
		self.sizer.Add( self.control, 1, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5 )
		
	def SetValue(self, value):
		self.control.SetValue(value)
		
	def Value(self):
		return self.control.GetValue()
class EZCtlLabel(EZControl):
	def AddPresetCheckbox(self):
		self.save = None
		
	def BuildControl(self):
		
		# Sizer
		self.sizer = wx.BoxSizer(wx.HORIZONTAL)
		self.AddPresetCheckbox()
		
		# The control's label
		self.ctllabel = wx.StaticText( self.parent, wx.ID_ANY, self.label, wx.DefaultPosition, wx.DefaultSize, wx.ALIGN_LEFT )
		self.ctllabel.SetMinSize( wx.Size( -1,-1 ) )
		font = self.font
		font.SetWeight(wx.FONTWEIGHT_BOLD)
		self.ctllabel.SetFont(font)
		self.sizer.Add( self.ctllabel, 0, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5 )
		
		self.control = None

	def Height(self):
		return self.ctllabel.GetSize()[1]
		
	def SetValue(self, value):
		self.ctllabel.SetLabel(str(value))

	def Successful(self):
		return (True, False)[self.ctllabel is None]
		
	def Value(self):
		self.ctllabel.GetLabel()
class EZCtlText (EZControl):
	def AddToSizer(self):
		self.sizer.Add( self.control, 1, wx.ALIGN_CENTER_VERTICAL|wx.ALL|wx.EXPAND, 5 )
class EZCtlPresets(EZCtlSelect):
	def AddPresetCheckbox(self):
		self.save = None
		self.sizer.AddSpacer( ( 25, 0), 0, wx.EXPAND, 5 )	
		
	def BindEvents(self):
		self.control.Bind( wx.EVT_CHOICE, self.Update )
		
	def Update(self, event):
		self.ui.manager.UpdateSave(self.Value())

# Desperation ...
class poserMat(object):
	def __init__(self, material, version, getMaps=True, useExt=False, compatibility=False):
		'''
		material = a reference to a Poser Material object
		getMaps = Identify the principle texture map for each channel
		useExt = Use external names for node inputs instead of internal names
		'''
		
		self.material = material
		self.root = material.ShaderTree().Node(0)
		self.tree = material.ShaderTree()
		
		# Keep a list of nodes holding the principle texture maps in the shader
		self.mapidx = ["color", "alt_color", "specular", "alt_specular", "bump", "displacement", "transparency", "reflection", "specular_color"]
		
		self.mapnodes = []
		if getMaps:
			self.GetMapNodes()
		else:
			self.mapnodes = [None, None, None, None, None, None, None, None, None]
		
		# Maintain a dictionary to cross reference Input external names with their internal names
		self.names = {}
		self.useExt = useExt
		
		# Set up node references
		self.DefineNodeTypes(POSER_VERSION, compatibility=compatibility)
						
	def AddNode(self, type, name=None, loc=None, collapsed=True, **kw):
		
		''' Creates a node '''
		
		# Create the node
		node = self.tree.CreateNode(self.codes[type])
		
		# Configure it
		if loc: node.SetLocation(loc[0], loc[1])
		if name: node.SetName(name)
		node.SetInputsCollapsed(collapsed)
		
		# Set values of any Inputs passed
		for key, value in kw.items():
			if self.useExt == True:
				input = self.InputByName(node, key)
			else:
				input = node.InputByInternalName(key)
			self.SetInputValue(input, value)
			
		# Return a reference to the node
		return node
	
	def AlignNodes(self, left=200, separation=230, cols=False):
	
		nodes = self.AllNodes()
		self.root.SetLocation(10,10)
		
		if cols:
		
			dummy = []
			for i in self.root.Inputs():
				n = i.InNode()
				if n:
					self.Traverse(n, dummy, xy=(left, separation))
		
		else:
		
			base = 1000000
			
			for node in nodes:
				
				if node == self.root: continue
				
				x = node.Location()[0]
				base = min(base, x) 
			
			base_mod = base - left
				
			for node in nodes:
			
				if node == self.root:
					continue
					
				x, y = node.Location()
				x -= base_mod
				
				if x < left:
					newx = left
				else:
					nx = x - left
					halfway = separation / 2
					column = int((nx + halfway) / separation)
					newx = left + (separation * column)
					
				node.SetLocation(newx, y)
	
	def AllNodes(self):
		''' Gets all nodes in the shader tree '''
		return self.tree.Nodes()
		
	def ApplyTextureFilters(self, value):
		''' Sets Texture Filtering for all identified imagemaps '''
		for map in self.Maps():
			if map:
				map.InputByInternalName('Filtering').SetFloat(value)
	
	def Bump_Value(self):
		''' Gets the value of the PoserSurface's Bump Input '''
		return self.root.InputByInternalName('Bump').Value()
		
	def ClearPoserSurface(self, keepTrans=False, severe=False):
		
		''' Prepares the PoserSurface for a new shader '''
		
		ps = self.Root()
		
		# Clear Diffuse, Specular and Ambient values
		ps.InputByInternalName('Diffuse_Value').SetFloat(0.0)		
		ps.InputByInternalName('Highlight_Color').SetColor(0.0, 0.0, 0.0)
		ps.InputByInternalName('Highlight_Value').SetFloat(0.0)		
		ps.InputByInternalName('Ambient_Value').SetFloat(0.0)
		ps.InputByInternalName('AlternateSpecular').Disconnect()
		
		# Clear Transparency and Translucent values
		if keepTrans == False:
			ps.InputByInternalName('Transparency_Max').SetFloat(0.0)
			ps.InputByInternalName('Transparency_Min').SetFloat(0.0)
			ps.InputByInternalName('Translucence_Value').SetFloat(0.0)	
			ps.InputByInternalName('Transparency_Max').Disconnect()
			ps.InputByInternalName('Transparency_Min').Disconnect()
		
		# Clear remaining values
		ps.InputByInternalName('Translucence_Color').Disconnect()
		ps.InputByInternalName('Reflection_Color').Disconnect()
		ps.InputByInternalName('Reflection_Value').Disconnect()
		ps.InputByInternalName('AlternateDiffuse').Disconnect()
		ps.InputByInternalName('AlternateSpecular').Disconnect()
		ps.InputByInternalName('Bump').Disconnect()
		
		# Clear Reflection*Mult values
		ps.InputByInternalName('ReflectionLiteMult').SetFloat(0.0)
		ps.InputByInternalName('ReflectionKdMult').SetFloat(0.0)
		
		# If requested, disconnect all nodes from the PoserSurface
		if severe == True:
			for i in ps.Inputs():
				i.Disconnect()
		
		# Standardise appearance of PoserSurface
		self.ConfigureNode(ps, loc=(10,10), showInputs=True, showPreview=True)

	def ConfigureNode(self, node, loc=None, name=None, showPreview=None, showInputs=None):
	
		''' Updates an existing node's parameters '''
		
		if loc: node.SetLocation(loc[0], loc[1])
		if name: node.SetName(name)
		if showPreview != None: node.SetPreviewCollapsed((0, 1)[showPreview == True])
		if showInputs != None: node.SetInputsCollapsed((1, 0)[showInputs == True])
	
	def ConnectNodes(self, n1, n2, inp):
	
		''' Connects Node 'n1' to Node 'n2's 'inp' Input '''
		if self.useExt == True:
			n1.ConnectToInput(self.InputByName(n2, inp))
		else:
			n1.ConnectToInput(n2.InputByInternalName(inp))	
				
	def ConnectToPS(self, node, input, value=None):
		if self.useExt == True:
			i = self.InputByName(self.root, input)
		else:
			i = self.root.InputByInternalName(input)
		node.ConnectToInput(i)
		if value:
			self.SetInputValue(i, value)
	
	def DefineNodeTypes(self, version, compatibility=False):
	
		''' Cross reference Node names with Poser Node codes '''
		
		if compatibility:
			self.codes = {
				"anisotropic" : poser.kNodeTypeCodeANISOTROPIC,
				"blender" : poser.kNodeTypeCodeBLENDER,
				"blinn" : poser.kNodeTypeCodeBLINN,
				"clouds" : poser.kNodeTypeCodeCLOUDS,
				"color" : poser.kNodeTypeCodeSIMPLECOLOR,
				"color_math" : poser.kNodeTypeCodeCOLORMATH,
				"component" : poser.kNodeTypeCodeCOMP,
				"color_math": poser.kNodeTypeCodeCOLORMATH,
				"diffuse" : poser.kNodeTypeCodeDIFFUSE,
				"edge_blend" : poser.kNodeTypeCodeEDGEBLEND,
				"fractal" : poser.kNodeTypeCodeFRACTALSUM,
				"glossy" : poser.kNodeTypeCodeGLOSSY,
				"hsv" : poser.kNodeTypeCodeHSV,
				"imagemap" : poser.kNodeTypeCodeIMAGEMAP,
				"math" : poser.kNodeTypeCodeMATH,
				"n" : poser.kNodeTypeCodeN,
				"noise" : poser.kNodeTypeCodeNOISE,
				"p" : poser.kNodeTypeCodeP,
				"phong" : poser.kNodeTypeCodePHONG,
				"reflect" : poser.kNodeTypeCodeREFLECT,
				"scatter" : poser.kNodeTypeCodeSCATTER,
				'specular' : poser.kNodeTypeCodeSPECULAR,
				'sphere_map' : poser.kNodeTypeCodeSPHEREMAP,		
				"turbulence" : poser.kNodeTypeCodeTURBULENCE,
				}
		

			if version >= 9:
				self.codes.update((
					("custom_scatter", poser.kNodeTypeCodeCustomScatter),
					("fresnel_blend", "fresnel_blend"),
					("microfacet", poser.kNodeTypeCodeMICROFACET),
					('sss', poser.kNodeTypeCodeSCATTERSKIN)))
				
		else:
			self.codes = {
				"ambient_occlusion" : poser.kNodeTypeCodeAMBIENTOCCLUSION,
				"aniso" : poser.kNodeTypeCodeANISOTROPIC,
				"atmosphere" : poser.kNodeTypeCodeATMOSPHERE,
				"background" : poser.kNodeTypeCodeBACKGROUND,
				"blender" : poser.kNodeTypeCodeBLENDER,
				"blinn" : poser.kNodeTypeCodeBLINN,
				"brick" : poser.kNodeTypeCodeBRICK,
				"cellular" : poser.kNodeTypeCodeCELLULAR,
				"clay" : poser.kNodeTypeCodeCLAY,
				"clouds" : poser.kNodeTypeCodeCLOUDS,
				"color_math" : poser.kNodeTypeCodeCOLORMATH,
				"colorramp" : poser.kNodeTypeCodeCOLORRAMP,
				"comp" : poser.kNodeTypeCodeCOMP,
				"custom_scatter" : poser.kNodeTypeCodeCustomScatter,
				"diffuse" : poser.kNodeTypeCodeDIFFUSE,
				"dNdu" : poser.kNodeTypeCodeDNDU,
				"dNdv" : poser.kNodeTypeCodeDNDV,
				"dPdu" : poser.kNodeTypeCodeDPDU,
				"dPdv" : poser.kNodeTypeCodeDPDV,
				"Du" : poser.kNodeTypeCodeDU,
				"Dv" : poser.kNodeTypeCodeDV,
				"edge_blend" : poser.kNodeTypeCodeEDGEBLEND,
				"fastscatter" : poser.kNodeTypeCodeFASTSCATTER,
				"fbm" : poser.kNodeTypeCodeFBM,
				"fractal_sum" : poser.kNodeTypeCodeFRACTALSUM,
				"frame_number" : poser.kNodeTypeCodeFRAME,
				"fresnel" : poser.kNodeTypeCodeFRESNEL,
				"fresnel_blend" : "fresnel_blend",
				"gamma" : poser.kNodeTypeCodeGAMMA,
				"gather" : poser.kNodeTypeCodeGATHER,
				"glossy" : poser.kNodeTypeCodeGLOSSY,
				"granite" : poser.kNodeTypeCodeGRANITE,
				"hair" : poser.kNodeTypeCodeHAIR,
				"hsv" : poser.kNodeTypeCodeHSV,
				"image_map" : poser.kNodeTypeCodeIMAGEMAP,
				"light" : poser.kNodeTypeCodeLIGHT,
				"marble" : poser.kNodeTypeCodeMARBLE,
				"math_functions" : poser.kNodeTypeCodeMATH,
				"ks_microfacet" : poser.kNodeTypeCodeMICROFACET,
				"movie" : poser.kNodeTypeCodeMOVIE,
				"N" : poser.kNodeTypeCodeN,
				"noise" : poser.kNodeTypeCodeNOISE,
				"P" : poser.kNodeTypeCodeP,
				"phong" : poser.kNodeTypeCodePHONG,
				"polygonhair" : poser.kNodeTypeCodePOLYGONHAIR,
				"poser" : poser.kNodeTypeCodePOSERSURFACE,
				"posterize" : poser.kNodeTypeCodePOSTERIZE,
				"probeLight" : poser.kNodeTypeCodePROBELIGHT,
				"reflect" : poser.kNodeTypeCodeREFLECT,
				"refract" : poser.kNodeTypeCodeREFRACT,
				"scatter" : poser.kNodeTypeCodeSCATTER,
				"skin2" : poser.kNodeTypeCodeSCATTERSKIN,
				"simple_color" : poser.kNodeTypeCodeSIMPLECOLOR,
				"skin" : poser.kNodeTypeCodeSKIN,
				"specular" : poser.kNodeTypeCodeSPECULAR,
				"sphere_map" : poser.kNodeTypeCodeSPHEREMAP,
				"spots" : poser.kNodeTypeCodeSPOTS,
				"tile" : poser.kNodeTypeCodeTILE,
				"toon" : poser.kNodeTypeCodeTOON,
				"turbulence" : poser.kNodeTypeCodeTURBULENCE,
				"s" : poser.kNodeTypeCodeU,
				"user_defined" : poser.kNodeTypeCodeUSERDEFINED,
				"t" : poser.kNodeTypeCodeV,
				"velvet" : poser.kNodeTypeCodeVELVET,
				"wave2d" : poser.kNodeTypeCodeWAVE2D,
				"wave3d" : poser.kNodeTypeCodeWAVE3D,
				"weave" : poser.kNodeTypeCodeWEAVE,
				"wood" : poser.kNodeTypeCodeWOOD
				}
	
	def Diffuse_Color(self):
		return self.root.InputByInternalName('Diffuse_Color').Value()
	
	def FindAKindOfNode(self, node, type):
		if not node:
			return None
		if node.Type() == type:
			return node
		for inp in node.Inputs():
			v = inp.InNode()
			found = self.FindAKindOfNode(v, type)
			if found:
				return found
		return None
		
	def FindMapNode(self, input):
		node = input.InNode()
		if node:
			mapnode = self.FindAKindOfNode(node, poser.kNodeTypeCodeIMAGEMAP)
		else:
			mapnode = None
		return mapnode
	
	def GC(self, node, gc, data):
		'''
		data is True/False. If True, the node is plugged into a channel which requires linear data
		'''	
		if gc:
			if node.Type() == poser.kNodeTypeCodeIMAGEMAP and poser.IsPro():
				texture = node.InputByInternalName('Image_Source').Texture()
				if texture:
					if not data:
						texture.SetUseSceneGamma(1)
					else:
						kd = self.GetMap('color')
						if kd:
							if node.InputByInternalName('Image_Source').Value() == kd.InputByInternalName('Image_Source').Value():
								return node
							else:
								texture.SetGamma(1.0)
						else:
							texture.SetGamma(1.0)	
		return node
		
	def GetMap(self, type):
		map = self.mapnodes[self.mapidx.index(type.lower())]
		if type.lower() == 'color' and map is None:
			map = self.mapnodes[self.mapidx.index('alt_color')]
		elif type.lower() == 'specular' and map is None:
			map = self.mapnodes[self.mapidx.index('alt_specular')]
		return map
	
	def GetMapNodes(self):
		self.mapnodes.append(self.FindMapNode(self.Root().InputByInternalName('Diffuse_Color')))
		self.mapnodes.append(self.FindMapNode(self.Root().InputByInternalName('AlternateDiffuse')))
		self.mapnodes.append(self.FindMapNode(self.Root().InputByInternalName('Highlight_Value')))
		self.mapnodes.append(self.FindMapNode(self.Root().InputByInternalName('AlternateSpecular')))
		self.mapnodes.append(self.FindMapNode(self.Root().InputByInternalName('Bump')))
		self.mapnodes.append(self.FindMapNode(self.Root().InputByInternalName('Displacement')))
		self.mapnodes.append(self.FindMapNode(self.Root().InputByInternalName('Transparency_Max')))
		self.mapnodes.append(self.FindMapNode(self.Root().InputByInternalName('Reflection_Color')))
		self.mapnodes.append(self.FindMapNode(self.Root().InputByInternalName('Highlight_Color')))

	def GetMode(self):
		return self.useExt
		
	def GetPSValue(self, input):
		if self.useExt == True:
			name = self.InputByName(self.root, input).InternalName()
			return self.root.InputByInternalName(name).Value()
		else:
			return self.root.InputByInternalName(input).Value()
			
	def HasBumpMap(self):
		return (True, False)[self.mapnodes[4] is None]
	
	def HasSpecularColourMap(self):
		map = self.GetMap('specular_color')
		if map == None:
			return False
		else:
			return True	

	def HasSpecularMap(self):
		map = self.GetMap('specular')
		if map == None:
			return False
		else:
			return True
	
	def InputByName(self, node, name):
		type = node.Type()
		xref = self.names.get(type, None)
		if xref == None:
			lookup = {}
			for i in node.Inputs():
				lookup[i.Name()] = i.InternalName()
			self.names[type] = lookup
			xref = self.names.get(type, None)
		try:
			return node.InputByInternalName(xref[name])
		except:
			ShowAlert('Unknown Input (%s) in node type %s' % (name, type), 'Script error!')
	
	def InsertBlender(self, target, input, colour, value, loc=(20,20), name=None, collapsed=True):
		
		mode = self.GetMode(); self.SetMode(True)
		
		originalNode = target.InNode()
		col = ((1,1,1), (0,0,0))[originalNode == None]
		
		if input == 0:
			n01 = self.AddNode('blender', name=name, loc=loc, collapsed=collapsed, Input_2=col, Input_1=colour, Blending=value)
			if originalNode:
				self.ConnectNodes(originalNode, n01, 'Input_2')
		else:
			n01 = self.AddNode('blender', name=name, loc=loc, collapsed=collapsed, Input_1=col, Input_2=colour, Blending=value)
			if originalNode:
				self.ConnectNodes(originalNode, n01, 'Input_1')
			
		self.tree.AttachTreeNodes(target.ItsNode(), target.InternalName(), n01)
		
		self.SetMode(mode)
		return n01
		
	def MapName(self, mapnode):
		map = None
		if mapnode:
			map = (mapnode.Input(0).Value(), None)[mapnode.Input(0).Value() == "" or mapnode.Input(0).Value() == "NO_MAP"]
		return map
		
	def Maps(self):
		return self.mapnodes

	def MapPaths(self):
		return [self.MapName(x) for x in self.Maps()]
		
	def Material(self):
		return self.material
	
	def Name(self):
		return self.material.Name()
	
	def PSInputByName(self, name):
		return self.InputByName(self.root, name)

	def RemoveOrphans(self):
		usedNodes = []
		usedNodes.append(self.root)
		self.Traverse(self.root, usedNodes)
		for node in self.tree.Nodes():
			try:
				index = usedNodes.index(node.InternalName())
			except:
				node.Delete()
		self.UpdatePreview()
		
	def Traverse(self, node, usedNodes, col=0, xy=None):
	
		usedNodes.append(node.InternalName())
		
		if xy:
			y = node.Location()[1]
			node.SetLocation(xy[0]+(col*xy[1]), y)
			nextcol = col + 1
		
		for inp in node.Inputs():
			
			inNode = inp.InNode()
			
			if (inNode != None):
				
				if xy:
					self.Traverse(inNode, usedNodes, nextcol, xy)
				else:
					self.Traverse(inNode, usedNodes)
			
	def Root(self):
		return self.root
	
	def SameAsKdTexture(self, node):
		kd = self.GetMap('color')
		if kd:
			if node.InputByInternalName('Image_Source').Value() == kd.InputByInternalName('Image_Source').Value():
				return True
		return False
		
	def SetBump_Value(self, value):
		self.root.InputByInternalName('Bump').SetFloat(value)
		
	def SetInputValue(self, input, value):
		if input:
			inputType = input.Type()
			if inputType in [0, 4, 6]:
				try: input.SetFloat(value)
				except: pass
			elif inputType in [1, 2]:
				try: 
					r, g, b = value
					input.SetColor(r, g, b)
				except: pass
			elif inputType == 3:
				try: input.SetString(value)
				except: pass
	
	def SetMapPath(self, node, path):
		if node and node.Type() == 'image_map':
			node.Input(0).SetString(path)
	
	def SetMode(self, useExt=True):
		if useExt == True:
			self.useExt = True
		else:
			self.useExt = False
	
	def SetPSValue(self, input, value):
		if self.useExt == True:
			self.SetInputValue(self.InputByName(self.root, input), value)
		else:
			self.SetInputValue(self.root.InputByInternalName(input), value)
	
	def UpdatePreview(self):
		self.tree.UpdatePreview()

# EZDialog
class DlgUI ( wx.Dialog ):
	
	def __init__( self, parent ):
		wx.Dialog.__init__ ( self, parent, id = wx.ID_ANY, title = wx.EmptyString, pos = wx.DefaultPosition, size = wx.Size( 400,400 ), style = wx.DEFAULT_DIALOG_STYLE|wx.RESIZE_BORDER )
		
		self.SetSizeHintsSz( wx.DefaultSize, wx.DefaultSize )
		
		box_Dialog = wx.BoxSizer( wx.VERTICAL )
		
		self.pnl_User = wx.ScrolledWindow( self, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, wx.HSCROLL|wx.VSCROLL )
		self.pnl_User.SetScrollRate( 5, 5 )
		box_Dialog.Add( self.pnl_User, 1, wx.EXPAND |wx.ALL, 5 )
		
		self.m_staticline6 = wx.StaticLine( self, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, wx.LI_HORIZONTAL )
		box_Dialog.Add( self.m_staticline6, 0, wx.EXPAND |wx.ALL, 5 )
		
		self.pnl_Controls = wx.Panel( self, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, wx.TAB_TRAVERSAL )
		box_Controls = wx.BoxSizer( wx.HORIZONTAL )
		
		self.cmd_Defaults = wx.Button( self.pnl_Controls, wx.ID_ANY, u"Reset", wx.DefaultPosition, wx.DefaultSize, 0 )
		box_Controls.Add( self.cmd_Defaults, 0, wx.ALL, 5 )
		
		self.cmd_Cancel = wx.Button( self.pnl_Controls, wx.ID_CANCEL, u"Cancel", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.cmd_Cancel.SetDefault() 
		box_Controls.Add( self.cmd_Cancel, 0, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5 )
		
		self.cmd_OK = wx.Button( self.pnl_Controls, wx.ID_OK, u"OK", wx.DefaultPosition, wx.DefaultSize, 0 )
		box_Controls.Add( self.cmd_OK, 0, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5 )
		
		
		self.pnl_Controls.SetSizer( box_Controls )
		self.pnl_Controls.Layout()
		box_Controls.Fit( self.pnl_Controls )
		box_Dialog.Add( self.pnl_Controls, 0, wx.EXPAND |wx.ALL, 5 )
		
		
		self.SetSizer( box_Dialog )
		self.Layout()
		
		self.Centre( wx.BOTH )
		
		# Connect Events
		self.cmd_Defaults.Bind( wx.EVT_BUTTON, self.Reset )
		self.cmd_Cancel.Bind( wx.EVT_BUTTON, self.Cancel )
		self.cmd_OK.Bind( wx.EVT_BUTTON, self.OK )
	
	def __del__( self ):
		pass
	
	
	# Virtual event handlers, overide them in your derived class
	def Reset( self, event ):
		event.Skip()
	
	def Cancel( self, event ):
		event.Skip()
	
	def OK( self, event ):
		event.Skip()
class EZDialog(DlgUI):
	
	def __init__(self, title="EZDialog", data=None):
		DlgUI.__init__(self, None)
		self.title = title
		self.data = data
		self.controls = {}
		self.categories = []
		self.labelwidth = 150
		self.panel = self.pnl_User
		self.sizer = wx.BoxSizer( wx.VERTICAL )
		self.pnl_User.SetSizer(self.sizer)
		
		# Map control type to classes
		self.ControlTypes = {
			'int' : (EZCtlInt, 'int'),
			'float' : (EZCtlFloat, 'float'),
			'bool' : (EZCtlBool, 'bool'),
			'choice' : (EZCtlChoice, 'int'),
			'select' : (EZCtlSelect, 'str'),
			'iselect' : (EZCtlISelect, 'str'),
			'color' : (EZCtlColour, 'col'),
			'posercolor' : (EZCtlColourP, 'col'),
			'file' : (EZCtlFile, 'str'),
			'button' : (EZCtlButton, 'ignore'),
			'label' : (EZCtlLabel, 'ignore'),
			'text' : (EZCtlText, 'str')
			}
			
		self.SetTitle(self.title)
		
		if data: self.BuildControls(data)
			
	def BuildControls(self, data):

		try:
			controls = {}
			
			NAME = 0; FIELDS = 1
			LABEL, TYPE, RANGE, INIT, TIP, INCREMENT = range(6)
			
			# Remove all previous controls
			self.panel.DestroyChildren()
			
			# Variables used in calculating the scoll area height
			height = 0
			margin = 5
		
			# Add the controls to the UI
			for row in data:
				ctlType, datatype = self.ControlTypes.get(row[FIELDS][TYPE], None)
				if ctlType:
					ctl = ctlType(self, self.panel, row[FIELDS], self.labelwidth)
					if ctl.Successful():
					
						# Add control to UI
						self.sizer.Add(ctl.Sizer(), 0, wx.EXPAND, margin)
						height += (ctl.Height() + margin * 2)
						
						# Add control to our dictionary
						controls[row[NAME]] = ctl
			
			# Update the layout
			self.panel.Layout()

			# Adjust the scroll bars to suit the new layout
			self.panel.SetScrollbars( 10, 10, 0, height / 10)

			# Keep a record of all controls created
			self.controls = controls
			
		except:
			# Remove all previous controls
			self.panel.DestroyChildren()
			ME.StatusBar('Error building controls')

	def Cancel( self, event ):
		self.EndModal(0)
		
	def GetLabelWidth(self):
		return self.labelwidth
	
	def OK(self, event):
		self.EndModal(1)
		
	def Reset(self, event):
		for ctl in self.controls.values():
			ctl.Reset()		
	
	def SetLabelWidth(self, width):
		self.labelwidth = width
	
	def Values(self):
		return {x:y.Value() for x, y in self.controls.items()}

# MyError Class is used for custom exceptions
class MyError(Exception):
	def __init__(self, value):
		self.value = value
	def __str__(self):
		return repr(self.value)

# Embedded versions of PoseWorks' icons
class MyIcons(object):
	def __init__(self):
		self.group = PyEmbeddedImage( 
			"iVBORw0KGgoAAAANSUhEUgAAAIAAAACACAYAAADDPmHLAAAABHNCSVQICAgIfAhkiAAAIABJ"
			"REFUeJztvXmQJcd93/nJzMqqen1M99wDzIVjBhiAAAkSIAgCFMXLIkXwFAlRtmTRlrSxa8nc"
			"iNXGetcRXkfs8ceGFasNx4YVWofXYUlrU6Qs2aJIiuYJkZTEmwRAYDDADDCDmZ6evvvd9erK"
			"/SMrq+r13XNhSCAn3vTrrrsy85u/4/v7/QS1duGFk3ci1R+mafLgYNAnHvRJ4gFplmLynFfb"
			"T1aTSiGlBCBNku8kcfyrb3nnh07V9xHuy7kXn/1t8vx/j6Jeo9NaotttEg8i8iwlz3OMMdf5"
			"9l9tV9KEkEgpUZ6Hp308pTGGfp6n/+ytf+ejv1vuB3DuzDP/Ijf5/9BpLdFcnqffbbPU7DCI"
			"U9I0wxR7iuFLbPFOqn23eMRaF9v6pda70prn2+SJxIZbt9XElp9JrPp17cM2P5mnBJ4SjDU0"
			"XjEQ8jwnjvu/854P/to/ARDPP/ujj0nBH7dbyywvztLptFhq9kjSbPhyl/sytj0AxLYvMXzk"
			"OgduMgA23nyjdf727slTgtFQEfgeQki63S7NduuXPv5f/Y+fEs89/d0LUdQ/uLRwicXFZZZa"
			"3dWX2nhqrN9+Kjp/+JckvUgYZvS6ewmCcMVBBhAE4RP4wYBu+ygm38/WEU2s+nq5s3+tQ8ZC"
			"RRh49Lo9zl+YPr3YVa/x4jg+2G0v0+121uz8TXBxwwtuH/q3u/MWDtkC9G+8WZCbPn7wJO99"
			"9JN0OpI0HZBl8NJLcPaFnwHzGGna4fDRr/Kmh75Mvw/Ly4KZGY8XTj9Alr4P399hzyZE+dns"
			"nq5m5wN0ogztSZSSZHl+jLj1US+OI6KoR6vTv1qXu8x2pVe6vOM3PkoQxy0eeeu/4c47z9Dv"
			"QxBAmkKvB54He/d/gwsXvsn7Hg3Ytz+i1wNjQClDo5Fw08G/5ezZ54lav8nIyE6ktMKZEKKU"
			"0LcH/VfW+oOcUHv4WtNstx+VSTwgSxPiOF39+K9w6DdmwP1v/CQnTpwhSWzHr/VTKcOnPh3R"
			"79u/pSnEsf2ZZRAE8zTb/4pBNCCJY9I0Jc9y8m2r1lcKj4I4M0gp8DyPfrf/ZpmmMVmWkeVm"
			"gwO3cfHVy9hl3OhVOOQqQP/4xJO85p4fE8e2oweD6uP+5j7dLjz+OOQ59PsQRdUnSUDIBS7N"
			"fot+LyIe1AfB6vd+taG//ovJTbEEQS/qH5RZlmLMaj3/SuXea7Pv1Tt+M+g3JuHo0e8QxzlR"
			"VHXqYGC/u9/7fTsYAF58ERYXq+39vh0YUeTQ4jTdbpeo3yeOY9Isw+SmevfXEPpXPnNuDEmS"
			"+t7QDbgdX+HQD5BlMY2RWQaDavb3erZT3c9u137i2M78Vgs6nWpbu233HQyKS8h5up2ufd9C"
			"IKW11Am5llC46QNt3DZT240hy3M8q7pstO8rD/oBpFSE4SKdjh0AcVx1bKdjP93ucAc7VOh0"
			"7GBotezfBgM7qeI4JOv3kFKglIfnaZRXDQK55n1enc5feRZjAAPeVbjcZR59Y0J/1XxaLdDa"
			"dv5gYDvXdXq7Xc32rLCZaW07vdm0P7tduz3P7QCIujvJ4wjP02g/JkgCMp3hae+6Qf/KJuu/"
			"vAr9w+0H338rS0tVh7qfy8urO98YGBuDpaXq0+vZtd9uFzTnbyZJEpIkJk0S6vLX2r6WawD9"
			"K1qFAK9C//A2Y2i1Xkur9ThpWkn/vZ7tfCfYlUcL8H2YmRnueCEAo3j2qbcickGeG/JC7spz"
			"Y22Ha/rZBJAjRI4xq4B6y8+z2Wsuz/wq9K9uI42jnDl9DzsmrCro1Lp+YTOrq/FCCDodQ54P"
			"/30QjfDiqQdIkwBfOwOQsEKgEquunOcxhvMcPvJVRkZbZKlm6sJxet2HkXJHKSxuLDRu5Zlt"
			"W2dovbKhv76LSf8uZ57/U8KRH5Dnw7MeQEo4ffIhus0d7NwzzciOJmGjQ9Qfo98dpzl/EINE"
			"KVW4Zj3rmfO8SgsQtkOTpMf9D/5r7rrrFIOBlTNaLdD+SWZn/4IXTn+EsZGfQUpVag6lNXGb"
			"0O/aGgPgFQ795Z+LWSolo433srx0DCP/liCYAgH97k5azT0szd1MnvpIJWgvH6LdPGwNLcVp"
			"lAdSSJSnCQKfIAjxgwDf12jPQymFEJLcpHzwF/5ndu7sMRjYgWZMIa0DUhp2TP4pF16CfXsf"
			"RkmFUgpjDFJJqituZ0KuiwBbbT+d0A+Us0sphfY8GsER2u1dzM13iPrWmpdlGQLwtO3k0s4v"
			"BSAKodrq+572CIKAkdEGjZEGYRiifW07Twp27/kcO3f2SBIrWDrV05mc0xTy3CDUn7G0eIzx"
			"8V1orfE8BYLSv7DdN7ViALwK/fVmZ65Ca00QBiSF+dZ2qiRLMwymxr5RKCkRRWfY1yRQSuF5"
			"HkEYEDbsAPADH601SirA8O73fIHBwHZ+FFHKHO6nGwiQs9w8jafuIQ8CTOiji2eSUq2hyW3c"
			"agPgVegf2ioEQgqUVGjfJ8gru71UEq01WZqSG1MghcTzPJTnlbPRdka1Tfs+QegThiG+71sj"
			"kJI0Rp4vVUo38+tWR2d76PXsPmn2At3OrWR5DgKEkGihMMIgtzkCLnMJ+OmF/qG9hEAqiWc8"
			"fN8HQEqB9jRJGFu6nDGFZU+VA8Cu6wDFQJB9bj50kl27p+h2x5ibPY6UdyKEBqDXG9Bq2fXe"
			"qZz9fmVxdFbHKCrkApbpdrogQClpyZ9KWe1i84ca+nV7A+AVAv3lIU7KVuD7vp3pUqG1T5al"
			"ZJk14gghUKqQ9B0TVwhyk3PPaz/Dw498B6gMSO32N3npJY+LF95L6L+dOPVYWpJImZOmlbrZ"
			"7VotoN22353JOe6PQt5HeQpPa7T28XSGJxVGmvVVxDX+vvUB8AqB/lWHFihAoaoppdC+Jsuy"
			"0p/vdPtKrRMYk3LPff+ehx56giSpZq9TJRuNlB2Tn+GFMxE7J97GwoJBKbtPFFUGp7rPwRTs"
			"3F5nD54cMNCaIA4tEmU5Rm+fub3NJeCVAf2rjiqQQAqJMQblqdKNa4XAiuYlhR0s4ch3eeCB"
			"J8t13X2cRJ9l9mfY+CKt5j187zvv4tgdXypJJg4BnFvZ+RMW544Q9cYJG5nlcWQZeZ6V1H2H"
			"SGs8xJrPtrUB8AqD/rWa61iDQSLXtN1XXL8ub3zjX5FlhiyrnElubXeOJcca6sf/H1nycZ7+"
			"8fNM7jy7SgU0xhqclhcPMH3uLrRP0dlsLV5jA8Fw8wHwCoX+tc9nSoPL+qZYQRjO0hiZL/0F"
			"zodQl+zdJ00hNyndbhs672Vp/klGJ/4GrfMCJXySOGT6/Al6rV1W6CvsE0pKlJTIGsl0Kybi"
			"erscL8NV3PfqHX8toN+2nDxvcuSWrxJFKTMzE2Tp7fj6eHX28qXbn9EgwZi4VNscT8ARSOqf"
			"PAcl+3SiJpIRjLmD+fZtxMmALOuSZwMGkY8x1uCklIfva3zfx/f9whjkIZVdolY/+sbPvvEA"
			"eIVDf5anHD7yWd75ri+xvGyl8aO3WFfvmTPjdFufoBEeKO3yQlgBMB4cHIJxRyVzne4EO+dX"
			"yLKQqAeeF5dahPZ8pFCkJiUIMgw5QsiaQSkkbFizsvI8VM0/UL2DzV/C+gPgFQ79eZ5x5Oh/"
			"5m1v/xpRZGdqXaALwzYLC79Dlv1PhMHOouNASEEQXmBmRiCEWbUEOMneSfVCwNL8EZIkKbUM"
			"q955mNyQ+VbQMzg2r8YPAxqFRTEIg8qkvF0zIJsuAa9c6Pf0eR55y1+VM9l9HD8wScCYhJm5"
			"z7Jv10fRvo+nPcLGaT72d3+f554znD5thbckqdb/lbM/zxXNhYP42qqUUkr84lwgSsJInudl"
			"sGcQBIRhQBiGBEFgDVBKbXv2w3oD4BUO/UJk3HLrN0nTyjBTV8nqdG+pTtJut2iMjBLkir/3"
			"938fITKOHYOnnrLGH6j2r6t0aRrw4qkHMbksrius0ygM8IMATymQolTvpCicU1qjfY3vazyt"
			"8ZQ3HG20Qeev3LJ6ALzCod+2nInJC/R6lQrnbPLtdsUHTBLQOqLb7GCMALmA72flQHn4Yfja"
			"1yiRoN4vve4E507fTx77aC2QhSlZ+9ZlHDYCtNYIVQh2BhACJSWep1DKs86nYuZXUUbba1dG"
			"CNn2vlfv+PpRSi0yGIzhaf+q3JPJNe12nzCspHi3drsB4OQCgH6/CSjGJ5fKbW7Q3H037Npl"
			"4wibzQIFejs48/RDSGE7XnkeWmv8wCcIfPuzcBcrpcpnkgKEjMky0NpD1MLMqhez9dkPKwfA"
			"TxD0S7XIxM6/5g33P87iYsjigmB5eYzZmUfQ6pHyRawdiLnZnRmefOLdPPimfz9EA2s2K3u+"
			"c9Nm6RhRL0OKHksLAQsLlZzgbPm9nj1zllnBLx40ys73POtosp0eEARBOQi0b93FhpyRkR9x"
			"+/EvsWPHIufP76HbmWB25u/gqVsKN7AAsZpituErZR1S6I0O/Ul6gUff+2/YtWueVgvGxwd2"
			"NoplDJ/i7Nmv46vfIvDHLN++dM9ufrMCyDGI/DWcPRuiVFS6Z93sz3M4fhxuuw0mJ7ucfv7L"
			"TF3YxdT52zl9OmR0NCJJKsRYXraqo2MQz0/fjiiMOb5vuQaNRsj4jow9+08xMbGM5zVYWrqP"
			"LJ/g0Ud/h33750gSez4pOywugvKe4NSzD7Nj9DGUp4eoYlsd9JdHCNn2vlfv+Dxf4CO/+H8z"
			"Otq1M7BQzZyalqYQhtNcnPpX7N31j/H9oCBqqHIwrH9HwgEAnjfCc8/+Pfbu/7fEsZ3Ny8tw"
			"4AB8/ON2TU9T6HQMx++YY9/+OQ4ffY65OcPCgj1ffelwnb80f4SotwNPSbT28IOAcMTjne/5"
			"Jo/8zI8RwiLNwgLMzHyZOJaMjfcxxmke9lltBDJMTP4NF17ayYH970KpHFl4JbfKEKq9jesJ"
			"/dvZWId+wx0nPsf4eLdUz5xKVlfT0hS0P83M3A+JolowZr5RRO7qi4+P3sHF8x9m+uJO2m0Y"
			"H4df/3UYHbUDYJgVDFobtLYDZWrKUsSbTdv5SsH0hbu4ePbukiKmfZ/GiM9v/KPP8XPv+TGe"
			"Vw1mpaDRGDAY9PnkJ+1gqj+zcxplGfjh5+h1O0RRRJIkpafSGLNpFw0hwPWD/jUO3orUL9qc"
			"uOuJ0qHinCr14E33gvIclP5P9DrHCRohxhh8NMLTGLHaY1YTf0o2j5SSkcbraC0fotWZ4uMf"
			"/zxBEJFlDH0sX68icAIsLhyg0zxGnvfJM0N7aRdp4pXEESvtB/zMO37EibtmSg/hSq+hcx9/"
			"+tPwgQ9UqqgzLLlB3209z8T4beT5cNYSWQqRa7fLYAS9fFK/MfMkSYYxqx0s9bi8OLadkWUB"
			"re4io/kkGOe2LWjYtRezVg4fIaRlA3keQTDK4V0Zhw5FZceszBPgOq9y8y4we/4hshTLHTQG"
			"Tws8z1LMwjCkMdLgrW97qpzRKwe1Q7c0tVHH589bpFlpVUwSRTxYoiu7pXtaCoEScmOCCNsO"
			"DHm5oN82Q0IUWdx1wZoriRPdrn1p7rhB1ESKoLSiKWV5eKL0m6++eMkHVKpcp19z76lyxjsp"
			"f8ilWwsfj2OQMkH6cyTxHoSU6ELoc+zgxkiDAwdnCYK0lGUckrlzr/QgdjoWEdptK1S223Zf"
			"YxSDvoa8V5FWPI9cZUizsYl4+wjwckB/0bLkMHGcD/nW+/1h40xdP4/6DXpdhSDC017hPfPw"
			"cq+cGWKdywlhCaGe1gRhzu49lXvXwbS7Dzfw3P24Abh3/wtMdfeUa77vW1UvDENGxgd84r/7"
			"M6anbaqZOhfQOYzc+QYDKxMsL1vZw8UpdjoW6frdvXS7GmMitNYkYUKapmitN+ULbGMAvPwG"
			"HykDvvPtezly9KnSy+ZULffCXOfnOXhqD0ovghyQxD5pmpKlGbnOUUauCf3Vtax87OFhfEOr"
			"2aDbHWbuOu3AOXdWItBtx6Z52zu+wLe+eQezF+8rPHgNGo2QRz/4WXo925luADjod+Hlzab9"
			"OEKIC0V3toU0tUvC8uJB0jgl8WKSJCEvhcB8VfT/yrY9PsDLBP1lMzA3+wFGx54qZ2DdweI6"
			"3xhrffvoR0/R653i0iV49uR+pi+8jyA4SpZllr69Hn3KXb8wsXqeZurCrew/cL6QLSq7vuss"
			"NwgccTPPYe9emJgY8MY3P8XJZ6aYm/4IYRgQhAH7D0zh+1bd8zy7v3seN6Cd5RBs6PnCgr1G"
			"HeWW5g/Rbe7B9wuKmskrupqxDKaN2mWRQrd/yJVBf701wglOnfwAu/Z8Zohi5fTsLLOd/8EP"
			"2t+TxM6ePXtnSLP/l7npT+Dpm6wsIASC9Q0nosjiYRQsLd1Ls/l1oCJu1vX8OsEDbIfdfHOl"
			"u+/avUi7/ecI8w9RKqbdViiVsnevdRopVS0rjgXshFkpYX7mCINoBiEzPC8lTQKmzt1Dt7UX"
			"pSpDlxRFsolCk1nfNmjbFgbAyw/9ZZO2Uxr+G7l4fpI0/yxBuFhE5AqUMhw4AO97H+zYUeng"
			"Tp0CyOW/Jer/trWxC/CUZdMAa+buc4GbjeAYz526lX37XyTPKyHNMXadOga20+66y6aUcwEf"
			"eQ5BOEt76Sx+cCvLyxoYIATcdBP84AeV6ldXK5WCZ370dtKBj6dvx9MpaRqQZz4CgaeK6KWC"
			"JVRnCImCx7jRm90aH+Dlhv7ijwI7wqVSNILDtFu/wsLSIr1+m527pvhHn/gBR4/mJYTWyZhu"
			"3da6S7M5UwZW4tt1fiNChdMKlhf/AZ3O7xE2ZoYkfzfIwHbawYPWVNzrDQuMWQaobxIPjvLM"
			"04e5+zUnSxXyzjutw2hhoQoKFQLarV2kgwApBcaMkKWWB+gpUVLWtdaWGxCG+IGPpzVKbfxM"
			"rm2LFLrVdi2g3+0mZWFI0ZJ77vsRtx97jkOHL9Hvw+7ddtY7eF5pJ7AqE0TJk/S6k0OGG0+s"
			"8KmvuCchBYEO6bR/jUvTXyRo/Ig8z8rOiiKYmIDbb4d77hlW5+raiTHQ7bQ589y7CBtnaTT6"
			"pTyRZTbJhHMeCQEvnX69jTlwkUeF91AWhNAqdtHaFcJGwxJKPGWXg6EBsPo9bzAAbiDoLy10"
			"sqBdwc++81Pc9/rnyrQtTh1z2kFdnXJ+fGeoEWKObrdnvWeFf0Cqumu1fheZdbvmRSfoBoF6"
			"J+3519Ptv8SxEz/g5oPzHDgA+/bZTnNyQT2TmBt8kNKPIvw84Eff/zB79n0eTy+XKOEGVJY2"
			"OPXMm8lT25me1uiSCOoVYWiWF1D6FMKQRsPFHXorOIJr98fGfIAbBPrLr0WHHb/zz3nD/c+V"
			"qpDz2QvB0PrspHP30wmK3c4BfNkrgjZVmbDBSFN6RwwDJia/wo4dFxFiiWggOPvCwyh5Au1r"
			"gmCUJDnC3KWMD334KyXtywmmjkW0vFwNTCmh0zxAGg0wOWh/nJmpD5Kbc3j+GXSwRBI3WJo7"
			"Qqe5C5N7Zec7+0HYqCKLPc/DK/kE3tAAKUPUNmlbIoVutV0z6K/tK0TE29/x16v8Ac5dq1QV"
			"XetStS0vVwJalvoMemMYb1CQMAKCICvg3LNSt/cdHnvs3wGmNLhYI8wZzr7YoNv9b9G+XXf7"
			"nYO8cEYTNpJSPazzB+rSvBDQbe9BkpYBnZ4XIvLjRO2jtBYS4oHV5ZU0CIXlCxSd3xgZYWR0"
			"hCCwRFBVDAAXJ+B5qhacuvnshyvJELJJR12Vo8TqrUJeGrICOhNpr2c7SSk70+vp3KKIkigz"
			"e+k4g56PHElJEmsYyvKsTNwUNL7NY7/4B6Vptu5+NQZGx/rMzf9LjPkttO9jzDhf+8ojvOnh"
			"x0tTsTPWtFqVqVgIaC7tJ+6P4Ad5wfGzOXvxKAJLLb3b85RNBiFEgTbWdNwYGSmTS3hFcohS"
			"7StC1+r8hzXf7Ipf1+YD3GDQX295PmBpSRDHphSw3MfJA0IMe9WsZw+mzp1gae4w2nMhVUV6"
			"tiLEKst6vPVnP1N64Oou1ziupH2tI1rx92noB/ADnyS+hx98b46bDz095CNwA0IIaC3v4/yZ"
			"16O9uoHJUsGUUuTG4Kcp2tcksV+ygB38Oxp42Ajx/QDtqZIAApQzfkOpf41NG5JCt9quB/S7"
			"NhjsYnHRlOt+3Wq2vFzx7Y1RLM7fTJ555LlmcfZgkcvHqpHKxdQrl6pVcujwE0xMLJVm3jq3"
			"r54sMo6hMfo4Wf8+G8ShNYPuOzn55DEa499E+4s21j9pIGXK+Rfuo9+ZLDyMqpTc3TLiaY0Q"
			"kGUZSZKSJgl5booBUPkQgiAo1TxvS2v85u95+xlCruBiWz5qDeh3zdd7OHnyEHv2XBjyBdQ7"
			"Xwi4NHWM+YtHS66chVtZcvCswKStMKWsWhU2Zkt1zXkbV+YGdupalgUMBgsE/u5CF/cQHCVq"
			"7We53ycetInTAXGkbFCHsjkEnMEmCAKCMCQopHYpBLkxdknKstKtW+cPaO2V6/7mlK+Nod+1"
			"NUih22vXC/rLZuDCS79Clv0LkiQfQgGn13fau5ifPlKFdStZvciazhyEYcmtl1Ly3Knj3PWa"
			"Lw8Jl84s6wZat1sgTO4RRQIpEnzp43leYXmrwsSIImQjs04ZIfCU5fw7KA/drPYtMmGwrCUX"
			"5g0VxUupIjPY1vmNW3mnG2QK3c55rz30u2YwNMLdvPTCb5CYTyJEuwyhFgI67d2ce/4+BPZl"
			"qULF077G1xod+IU6FTIy0iAIG4WBRWLyPVy6pICs1MvrCNNqVdrEYDBOMsiJVVyoYQLtO928"
			"8v0naVLm6PdcVE8h0DmjjWP/AuQmL56zeAM1+/7W+f9bf89XmCZuexfb9KgNoL/aYv/5/mHi"
			"9m+yuHSWQXIJk7fodUbptscRgNKySMqsCXwfv0a5tvAbEISNMgxLSoUSezj9/F527b40FBHk"
			"1ECXIVRKmJ8+gSS2S0pgJT3tefanrnj+josohET7OX7Y55Zbn8b3DUuLd6DUrSgZVh0sVlO4"
			"SgHvct7sJgdd9gC47tDvdhOUSZm09miEB8iSMfpRD5MNCHzbGY5y7QdVJK2DW6dHe57G0zW9"
			"ORdMnf9vEPJ/IY5NaV10qqS7/tRLdxN1xwjDgmRqbN4AJ9yBIAuzkqBpjOHgoSd58M3f4dix"
			"2VoC6m8wNRVwceptaO/daK8BjtK1YqZf1jTbwkGXNQBeDuivNovKH+D7hGFAlqWAQSpFlmYg"
			"wPMUvh8QNqwKFRSBlNrXaO0hVWUts0mdQOaCMBjj1MlfRft/jmF5SI/Pc8mlqTtZmjmCp+yt"
			"utlZTwUnpcRg8DOfPM/Yf+DrfOSxL5Q2ArAcAM+zMQ279/wXzpx+iV2Tv46nLX3NFOnnth/U"
			"sr12BUvA9YX+co9Sh1b4vk/eyDFYJ5Hv2zIsAmtB04FPWCCAX8x+6yqtePPOvCyAXFinz2jj"
			"OK3m36fbewLlPUecCHqdSVpLexn0R6uSrMU6X6aHK+QNT7v13GDMGR59/1eAyqjkDEvORS0E"
			"jI6d5OL0V9m/7x2FaVqB8opchZex7m+xe7Y9AK4E+rd/wjV2d4RNz8Mv8+jauPksSWzyRCjh"
			"2IVdWcj3ihm/thrlCBVKKYJgknjwBjqtO+h1ewyiASZL8X07M3UhV5TnL2zzSluJXQiBIue2"
			"Y9/F95Oh/AKuqlg9LaylsX+edvt+Go3RQjgEjbcmjf1qvdNtDYArhf6N72vrd12igPYsdEtZ"
			"5O4rUrcVf3Oh1M6Naj1+K4QsUV15GMqtkcat4VJK0iTBQHlea56tkjR4XrWkCCHJ8y57954t"
			"WUH1rCF1nkAVByCIOs9i8hNltXbpA8KrBYle2btbue91iwu4UugfOqyYDTaVisaTikznQ1XO"
			"LS1KFkmVXFjY+p3vzuvkiyAIyLO8UC+ttS9JEjA22MLG9FlVMizsCWX9n+L+0ixiZGSh1B7q"
			"RScct6/u1JLS0O3mKNkFROnzlxsmgNwO9K/euOUB8HJD/6rDi/XbCEBKZM2u785ft49vKTC0"
			"WF48T1tjT8N2vqcV8cCvoUEVL1A3z0qlyE2TPBUoNYIndwBxaTyq5xqoxzM49pIQkCYJUb+P"
			"V2g5zu+/Gb//ct/plgbAjQL9azUpBNRy5JYIsAa/b+Wva84nYSt5eHhleJgugjmyLIciO3hJ"
			"0tAenn+O2499i2PHv8NgANPTE8zP7WL64kOcPDnJwYPLpYNprcJT3aJks1IQdUMklt8fxCFZ"
			"kA0h24bPtGFbe99rHhdwNaF/9WlW9+JGMJnlS+R5HylTBBMoNbF6TyEBY20DCEThtXOz35kc"
			"3dKya89XePT9f1EGdhgDu3c38bwm2n+RqSlFHNvAUif8Ob+CyzzmPIbzs4dJ04w0SUmTlCxN"
			"7XXXqO14pdDv2qYD4EaD/ss5UZp2ufX2P+XOEz8kTSOiyIZWnTunuHjhN5icuNeesYgbtN9F"
			"6Sk0yuAZD4wpTbRCSEbGvsmHP/IXpZ2gPvaktDNa64yTJ+HQIft7nSfooogsBSxg5vzd6EKW"
			"qfsEDLWl7cpexapdNxwANzL0iy2eIkn6PPCmf81ddz1Xzjynf4+NZYxP/D+cn3o3B/a9p8gh"
			"sLq6t5t9Q8uLWOaxj/3xkE7v/P/uu/NPGANnzlgUcNwCp/oB9Do7eenMG8B4lV1CVpT01Uh3"
			"9YxD1ywu4HpDf27aaH2JqB8g5W6UGgUER275Evfe+1xZ39ehuPtICZ7+L8zM3MHuXUeLBExq"
			"lbEIhpeXkbHvlut6PVzMkUjq6p6Udp+nf/QuwpGXCBpLYKDbmaTX2UmvtbsQPit+olVdvVX3"
			"cCXQv9au6w6AnxToT9M2+2/6Io+85SssL0v6/ZylJXjxhVGWlz7CB9/0+FBJ93rsfT2zSLf/"
			"V4z0PloRK4vzr22ONRy95Un6/eEiD3U3spP4HbkkzwXGRCxcOjGsriLwtCjyA7pEUTZNnGML"
			"ucCVq/fWqrbmANg+9G9nl6sH/UnS5fUP/Dvuvfck3S5onZcq1+hYl0OrtHrFAAAWPklEQVSH"
			"/7As/VoPEnH6dz1gIwifpN16N42RRtk5eh0VMs8zms3GqgGwMqTbVRZPU1DKMBj4QzZ+R/hw"
			"9YZ8rcuaQmHJVfBqg/DqCH71dpXiAjYcMtcI+nMOHfkK9913spS+6+tvHNtULo7BU88nsDLL"
			"hkvc2G7PYsy+ikjiagSsNMUKgZJdeja0YCgfcL2svPuZZUUAaFrl9NPaGo7qkG+pX5as0ij8"
			"F858LVYmgr5c6F9x3PaKR99A0O/pJV772m8PJWReCb2Ok++8cCuDRZwkDpAkI0RRHyV7VZFH"
			"z/L3hKln4QSM4OzZu9i774WSKOp0fMdQrlv4RAFbWRLgaVn4Dyw3QRe0NF0EftgUcdbA5KqK"
			"bT/V3TpvbY1TrJMj6MaGfoBWq42n20Ol1dyM6/dth5w+bbN61SG6Xoen1api+gb9cQYRKBmV"
			"1b2yzLJz17LDD6J3MDv7l2idl/KEm/V1K1+aWiFw9tItJeRrrWk0Clpabb13nsXyexHft5YJ"
			"+2q95yuMC3g5oN+2MEzJsrSc7SuTNHQ6Nq/O88/D5ORwPH+nY+0AjuRhjGJ+5hhkOalOi8re"
			"mS0KlRvSbAktOwwGIcobR8kQJTXf/ttf4fidf4SUZk05wMUjDKJxZs+fQPvWjax9n6ARMlLQ"
			"woLSlFwVnlIb+C82elPr/rbOcVvLEXQDQb9reXaEVkuSZflQIsdms0oZkyQ29v7IYYWns9IO"
			"73LrgO2gqXN3E3UmCILKoWQDRRa59diXeO3rfkyvl5EkhuVlOHlygk731xgN7+LHT/wCO/d8"
			"FqUGQ0keHLL0upOcfe6B0r7geQq/xkt0TCXPK2wQNf7f5cL+qre2wWk2TxR5g0F/+Xeh+Ju/"
			"vp9jx79bygBuACwt2U5w+vdfffl+du6ZYcfOS4DteSmh193B7PQxOov78DwraJXcPLXEL/3y"
			"H7Fnb6eMK3QhXnv3Nsnz/4uzL/wSvrqTS+d3kGQ/RnpTKK+J73fptvcwP3sLneZuJBKlh/kJ"
			"QSEHlClha1Tvq6Hzb7VdZlzAywf9ALlZtJE36XuYn/8uWTacytVJ5zYa6C7aSzvpLO/ED27D"
			"CyKkzIj6O8gzBblCeZZb4BVkTu0bPvKxP2P3ns5Q3D8Ms3iCxh/TWf4tMOOk0X30+3eQxD0G"
			"A0OWZWBAq6rSqF8T8IZIJGsGeVwlg8+q41YI0xud9kaDfqme5zX3/glhuESnnTG/ILl0KaDX"
			"G5RqXpJQmFAlZ8+8jvb8vgJ+BXk+RjoYRwiJErayt1XLRJVkIQjYvW+GQ4fmSgnfWfScTu9M"
			"vsZAmn8Rkb/bpoLTPhiBkJUTBwFKVrDfaDTK0rGVinfVXtq2LcZXFB18vaDfGMOOyb/kfe//"
			"bBl4CZDUpO+ZS7sZRBNASrc9Sb8zRtTdUVX0rOXYd7PNvXgnmYcNG4H7utc/A+RDqVnrxZyd"
			"epdl4Afn6SxHCFTJN9RZVgV3lAxlv4xFaDRCtKsdvOYAuFbQv3rfyy4ceT2hX/tP8sEPfbac"
			"ic4B4+L9fR8aIy3mLrydeGCjfo3JLbSXfvvq40q7OmeL4/gFge2kKBJD2TtXVgyph4jlRpJn"
			"HaQYL3PzuJt3SOSqjQYFQTWomXmvZudvB/pd26BkzNZOsNXNW2+rT/To+/9gCI7r+YDz3Dlc"
			"Eib2fou5Cw+glARsmLXLAeBs7E7gqqNAGalbROkszN9Er/f0kH6/suafMwPnmU8yEOjAIJUs"
			"rXeqiN6VhVvZ6ff1Mi8lJf1qv7Vt9MVlRQdfT6k/yweMjUVD+XPrmcHraeKCcBmEKXh9Rdl3"
			"V1wpdIEhGuklCHyk0giwuftrqWQX5u9mcfErGGOGyri6bCMu7l8I6Lb3EccG5WWADf8KG2Ep"
			"4JXqX7kUeRukrr9+0O/atgtHXg/oF/IJ9u1/gqO3fJtWy/D1r9uUa/v3U0r8Tqd3Zl1jIGyk"
			"HL3tAvOzB/BUiPJCGg2bVGF8osWe/U9z/wOPs7g4ztSFvfR7++m034KndlojTNExDXkLL7ww"
			"xsREuwxDdwOg2axsCMZIFmZvxffSMg29kyeCMCyXG1mL66v/XO8db/4qrxz6XdtCdPD1g/4s"
			"S7j9jv/AA2/8dmm1c4aVF1+0ufTe8Abb2S5eb2kJXv96m5dvfDxibu77XLqkmbowwvOn7iVP"
			"7mffTS/yy7/6Fyg1oNUCrRcIwwUWF5/l4sW/Zm7mo0zseBBRygWS82c/wdzo7zEyslxqGM53"
			"4ChcZ5+/nzzRGM/VdKoVfyri/2VR2pVi+9WU+O1bW++XrbVtRQdfS+jHZNx86C9588PfXlUZ"
			"A6ygJwR8+cvwxjfamdjrwc//vM3J56yBngc7diTE+5r0+t9k756zfPBDF0rXrxCWmuUoW2GY"
			"ovSnmZufZO/u4wjpCkNPMHfpN0jSLyDkaXITY0yOlNBcupm56duJ+2Noba13qgxDl2XiCQv9"
			"G+frX/PdXQfod23LhSOvBfTneYSQi/R7yygtePgtXy396+7jfq8XZzh1ynb0sWNw4sRwjuD6"
			"Z8cO+Pn3XlhF23Lnc0JkluVk5vfp9v43fN+3dPPcoL1xksHP025O0Wq1gSbt5jhpasAUCSe0"
			"c97oisBRzHq5pdn+8kC/a5fPB7iCCR/HPU7c/R/Yu+95wrDD/LxbO7Oh0Cknhbuii27tbTZh"
			"bAzutVzOoTz+7tg4thk7lVrNBlpp2LHnkMzPPcPOnXfaen2lduDj6300glH6vQkCf4CS1gok"
			"i/jEoBFa405h3VOFXf9qw329XSn0u7bNuIDtb13Z4rjJu9/7u+zePV+GXhsDk5NZ2Ul1Kd8l"
			"WXR6uY3SFUxMGEZHhws015k/UQRhuHog1bOG1tPISpkzSC7R6x62lTsLJNC+JsxsyRkhJV7k"
			"kWapjRDybIRQGBYhYkFYooDa0gB4+aDftU0LR15N6M/zlAce/CNuvnm+zO3vZq9zo7oBUA+g"
			"cNK+y84RD8Y4e/oWsuypMuauTvdyner8Ao4Q4jQHJ9W77xUKdOgVURpSyJKsIWq6fFwkfQAb"
			"kexiCIOwqPbp66uaw2etjVcD+l3bPh/gSuBGT/Ha150sZ2UdjjsdK5hBlZrFBU44Vc9Z/hbn"
			"b2LQ2VOqaG7A1A02UQRzc9VxKxHAcQecVK8URL0Rci8qk0+4pEyuBIv2/ZIrAFWAqvZ9m36m"
			"SNF6te379Xa1oN+1LcYFbH/rypYkEbfd8rUhu3q90pcrt+YKQjnmTt3wApBlms7yXuJ+wNSU"
			"JgyTIcRwaWE7HZt5+9Zb7bkdZasekuWWAtdXUXcHhAMS385yR+D0PIUxVq3LUmvnh2G1r0zk"
			"PJRzeIvv7jpC/8o91uUDXG2p/6FHfo+77z7D0tIwM9fNRlfo4dw5u79j9jioLtOnN3eTDEaQ"
			"UvJnn76bdz/6ROmrr+cMbLft3555Bo4cqZaJlTWGsqygbE0fI00Umc5sXp8srzlzrI3fVfHO"
			"czcAKC19SqrS4HOtOv9qQr9rW+cDrLF5udlhZCTE1xvLkjt3f4G77z5TSvHOo+esa52OLbK4"
			"b58V3J5/fjiwwnU+wNLcUQRW0HrxuVt4/vgZJnd2huz0DjGkhKee3M2lacHhI/OlraDbrfL8"
			"KwVzM7cwN3VHWXal/tBu/XcFJkDYzKJUTOGqWseNLfWvddgmcQFrbzUGzk/NkOWKhaUeIw2P"
			"XZNj6GIgrGS2vOH+rw2pZ07Qc+nXXJbPs2ftADh40FbedFK/GwDzs0eJOpMoT5YZsv/28fez"
			"9+B32b33dCnouUobzaV9TJ19Deee9Xl25yLH7voeUpri+j5ZGnDxpbvptSdRSlQzuYBxKcWw"
			"GXcDx83W+uTGgX7XtkYLXwH90zPzGDTGZAgh6EcZF6aX8RTs2jlKGPglt02pPmHYHyqg4L7X"
			"CyzHCTz9NOw/4CJphvPoNJcOMHv+DoRUQ+5V7Ws6Sz9Lc/5BlD5LlveJ+gH97ihJZM2HyhNE"
			"nf08/b33Fc6fFFRKGtuMXp4nSm+gzR7ql9nDXDqZl7PzrwX0u7btiiHtTmSLN4rhcFUhBFkO"
			"cws9At1jYnyEIPCJky6djsIPMwY1xmy93FqSgGkDfwMXpqH1M/toHFomGTRot/bQ707QWdxb"
			"Jmdy2cGC0DJqhZRkaUAc3002GJDHAyQJfmCqAEtnk5cWUnJj8EJqgp7N4hk2GoRhgC7zB8oy"
			"IfPL0a4V9Lu2jdrBduuBm25iqdknSZJ1hZ1BArOLXXyvx/hYSrOlmbgU0x2Hpgd9V3CpA70Y"
			"4gwm/xSS05ACc7uP0pt9PVWeX/C0rJg1YVCwdyyvXkpJlmV4A13SqpMkKRI0DodgYcP/h+Pz"
			"hKWEubTslqod4Hm6Kr50GS933b1uAOh3bWNa+BpS/9zsDG9605v4xje+sam02x94vPa+b7N3"
			"T5dLZ+HOh6AfwOIROHcHLNwEfhv2fhEG5yxfVwHRrslS+HIOFZt6VZesnZGR0TJBk5SSPM9L"
			"H7zL52Nz7lPk2ZVIz8K5MaZIuuDy9hdZuQOfIAiLlOxVSrn1nvMnGfpd2zYnUCvBkSNHGB8f"
			"p+tym6zRjBFkmeT221/kwAEYnYSlP4BdH4P+9+HA9yEEWsAckGBdqgZQWUbuOtPzylQsZenV"
			"RsOSK4vyKTaxYl66YJM4KPV4sBY75VX6uYFyEGBMWSfYOXRc+tj1OXvXvl1r6Hdt2/kBsixn"
			"fn6eBx54gK9+9avrpi/z/QE/93P/mcOHnwNgPITxD0H6eYjfZWf7AOhiO99m36mSJJfVsFx+"
			"39CGUNm8+TbTdhk8WejoNkmjRxpk5C6lC4UkXzPQ2FoRtvOdjd+mn3VVuSzibOTQ+UmHftc2"
			"4QSuPo2QgjzPmJiYYN++fSwsLKzaR8qcd73rCzzwwA9LvXzHjuKC74SDp2HursI7h/24zs+F"
			"QCiFVB6+r2mMhOVsd0UT3Ez3vIJYWQhpuSuglOUlvIOoIm2KwhC4AYApVUxRL71SqHw/zdDv"
			"2rY5gUpJ8tyQpikPPvggn/nMZ4oEyVXbs2eO173uqbICprPKaV04dHbBLWfg4t0Qd6rZD5B6"
			"Hl6aIWsI4OrlBCWfXlUqWq00WpnWRVXpXKze52wSVdbtlUmXhow6Umz+Iq5Ru17Q79oGlNT1"
			"TzNzaZosszaAEydOrHqZzeYkg4E3lBHTlVVbWIAXnofvPQ29fwmDW8GICgEQgnR0ZEjlc9Bv"
			"Cyz4aF04XWqw7jrfQbhbw10MfgnvznTrgjBd+RhZt+Gv/+w/8dC/AtUuKyd5r9cmjm15MzcA"
			"6oOg1xvha197BzCcKaNuDIoj6AMLvwnLvwLZEcikZPnQQbteC1EGdVg+f1ESdUXHb/is8jI4"
			"eBvs/xMP/WucYh1O4EYvwb5UJ2UbY3jooYf41re+VYu0yfjhDx9k9+4zHDjwLJ2OPTZNhytt"
			"dzsw6ALHIL5T8uzjb0OEGl/KMi5eeUW5FGW5d1vm0a/xPJvPjq2d+lq06wP9q/dY421uchoB"
			"reYyIIoY+ozJyUkefPDBIRQQAj7/+V/j7Nlb6PVsrH6dW1/PzoGBxUuHyAONcHVxa3HyLpLn"
			"8l7MFjt/k71udOjf/BRrn+OywlI8T5IW1a2yLCOKIhqNBvfcc08Ncg1B0OMb3/h1zp17LRcu"
			"VE6fZrMq8uxat3lfuQ6XKVocIcPFzW8Vzi/nfd2w0L+1drljbTuZh+wewvL30zQmy6zvPEkS"
			"BoMBIyMjnDhxYvgCMuPFF3+B6elHmJujLJdSzn4A3sfRYw8hlEQpa/hxThnHytmyQeanDvqv"
			"hs6//h5yKzutPp2k1+uRpunQJ0kStNacOHFiVWc1mz/HSy/91ywt7S9mvgQ08IvAW4CEA0du"
			"KwxAXplIwdPDsXxbb69Cvz3Fxue4rJIxNpuGKCti1T8uqfJtt93GuXPnSgIlGJLkZuA3scbf"
			"JeA27CCILDN4127ai3NFjb/hQg9bkuhfhf5tbLRt2zKAEAJPKdI0GZr57uN+z7KMAwcO2OLI"
			"ZXPa/h7gONb1kw+df//hW6wVriBj1GlWGw6AnzDoH2ovA/S7dllCoMGwtLhIlmXEcVx2ehzH"
			"Q58kSdi5cydBEGz53H4QkktdIIpL1rRWuvT12k8G9G9iwdjS1Tc+xdbOsf2iUcVMTBLbwZYk"
			"ObwErPweBAF5ntuSK1tou/fdRGfxEo0kIUtT69hR3vplU16F/m1sHG6XVzdQiHIJWNnZ9QFQ"
			"3+ZUPBdGvVEzxoAXMhgMaoKgVzprhtO2Dt3Z6j+t+QDbfeJr1F5G6HftMgtHCvIsKdXAlZ2+"
			"1iAASnv9ViB90O8jM4nvByXRQ25YOPlV6Len2Pwc9foH3vbWV3cNgckr4W9lp7tz1ilXjmM/"
			"dI51rh31+/Q7TeToCEE8II59tJ+WWbWqk2zrtt2F19+0tRNs46AbF/rTNCXLczxTzs4qFn8r"
			"tyCkpNPplMQLFyChtc9gEJV8PKVU+T1N08plC0ODpd7mp6fI4oTET4hju9RkaVET0AzdxvA9"
			"be3WX/72MkK/ANI0o9ePiJOk4+UmL4Masi2MAAEIKZjYMULUj+hEXdqtDu1Wi6jfp9fpgpBl"
			"3rwkSZBSkqQJYWhz8WvtE0URYRgyGESMjo2TFZy+qN8j6rTRWmNyQ55l5FlObiqSxzCavAr9"
			"9hRbfAIBcRzTbLaI+tGU52anryX9QbbJbVYXcZ45Tyk8ZePnlLRZsqynMCcZWGN/lmdIIIl6"
			"AKSDPsZAL7GhQq3FQQU/zg08VIVzRTe/Cv3b2DjclDS0Wz1mZhfo9aPve2lioy4bvtpkAFQv"
			"35ZTK4iUnmXThkmIyW2qtDTNy9Kn4Hh+w5WvTPGfcX801Z5SVeVTKgHQBWisrp/zKvRv/QGl"
			"ibk0M8f5qWniQfo5r9nqnPaUPKZ9zVjDo9NP1zxQrLhOGTOvPYLAJ89yW/cmCazuXncNG1Pr"
			"e1NMdptmpV4SzRibWtVV1LBMIJtVs0zCWN7Dq9C/3VP4KmNhfolnnj3DpZn5FzOj/qPXbrb+"
			"WZxlfzw2OsLEjnEmxzSdfkaaVTNYrHyoQhXzlML4PlDU0w180nRlpUtTzm5T72iGv9fLvooi"
			"JLue5NH5BX56GD5ba5tfbvMTKwlapiwtLvHDJ57hiadOIjD/9JlnnokFwD//p//4/wR+e8+u"
			"nezft4fR0RF6g5Q0gzTLWbNMW9HJWZaRpdmQpO7y5CIoIsjMkIZRIkDtu6ktAfXsnVUSJscM"
			"Umx59m/ygq7bALhW0L/JZFASlDCQRVycnuUHTzzN937wFMbwu089feq/h8IQdPL0zD+ZHJOL"
			"F6dn//m+2Xn/5gP72LlzgtGRBlr76xpfquIKua2uYQymGABlc/A/tP6bVet/uRSIYnkRBVff"
			"W4u0+WqzbW2tzRhDkqZE0YDF5RYvnrvAD594hulLM7Ex4n89cfep/+Opp+2+Q2/zlx/7wJ29"
			"qP+HnvYenNgxzuSOcUZGGxu++JVVNTcyKlWbzNCP6qsLC6nUzfrge7Xzt9ayLKPX7bO41GRq"
			"eobpS7MY+I7Ks3/4o2dOP1Pfd9Ubfeyxx9TC7IUPN5vtjyZJfAwh7r9udz6cm+HVdqXNmO8j"
			"eNHk4lMnXnPqP/3Jn7BKzfv/AajJm15NDPu6AAAAAElFTkSuQmCC"
			)
		self.emitter = PyEmbeddedImage( 
			"iVBORw0KGgoAAAANSUhEUgAAAIAAAACACAYAAADDPmHLAAAABHNCSVQICAgIfAhkiAAAIABJ"
			"REFUeJztndmPJMed3z8ReVVVV3X3XCRnKJLDMZvkDA9LIveyZHmxogx6JetaCbaOlbHG+h8w"
			"4DUMLPxg+MHwAnrwi5/8otXCItayZRjGYgEbhmAsIOtYSeCS4iHxmiFnuqevqq4zjwg/5FGZ"
			"WZlVWUfPNKX+AtWdR0RkZP5+8Y1f/PIXkYIUbrzxs8eQxjd83/v10WiAOxrguSP8wEcrxSne"
			"X5CGgZQSAN/zvu+57tc++vHPvppOI+KNt9985Z+j1L8dDvv1bueAXq+NOxqiAh+lFFrrO1z9"
			"UywDISRSSgzTxLRsTMNCawZK+X/8sU984etJOoC3f/Hyv1da/Ytu54D24S6D3hEH7S4j18f3"
			"A3SUUmQvUbEm47QVcxRdrPqlyq5UWN6MOxJTz84FUfmexMRucbbZhZmGwDQEzbqFGSmCUgrX"
			"HfzJ85/5p38EIF5/5Sf/SAq+ddQ55HB/h263w0G7j+cH2cst+jDmVgAx9yWyOUsyzlCA6adP"
			"mvDnq5NpCNZqBo5tIoSk1+vRPur843/yz/7lC+K1l35wYzgc3H+wd4v9/UMOOr3JS01vGuX4"
			"pRD+ghVKcs5zT2Jic9HWX5SlWTOoOSb9Xp/rN27+fL9nPCFd172/d3RIr9ctFP4MXpx6wfmp"
			"f97EFbJUoP7pp5dr/dWLmEy0SuEDdIcBQaAwDEmg1CO4B1+QrjtkOOzT6Q5WdbkFseyVFss/"
			"PddJpP7lMBgpDMPEtizaR0eflJ47IvA9XNefrNavPPUvDnEsIlyWHgVuoJFSYJomg97gt6Tv"
			"uwRBQKD0lIxzXHyyG1ugoivI8n6l/iWs/ulZwh2tNEIIhID+cHC/DAIfrSfH+cuS3/GkXV3+"
			"XzXqz19NaY3n+bbUSk8K/5T6F8ZJpf4JaE2gFBJOqf9XgfrzpWgNaJAruNyCuU+pP717p6g/"
			"j4wCnFL/4njfUH8OYwU4pf55qzP35QoT3WHqzyNRgFPqXxzvR+qPMWEDVM8aJzul/tXj+Kk/"
			"RoECnFL/3HgfUn+MEgZY6MorTLu6/KfUPx05BTil/qp4v1N/jJQCnFL/3HgfU3+MBbuAU+p/"
			"31J/ztkznwKcUv/7m/onPX2Yi1zwlPoXSDSF+tVwhOf7eKMR+uY2fqdDX4N++x3kmU0uPv/c"
			"OLXIF7QY9ceorgBzX2Yy7WG7i1KazY0mUs4q65eD+nUQ4Hkeut1BHBzSFQK9u4f59nXc0Qj/"
			"sINxaxt9dITyfLTvR29qwtd0vY//Nr4fBuuE7/EFQook3r/yXRW0fqiqACugfs/3OewMMAyb"
			"bm+P5ppDq1nHMGRUv+jmpl3lhFG/1hrVHyC6XYZ+gH/YxnnvFkG3R9/zcN56B3nYJnBd5FEX"
			"YzRiDVDRzwKC6Kdy2370367XGA1HSCmQ0sAwJVIYKKWQ0qhY+fKHMFsBVkT9t7YPME2HIAgQ"
			"wuSo59Ptd5AiYHNjjZpjIyMlKLyxO0z9WinEcITyPIYjF+fWDsFgQH84on7jPervvsdIKcz9"
			"A8x2hw3XJXrDigA2CIVJdEylfprQ+IqFHafRuW0NKMti0B9gGAambWBhYwLSMBe4q0lU7AKW"
			"o/79gw4aCxWEj2QcgCJRWrK7N8AyB2y06tRqNlqnGGGK9lav4fSzWil6r7xO463rrO3sMBy5"
			"yHaHxu1drM4RRLRsaI1k3Do3yQoVUoIjFHLcskWqFiKVXjBWAkFW+AD9Rh26XSzLxFFOmE44"
			"CKmJHtSMBzP9/HQFWAH1D4cjOl0PwzAnIo/ifWkYBBr22iP0/hGtZo2N1hpSSoQM+7xJCl4d"
			"9Xs3t3n6T/4DEArXIBRAQFYgY7UtFni+laeFXFQtkfufzhfn7UkDfdTFrjkgCJnAtKpN1avQ"
			"eMoVYEXUv327g2GEU5LKkL4Zw6px1A/o9g5otWo01xyMaJLjmBFWS/21H/2EOuARCh9CAaS3"
			"00KJBZUXeJmgZ1WxiP4BtGHgDgaI/gCNxrJMAkeFz3JFczVndAHLUf/2zgFyhvDzUEpFlq5F"
			"tx/Q7hxScww2N9awbQspJCJShtk1nF1/7ftc+uGPsRm3/LxA0gKP+/X0xLky4edpv6gLyPT3"
			"qW0BuLaNF/jgjjAskyAIIuHPvK1KrR/KFGAF1N/rDxiOdNhXLYCYFQzTxgtg+/YRNcdkY72B"
			"ZZlIKUOjUS72OiPG8J13eeD6uwSED6OIzmEs8FkCTW8XIc6T71qKuhrfsfGEQAYBSgXVG/0U"
			"4efPTCrACqhfKc3t3SMM05mr9RdhbCdYuD7s7PWQBGysN1hr1NBaZ8bH89b+3r95mRowothg"
			"WxTTalLEMvljAIFlo6SI6H7cIIWgsnE8C8sFhJSkvbm9txLhp6F1HL4u0MJivz3k5vYh3d6A"
			"wA9QgUqtY1C9/jsf+zvsnTtLlRH1NAMuv12GvLDzhl9aAZQhw1KFQAoR2UHhMUHJCGmO1g95"
			"BVgB9R+2uwTKXKnw89BaI6WBwmC/PeL6e3vsHbbxPD/pJ6suaGG1mvzVH3yV6FFP7bdJbevc"
			"/qx+H6ZTfnooGcNr1CN2C+0eGY+IyryocwofSoJCl7H6O113ntwLY8wIIE2b3kBzc7vNwWGP"
			"4WgUKkIQzFYEDebDD/K//+E/wKpw3TLhzqzvHL+4/NHGeugYEyL0BAqZOMuOqQtYjvoBHnnk"
			"ETzPW7xGCyARsjDo9n1ubXfY3T3E8/3KitD+6G/w+pXLiRLMb00U5ymj+TLBk9r2Go2UQ2w8"
			"+il8J7FA64eCgJBFqT/GeqvJxYsXkxcYdxJjg9GkP9LcvLWLO3LHilDUNQgQUmBYFt///Gc4"
			"atRK35FP6xry52ZR/SwlUIBIhsQhA4yVIV+xxYQP+YkhMxLPKllrTbd7xDPPPINpTnr+7hRC"
			"G0GCrLGz2w6VwPNCYzGnBCHFSkzDxDm3yV985pMZFiiyAWDSBkgfTwt5msDT/oU8lBAEjp1h"
			"AClFJOxj6wKqoPzCWmts2+LcuXNsbW3dFRZI10VrjTAcbu0c0Ov1S5XAMAxMy8SyLbj6GN/9"
			"jWexS8qtagOUDfOKzpUhqNcRgrDvzzBA2mBbvPXD3BNDyqk/Rqd9iOd5XLt2jc3NTYIgKE98"
			"B6CUwq412Tvo0+32cF0XPzVaECLsVw3TwLZtHMfh+sc/xo0L5wvtgSLkKX8aA0w7N1F3y0q6"
			"gLCeMQvEF15O+LAIA8woudM+RGtNrVbjySefnLv444BSCqfW4KgfcHjYDpXA9xOfupACQ4Ys"
			"4NQcGs0m3/3iZzhqNCq5eKsYefmXRCr3Pw/fcZAq9GmIaBQQDv+OdRQwDdUu2Gyu4bphn/vQ"
			"Qw9x+fLlOz4qKILWGsO0GHqSvf1D3JFL4I27AyEFpmFiWRZOzaF+8T7e/MClxEFUdvdlhtw0"
			"C38W/QvAq9fBjKx+mRoBxIJfQeuHygowm/pj+L5Hr9tF67DlPfnkkzQajWN1DFWF1hrTsglw"
			"ODjsMhwO8TwvqwRRV3DmJ3/D3339F9TIGmmzfPn5ll5lBJCpY/TfW6ujTXPsB4h+sUKUYV5e"
			"qM4AFUsOlE+vd4QQ4Ps+rVaLJ5544q7bAjG01hiGgZYOt27vMxoO8VwvDFbRIKTE+d4P+dS3"
			"/zv3as29hKFbRa24qpU/zRYog+fUUv2/yDBAqRt4AVRQgOoXEkJgSiNp/VprXNflypUrd803"
			"UIR4BODUN7h1u81gOMR1PYIgQP/f7/GJb77APYTxASZwb5yv4Fd2fJEuIFOeIZPWL+S49ZM2"
			"AnOYy5CPMEMBqlM/kIQs9/vdMGBSqeT9/tNPP41t2ydm0elwmKhw6k322wN6/T5HP3uN577x"
			"n7mX8K2gH/1vAheo5siZphx5RiCVJg93rRG9+ZPjGAg5dgXnsSgfzGaAOUsWQrB7ewcpZTIW"
			"9zyPs2fP8uijj54YFoihtUY3NxA/fJHn/+N/4qLWyWvhWFABcBa4j2JBzjvkK6L/9L4AvLW1"
			"bBh41WioqZjMO0UBFruQQGAaMqTTSAFiJdja2uLs2bMnxh4A8Go1Lv3oR3z2v36HK6NRIvC8"
			"MAPCINAHCCOHYgWB8hZfdi6NMj5UtgWJ8GVmJJDHItQfY3o8wAI6IARIKXBdN6MASiksy+Kp"
			"p55CCHEiugKvVuPcG2/wO9/5DvdHzBTTfj4qKI7VXyNkApPysf0iDJBGICVGEIwZwIiCYyMC"
			"SCvBsqbg6sYTxA4SQTtyBqXtgHCdepf77ruPy5cv3/WuwHcczr3xBn//m9/k8miEYBzuXdbP"
			"B0Cf8D43mYwhrDo6iFGmBFpK/Fo9UQAZeQEXt/7L8yy+QkjptQSGFJn39TG01gRBwNWrV2k2"
			"m3fNN+DbNptvvcUn/uzPeKTXy0zSyEf8xkLzCcPGRoRKEI8QoFzI0wzEaU/ZN01UzUk5gVI+"
			"gFS6hag/t1scD7Bg64/hODa93hFAhgG01vi+T6PR4Nq1a3dFAXzbpr6/z++88AJb3W4y0SM9"
			"LStP1z7gMlaAITCI/o+iczGqMkBR60/OGUZ2GBgGAYYOoAJFqIyCTJMMsJTww8pppei020lf"
			"n/+5rssDDzzA/ffff0e7gsCyqB8c8Nyf/ilXDw4wyAo/3/fHtJ8Wfiz4+P8QaEe/qn6BPPLH"
			"vHp97ASKJoLKyAaYT/Sz086/QkiFiwkpGI2GhcKPmUBrzRNPPIHjrDZ4tAyBZeF0Onz8G9/g"
			"qVu3MAlpPC34oj7fZdzSYwUoUoId4CbZ+YBVhE8qfQx3rZHy/GUnxMSsMBvTqT9GQVDofJgM"
			"TgkraRoGvu9PKACMbYH19XW2traOfVioDAPpunzk29/mqVu3sMgafEWGn2JM925qO/6f/g0I"
			"lemAsRKURQXlkT8W+gDq4XYyWVYmtsCqqD/GnJPMy8qdzLy/v4sQItP/50cFnudx5coVLly4"
			"cGxKoKVEui4f+9a3+LXXX8chS/lFfX/AuG8fUU7/MRuMojwS6BEqgctYCWYN+/JQhpkRfjIJ"
			"pvIooLowl1wmrvhiQgj6/W7GG5hXhJgFDMPg2rVrSdpVw7MsHvvBD/jNV16hRtjy82P9Itof"
			"MSn8MvpPq66M8u8YBrfPnk0mklZp/UT1icf68bf/4hdCspIjsBr1p+u7EKaVK4Rgfb3FcDgq"
			"tAPyHsJz587x8MMPr94gFALTdfngSy9RZ7qTJ/7lrf085acVIJ5NlLlkdP9vPvshfvqbz/LG"
			"Y1vJnIMYs9R8tLGRBH+m7YCq8/0ylZmBhRRgqh8q0lJ3NMJ1RwAzlcD3fba2tlhfX19pVxBI"
			"yZn9fR6+cSMj/Dz1xzSdb/kuYyMw3eLTtJ9XAC0Eb//aM/QunEcYkuuPbfH244+ipMxEEpUp"
			"gQD8OBw87wOYGQk0fz++RBcw/WJK+eHaODnLv2hEoJTCcRyuXr26UjexVIrDc+d49fHHE/99"
			"0VBPERpxedovMvZiVoiNyPwTufG3n6J7z3mkFBhmGGi6+/QTvPr8c1z/4NOM6vUJRoiRdBVy"
			"7AOIJ4XMLfyKujDnIlEzyk2ilQS2bRNE3x0uE2r6uOd5XLx4kQ984AO8/fbbWFaVeToz6qo1"
			"vmHwl5/6FPdev07z6GjC+tcUCz9mgHzrHxIKP9+vC+DmMx+iffHe5KtchhmGmBmmgWzUGVw4"
			"xztbVzBv7XDm1ddp7twGsiuEuLaNQGVHAPPGAM6RdC4GmEn9ud3uUWdiJFC0Hf+CIGBra2ul"
			"IWSm73Nw7hz/63d/NxF6mvo9sn19GQPESpBeNSSDZ5+l+fHnkJYZhZlb1Oo1Go0GzWaTZqtF"
			"q9Wifu4sbP0ttp/7bd75ex9l7+GHIArxEkBg22hn7AaOJ4RO7wIWdw7NzQBVLhYHhnQ6bS7d"
			"/0DSymM2yG/HCIKAZrPJ1tYWL7744vxVK4E9GvHyBz/IA6+/ztN//de4TNJ+3N/nFSBt+edf"
			"FiV45hn47GfZlBL3/gc52H4Py7ZwajUa9TpOLVzlJLZ33JHLyLZwaw7tB+/n6NFHqL99nY23"
			"3gnp37JSk0HHo4BizEP9kycrK8Cc5SKExLasJC6gKjzP48EHH+TWrVvs7OxgmgvoaL4uWmMG"
			"AX/1/PNcfOMN1g8PC+m+aOhXRPt54YvPfx60JvA8Ltx7kWG/h9QBjmNTa9Sp12uYUZcW+D6u"
			"62IPHVx3hDtycW2b/r33cPTEY5j7h1i2jWWZmIaJIeeYELqAL6dSFzAP9ccHBYL4o5SzRgF5"
			"wxDgscceW2kImQwCeuvrfPfTn2ZgGJm3e0W/NPWnaT/TMX3oQ6HwlYLUCmiXHriMNEws08K2"
			"LBzHoVYLu4O1qDtY32ixvrFBa2Od1vo6zVaTxtmzmFcuU6vXcBwHy7YwLRNpGMlCWVklWP69"
			"wMrnBaSTd7sdgAlPYBXfwObmJg8//PBKh4XWaMRb167xl1/+Mu1GI/H4pQWeb/1FRiMAV68i"
			"Pvc5ogjY5Brh/AOTzQv3oqKvdEopMU0T27bHitBq0lxvsb6xnvxiRVhrrlGv13EcB9OywkWy"
			"RD4aaDnqjzGTX+cvNzwohKBWcxIBxi25yjBPCIHv+1y+fJnt7W0ODw9X0hVAqATXn3ySXqvF"
			"Uy+8gLW3V2j5DwlthHRsQCLmK1cQv/d7YBjhGoI5KKVorLXYHwzwg/CL3VpHymBIDMvEDCws"
			"y8ePGMJ1PTzXJYjenwgpsSwL27YxTRNpyHJhLDFAmMoAi1B/GqZhMBwOASYs/rIRQbyddhM7"
			"jrNaJhgOOXjwQb7/h3/IzuXLmff96Zc7hS9yHn4Y8eUvQ71eKPwYQRBw5twFOkf9aD6il3Rv"
			"hjQi4TrU6nVqjTprzTWarSat9RbNVpNmc416I2IB00yGgyFW90p4pfMC8rmU7zMchp+ln/Za"
			"uGzb933OnDnDs88+S7PZXKmr2ByNcDc2eOVrX2P7ySczXr+ilq8hbPlf+Qo0GlOFH0NrTWP9"
			"LJ3OEaORi+e5KKXRRPMRzWg+ou2EcxIbdeqNBo21NeqNBrV6Ddu2Qz9CogCLU39R0lIFWJT6"
			"kz0hwuXcBJX6/bKf67q0Wi0+/OEPc+7cuZUqgfQ8lG3z7pe+xO5HPpJ4+PLCVzAWfr0OFec6"
			"aq2xLItAWAz6/UgJvGTFEiEEhmGEM5MtCyuanezUHBzHTujfMIylJ4SW5SxUgPmpvySJgOFw"
			"UNn4m2YU2rbN008/zaVLl+YeWk6taxAglKL9yU/S/v3fx3v0UZRtZ63+ixcRX/rSTNovglKK"
			"1vom7e6A4WCIOwq/DaCC8RoFsaEYr1NgmiamZSV9f/xKeFWGXxpTLKvF+pmkl4o09qjT5vyF"
			"+zKzhOIb1lpn1r3PIx1AEud9/PHHcRyHt956K9cvLgGlEErhXbuGf/UqYm8P8frr6O99Dy0l"
			"4qtfhbW1yi1/snjF+pkLHLR3QzrPWfVG7AkUIvM/ixVRfy7fhAIsS/3ZMyJa4XIyQjgWaLju"
			"fVYZYnpMjxjS+S9fvoxlWbzxxhtJ/lVAuGF4pz57Fv2Rj8AHP4jw/aWED+P7sestukc9jNjB"
			"E3n54vPHgWnCh5wCrIz6UwdqNQfXdSu3dBgPFfPH0y7lS5cu4TgOr732Gp7nrUwJgJDmfR8s"
			"K/ytwO7QWuM4NTqHPZzBYMwEmWlfZVg99cdYcl7AVJVBCMFw2C+1/oteCxeFjxWlcV2Xzc1N"
			"rl27RqPROJ6QMqUyTp7li1Osb57joN1lOBjgjUaZ5WqKcTzUH6PaGkFzUn9yWpCMAsqEOc0l"
			"XPTGMH3O8zzq9TqPP/74iViPqAq01qxvnqfdPmI4HOK6buHCVcuiivChykKRi1B/CqGXqzg0"
			"DIqjhcoUpUgJfN/HMAweeeQRzp8/f+KVQGtNOIHSodcbMBqNwtXLCtcxPD7qj5GyAVZH/ePA"
			"EInWPoN+j1qtnrm5/IggjbK+P52/yG64fPkyjuNw8+bN5NhJhNaatdY6B3s72LY58UGMlTp8"
			"igyzFCp8MqbSwdLThiEZjsLYwFjoeYEXKcMiSgBw3333YZomN27cOFbrelkEQcDGmfMcHN4O"
			"RwWJQRjef+UvguVQlfpjVPpkzGJJxlGttm1mPHhlfV08LEynqyL49LkgCDhz5gyGYXDjxo3V"
			"jxBWjFpzk6Oj7pgFpMSSdlZ5V0b9k2nnfDLVqD9/dtDPvg8oMwLzBl8VG6DoXLw41UMPPUSt"
			"VjsRK5QVIXQV24x8wXAY2gMTo4Jjov4YUz4ZU62AWaeFEIxGoQKku4BZjqD0/6Jjs7qC2H38"
			"4IMP8t5779Htdk8kEyilaG1scngQdwVmZkUwaVSr87zUH2PqJ2MqXazC2Wazged5CaXlu4CY"
			"7mKtX4UCxNtSSi5dusT29jbtdvtEKoHWmubGWQ7bBximiWGkZgTp1QeCpjH3hyPnoX4IGSAI"
			"fHzfT4I68u7b9P48gq6qBAD33HMPpmmyv7+f1OukIGwABtJq0Ov2QnvAMJDxGgET7zyWp/4Y"
			"WQVYIfWnobUiCPwkMjbdBcSCz7f+9Pa8As8fixkmNg53d3dP3AhBa0W90aB9sIttDyNX8axw"
			"8MWpP8ZYAY6B+mPYlhUGQizZqhdVABgrWLPZRErJ7u5u9B3jk6MEoav4PIeHtzFMI4wVkDI1"
			"NCybU1SG2WkrfzhyXupPzkRBD8NhH9sOV+FPU/4q6b9KXqUUtVqNe+65h93d3YxtcjKgqbc2"
			"6XSOMM1QAYSQWMJCCzLzQ5eh/hiLxwPM8cyECANDWuubicDzIwI4vtafP6+UwjAMzp8/z8HB"
			"AcPh8MQogdYa07QYapN+bzC2B6KvhSQew3SmJapeyQhclPqBpP+yTCMz7o+xiLNnGQXIb29u"
			"btLpdOj3+ydGCZRSNFvrdA53sazYS2ggRNXQsOr3UaIAy1N/NrnIDPPyXUAaq6D/quXEaLVa"
			"SCnpdrsnSgnWWmfCoaFhIo1xZLBMrxWwIPXHKFCA1VF/Os9o2GetdSZjlOW1eZU0P+/5erQy"
			"V6/XK5y3eDcghMBy6gyHQ4yICQxDopRI/ATLYmoXsAz151Mapky6gLK3f+ntOyX49LZth1/p"
			"6na7J0YJfM8nGHlhoKhpYqbiCSeXjJpd33wKs+z0Kqh/nE9Qc2q4rjvxOblVWffz5inbllKy"
			"trZGv98/EcPEIAjwR0Ms28K2LXzTDH0EE69xjjseYOnnoPC8UAHS/n+4+0LPbwshqNdD6vV9"
			"/64qgVIqXG5naOPWnHDSqArQykTL+ZxZRSkLu4BVUX+SQwhMy2TkTY4CYtwtYZdtQxjNFM9T"
			"vFuIl9PzPQ/f8/Gj8DGlFcbMT1qNUZaiWlj4otSfQhj77i30wicvmLJhZNn5svKnbcf78fuL"
			"u6UEWmtUEISTTP0ApRVKje9DiOVebs32A6yA/eKlzkbDEU5tLXHG3A1Db5HteNbO3Yg39D0P"
			"raJ4Bx0U+lJmYZoIS+YFzFtMtUooFU6uKFo4alVj/OPaBjJeyzsHHU5M1dF/NCT1Wpz6YyQK"
			"cFzUH+YOV72u1Wp4no8Qk9R90hUg3l7lMnZzIRaBzh9YDkvFBFZDqiCt0FolCvx+UoCi/Ts2"
			"OigPBaicrQwV3gUsT/2I2A7QGQpdVpCL5k9jlpCr5j0+ZSjyzSxP/TFmxAQuT/3pImr1Gr2B"
			"iuYLVB8NVN2edqzKYpXxfr6cvHBnXXOVyqBU3vAUK2TnBWMCq2GyIENIBv02hhEGiMQLHyit"
			"M46h+AGGxyDu+OLz+VnEaYUpevizBLmq42X7yyiEUnpaAFgh5rnaitYHKMmdDl4QAtOUnN1o"
			"0O/36Xa7HHSP6HUH9Pv9SAkMTNMKX4GaZhTTH6666dTqWNEqGkEQQLSUuhWtomWYk69K0wJL"
			"K038S6efpkSrVIjl2GEFwheClCVZJSZwfuSpf3xtiWlKTNPAtIzk7ZZtmXiuR+C7uL6LO4iF"
			"QVhZTW4oRKJhUhpJFK3SIWWGSmOHCy7aFoZh4nouhmGGCmbb2NHiS0opEDIMvcotxRKzS1k3"
			"UdRV5bE6ZViFQ2byUElM4Oq1NAlulBLTMLDtcHk0FeiIHUwCpRBEw0MdtVaIFCAc/2ZHEOF+"
			"7C0DjdCgAw83cHGHqSFzUka0Ez98DVqE3YqMpmiZpoVSAX6gMC0T265hmCa27SClxPM8DNPA"
			"suwwjNsMV/XUqftM3/eYfcjYPlqHH9msNKwURMb0lGdcqZAs5o8HqHKJsiJEuESaaVnYdo16"
			"PTQILcvEC3x0ECQCTQSsQiXIKsVYGZK0epwuzgvRQ483UgoDuf9ao7Ui8AICz02Ou94It98b"
			"CymllEKIME6PcBpcoCKBG0a0Sng40cPzvYRlbKeWLP6ko2oJKbAsK1HAdBj4yuzJkoJWs/pi"
			"fI0K4WHxqplOLQp4sC083yfwg7B1aJ2KII4EFi2tFreguPWEadW4e4jyIlJT0HMCT46hS/fj"
			"rkePtSerOBk2is+F60aOFWmEH02LJ64f4UcnI08YRF1L+E2ASPhmGATq+T6GaaF9D8uUEyuF"
			"zhUJMOVkaTzA/BAziwi7gPBDCmH/LbHsaEHp6A1XumXDmCqzrVwlCpAWmtYqpSi5NQiUSpVb"
			"tCgFE8fQGqVDKzx7PK9AGq1FcmySjfJsk7IFEkYDrX18d5wuCDxM00pWDIvZIQ4QrS618hQL"
			"rg9QcolZIQVCRMGN8Rp5EqXNUDg5Wodx/53v75NjudYapRoLJO1ryJSdFXqcpuzHlHPh+XAo"
			"m6SLu654OyX0pJ4q6qbSCojOMJwQYBgmthMas6ZpRUPn7KpiMx761NMr6QLm+ZpdeqaLNiQy"
			"amFpIy18RCLRAF10PqMccZ60MEWcadzaUoLP5NWpFy6JgNLKk7NLChR17OLWyWqgcXlKpRSQ"
			"MRuNha9QidJETKUUEE4Otazw2wPhqqFm9Bm58UcmpghmJlagALOpfyItKUUoSZe2jMus5Ikh"
			"VqIYIR2LXJox7cYKk77WWEApjZtgiGhnbJjG+VMMNFYalSozZqUxq8Qm75iZAAAC/ElEQVT0"
			"n2YMFbFhSF46GjWZkS/EifweVT8iOTvF0gpQhfpTKTO707Llh1LVixfJ35lKlHcPpxQg5A+d"
			"VRY9VpusnkQ7oshYzDNFuC3S3UhY2VQ3kq2vNEKfiWWZUTcw/rBk+fOoxspLKcCCHzJdnrcq"
			"ZpmqRFMeUHxmgmEy+1lnUPp1baaLIrQR0kqUL2+a/RPXXcpweGlII/mU3MwbqIAlFGAx6i8/"
			"UrmwKVnE5KElLpNXmmlTtMvKjLuJ9H6arLIMFHVcmS5q7FlNFo2YOTuo+nNcWAGOi/oXR0Xh"
			"z0i1UP6pTJyN2cu03Il8WUWZLKvCPVag/rRdY2aMm4o46dQ/Pc+dE37+5Dze9krrAcx1Mgvf"
			"9wmUQsbDneoux19+6j92FNZjFTZ9tRS+H9AfDHE9ryuVVuMJhxVwSv1VM01p/QtefXoRFe9A"
			"gOu6tNsdhoPhuzKO0LWt2fHlp9RfNdPi1F/x6nOczMKQmm6vz/bOHv3B8EfS91y01tTtWStT"
			"nlL/0riL1B9Dapdb27e5/u5N3JH/P2W70/15r9cnUAHN+owg4VPqr5DpTlN/9aS2EbC3d8DL"
			"r/yCW9u7bw7c4L/Io3bnj7dv73F42EGi2GxamLnFCU+pv2qmu0H9sws2JNRMn/bBPj/+6cv8"
			"9MWfIdD/6uWXX3aN//EX/+elZz/85PrBYfu3PM/HsW1aa07yQiZ8HVq1GVdt/XeS+k+AAhwX"
			"9c8w/AwJltRINeTdd2/x/374E773/R+jNV9/8aXXvg6RI+hnP9/+o82m3H/v5s6/vmdn1750"
			"3z2cObPBWqOOZdkV16U5xZ1H+Usyz/cZDkfsH3Z48+0b/PinL3Pz1rartfg3j1979d+9+FKY"
			"NiPVr3zx04/1h4NvmJb56xvrLTbXWzTW6qv7Otcp7giCIKDfG7B/0Obdm9vcvLWDhu8bKviD"
			"n7z885fTaSek+sUvftHY27nxuXb76Aue5z6CEM/cuaqfYqXQ+kcI3tRKvPD4E6/+tz//cyam"
			"N/9/kO/MzI0RNsMAAAAASUVORK5CYII="
			)
		self.particle = PyEmbeddedImage( 
			"iVBORw0KGgoAAAANSUhEUgAAAIAAAACACAYAAADDPmHLAAAABHNCSVQICAgIfAhkiAAAIABJ"
			"REFUeJztfemvJcd136+W7r7Le2/2fbiYFEWJkkORkklREhLbARMBDuwEkJAAARIkQP6BAHEQ"
			"wMiHIB+CBNAfYSBBDDuIlE22EkQfEsQWRYJkhpzhTg7nDd+8fb9LL1X5UF1d1dXVffsus0Vz"
			"gL63u2vrrnPO75xam8Ci1U9uPAvK/jBNk5fG4yHi8RBJPEaapZBC4BE9XEQZA6UUAJAmyWtJ"
			"HP+97/3Vv/m+HYfok5ufvvePIcS/HI0G3aODXRwf7yMejyCyFEIISCnv8eM/onmIEApKKRjn"
			"4EEIzgJIiaEQ6R/85Vd/8KMiHgDc/Pj6vxZS/JOjg13s721heHyI3f0jjOMUaZpB5jFJuYiW"
			"T2LitkzhK6x9UXUlefOb8EakMXQqIq3fiVQu/ckmZ8YZAWcES90APBcEIQTiePhvvv97//D3"
			"AYB8+N5bf5sS/PvDgz3s7Wzg6OgAu/sDJGlWLm7WyphaAMjURZRT1iScIADNwQ8a86d7Js4I"
			"+h2GKOQghOL4+Bj7hwd/5+//o3/6R+SDd3+5OhoNr+xu38HOzh52D46rRTWrRj39f8H8GR+o"
			"SDnNO5HK6aza70uy1GHoRByD4wFura59tHPMvkbjOL5yfLiH4+MjL/Mn4GJjgdND/7SRWyRp"
			"Af3NwfNpf/ssqpEWyXwAOBplyDIBxigyIb6EePcHNI5HGI0GODgaLqq4GWnekmZL35zqQYT+"
			"+Wg4FmCMIwwC7B8e/g5N4jGyNEEcp9XH+pWH/tmJ3BUWzguPBHEmQSkB5xzD4+ErNE1jZFmG"
			"TMiGhFMUXjVjMzzoApI8rNA/h9ffnERdSCFBCAEhwGA0vEKzLIWU1Xb+vOB3d+IuLv2vGvS7"
			"pQkpkSRpSKWQVeY/gv6Z6UGF/gpJiUwIUOAR9P8qQL+bi5QAJEAXUNyMqR9Bv315r6DfpZIA"
			"PIL+2emhgX6HjAA8gv5pH2fq4ryR7jH0u1QIwCPon50eRujXVPEB2ifV0R5B/+Lp7kO/Jo8A"
			"PIL+qekhhH5NNQgwU8kLjLu49I+gv5kcAXgE/W3pYYd+TXzqxA8F9AuApICMkWbbOHX6M+zt"
			"UUSdbZw6fRv93g4ykUEIAUhACCDNAEIIRNbB7u4lHB6eRRR1QcgSkvgSAn4ahIQAGAjhU7zT"
			"gwn9mvjkKBNLXmDcedKnSNMYmdjF2bMfgrI7eOzxdxBFRwiCDGEIZBmQJOqfEMV4KdW1EECa"
			"qvOTp26DEBV3OCRIU2B/7wrG46sQ2WXI7CkwfgaU9kEIA6l2oPif/UGAfudZpxOABxD6CTkE"
			"pdsIok9w5erbuHz5MzAmEIZAHBsGp6k6JwRgTDEcMPf0fUCF6fAokggCIIpWkWWrGI8JkoRg"
			"d+dZjIZfAedPgtOLILSXj7KRBoFoftvZk7RF72q89gLwgEE/43dA6E2cPn0Dz37lLSwvqzmM"
			"aaqOJDHvyxhAqWIqIWWmu9dA+RwohweBBGMSFy/dwHh8A+Mxw/7uNyHEcwiDxxHwcyCUgBBa"
			"ru8HDPo1TWkC7j/08+AL9Po38NRTv8Bjj6+Bc6XpSsNVGq3JUppDa7SGfpvZyiTIklDYiKDj"
			"2ZSmAOcAkOHMudcQx7/Ewf7TGI+/hSB4FmFwNhcEAsroAwf9mtoJwAMA/ULu4ey51/DEr72J"
			"xx5bBWNKy8djzXQCPaptM14zP18fASEAxmThB2gE0GWakXFZSielOtd5auFgTF0HgcTJUx9h"
			"NPoYx8dPYzR6AZ3oRQRhHxISlJBikUalnqavoOnSN5ikyQJw36FfgAfv4smn/xe+9rUb4Fw7"
			"awSUApSSgilAlfn2fU1SkoKZisnSCrOfRhb3bEFwDy0EQgBhKEGpEoT9/fcRBt/FUv/LYIxB"
			"SglKae4j3F/o19TSBNwf6Cd0A5ev/By//vzrWFkZ5R68drRMfM0I27N3BUDDvvIFZHGuwkiO"
			"Bq4ZIHk8WUrvEwIljOY8DCVOnf6/ODxcxfrGKzh98tsIo2UwypRJIFoQFlFTTZGaYzULwH2E"
			"/ky8h1e+8xM8/vhqbqcJKCWVFLYA6Mq37X15spMs4N5otbScvGZB8PkJWigoNUhgBA3o9XbA"
			"2E+xe/AhQv57OLFyCVxyUKbyrZqFxgryXTSkmRyvXgDuG/RnCKI/x1/77f+C02eOkWUEUhLL"
			"ISOlf6WZZSHQjLcFQTGPFNdGq33MLd8zeSiz4BMCWxg4N8LIGBAEAkvLH2B/798h3X0VJ1e+"
			"jiAIwPKXmq7puBjo1zTBBNxj6CdHOH/xT/Gbv/W/QUiGLKMFTJsKKmu+RgWb4Zrspp/tBGph"
			"MIzT12XmFk9mXdfFq0MF7UhSCiwt30Yc/1usrf8Ozp15GVHUAQcHpdTyDfxV03BZk6YdP/wC"
			"cB+gn7JtPPnUT/Dd772JOAYACsYM46uOEyngXDPets/lexIAKTFb3zcabwuFez8vsWB4VQia"
			"TEPZTCTo9H6CtfUxzp/9DrrdHngQ5M/qEYIFQ78bUhWA+wD9o/EBnn/hP+Dlb1/DaEQKxuvK"
			"1rHtypGS5FBf9ug1GV/Ahn7DZCGklbeOI4uyDNSXYV/lU46j7zUJgo0ElEr0ln6K9U2Bc2e+"
			"g16vDxkE4IyD0Dq/YLHQr6nGBNw76KdsF8+/8Cd4+dvXEMcEjJFca6oC4Np/99x2AIGyturr"
			"MuSbc4UGtmYbgXDNgcmrLATlMsrXLhJQCvSW/gwbWxLnzryCXm8ZCADOOQSEEoIFQ78vhFZj"
			"3DvoT5IBnn5GMT9JlJevKoxazCf5Y5bPFbNMm1o7UpTqfMx//QHvf1EHRPsXZfSp9w+cd6xB"
			"A901LSXQW/oZNrf+D46PjxDHMdJUbcghpO3QLB76NXFfjHsB/VKmePa5H+OV71xDktBCQwCS"
			"d+6UNd8wgEJppnECld33a6LWbO0vUCpyhhrYN9ruh37jTNpmwryNWzZQdkBt5tvjCpSqbuje"
			"8n/H1naA8+QVAECAAAEJciRgs1ZzK5ptQsjUcavU6f05vvu9v0CaEhhnieb2z3SUmCaSjQIk"
			"j6P+lfNE83ONArQIU4duCehXpjkT7HJglWfbetPxZIfpe3b/hH3fvWeEXKczQtFd+m/Y2rmB"
			"0XCEJEmQpin0qq27Af2aKhNC7gX0E3odf+N3f1xUuGoOuwyuO6pxVeWSookH6H+bWTTXetUd"
			"K6VElmk0MYwtO4Lquro9kg8JDA90fJ8wAOUmokKp/D7/GXb2lnGaXi0y4ISANDUR7YJ9QfWp"
			"VJnTRJ4q55okSbqJv/JbP0a/nyDLKBjzabdt76kVTj2HMRG0GIalOZrYpqOaf9nPKA8KNdW3"
			"992mMNPuocsOwi1k8k9xeLiH8ThGEicQWXZXN+maYVLo7NAvRIZnnv0ZHn/iTtGnX2W2rem2"
			"cLhCUBUcA98upFOUnbzyu7jwbr9jtTVSVw9lMzCJXHMAKETodG/i4PANjIZDJOMxkiRpFoA5"
			"tB+wTMA9gX7+Dl56+Q0oBrmevj6n1jV1wlwGtX0QbQ7K9ls7iNV45T6COsi375dHG/1P4eOV"
			"eRaTtrf0Ovb2LoOSL4FQCkoo9CTUUh/BnMwHZkGAmaF/C7/x0v9Ep5Pm2u/TRhcR2vgEfj9B"
			"tQwIhHB9h7w0QtBoV3OyRxTtl/cNNdu2v02+vnhqbsERQF/H0dE+xqMRkiRWm3XK6lL+eWkK"
			"AZgd+gGJC5d+ga8+92nRpq7C/2yMrg/T/QUmjnEM61DFjzCuhlbeziMgk8juO/CZgm7/AwyG"
			"72A0HGE8jpEmKbIsMwKwAO0HWgvAfNBP+Wd46eVfAECh/fUMrQtrd6j+A9seGybqph9gpo81"
			"v5j0hDVf++Yi2GE+coVAowOPbuDwaBPj0QhxHCNLM4hMQDSgwLQsao8AMzIfyHD+/Ns4f36v"
			"GNOveub2+TyHW7qbtywExG2yuU9dhXjiRQF7+NlO34Z8rQE7327vC8TpJxgNhxiPx0iSuECB"
			"RZmCFgIwD/QDhK3iGy++AcDWukmMr4vji199Vlu79bmqNPu+SeMyz50j6HPwmnwAm9rwqU4A"
			"CAG6/Ws4HmxiPBojHsdIM4UCPgFox6lyrAkCMB/0S5ni3Lm3cf78vjWVy8dUOPd85mBSGueZ"
			"vedlprqCoqeT2fcAV8P9ZAuBPS2tDdXFlRKIOltI0s8xHg0xjhMkSeJFgVnVdDICzAz9gJC7"
			"+Evf+CUAQEpqSbrLGOq5N58gGM23w6UTV5a03aY6rdd523Z+0k76LlrU+Qk+FKAUCDsf4Oj4"
			"CPF4jCQfMCo5hK2p+p4NAjAf9EsAYbSKixf3nWZfk+12O4Mm2fvqPcUcw3w1EESgJ4W4Wu9O"
			"JTP37fY9qWWaj7EaMeri+qiu2zjLgP7yGpJkW5mBOEaaJoUZaD1WUBOreYOIObQ/Tcd4+pnX"
			"KmP05ZhNGu3mWHfdTDYCCCELjVUwbZhcvq9S6HP7XhsfwGV6WwSoe35KJQhbxWg8QJx3EWdZ"
			"CpGJyRlMoHoEmIP5ysvexNe/fgNZZhez2E6MdiQdBphnsDW9lKKGuXVHUzw3X+8TThAGIYCV"
			"kzeQxNsYj8eI4wRJkiITbc1APTNn3yGksbAUJ0+9j15P5PCvu1zdvn67vDrnbVoqa7X/kCWG"
			"VTVdQohqs2+ShtcJRpvDzdcmIYCoc4Q4Pc4RIEaWphBZBt9Gn2UijZf++QBzaT8ApLhy9SMA"
			"kxCqbUFuPH+mTWXVMcVN28S8+niuw1mfftKzNcXhwRqSeKjmCySJ6hjybPNbS57qriLAnNAP"
			"AEIe49Klz4v1d9PlNClB3cvami+dyvehgvYHSAkRdDz3fFEmoCmPSdTp3UGcxIjjuGgONrcG"
			"Jle+PbTU7ilqyaRn/A4uXDxEmjY1r5og332h8nWVkdI6L9/Xmz+oIdVJaauCUydQZea54c1o"
			"46+PZpIS6HS3kSRDpInyAfT8Qb8ZaIZ+TZ5JodNRJQnJcPHS+7nzp+DRXwHScy4BCOd+OZ6p"
			"cAkhZOG5m/PyfR1XO3y6JWA/k+0DmE6cZqYWTyRVU80WBk3uOkXzrPUmqI6kBIIggcBRYQLS"
			"JG1nBhr4SttEmpyvOROZxNLSdjHXrZmaIrgCIiv3bc3VYX7tRs58zQhZEhSbQcY0lAVDl+cT"
			"FvMsZSbb933mwHfeWFsSiDobyJI0NwEphPB1Dbdn5gwzglwqFyZxjDDarXSG+FHA7p3zIQKc"
			"8PJBiLSYKy0GSosh/jBb07Xg+LVfle8iRxuor4vfluEuKRQ4QJJlSFN1ZGkGKYVVZe2gX9PM"
			"AlCXb7e7gwsXNhDHtgNoul39QqCpCvnl//xK2uG2bReWBgpHw20myyIfl9FlQfAxTpY6tnym"
			"oUnrZ2U+oOqTshGyTDUDszRFJgSyfB1BxQy0AIKZdgnzQb+mOBlgaWlcrLl3X0Ddk/kUbQGA"
			"eeBLwzrN4+jZu2UhUrN9FLMBzTzDeMVILRQ+H0GWDoMYqLnng3rbtEw+bPLdayLVH7CPY3qI"
			"LFtWrYDU9AVIOW2ra+Zt4oA68RJy4NEECSkN2Nh2FRDOGIG9hl/k90QRbsNzlknH1PiYW2Zq"
			"2dZLL5NsDXYdw3K4/SzNkF/nI0xLQTgApYOC+coHyKrI0lIQphaAxnwJEIUj6IWQdXZPX5el"
			"VcN/uXvWB6M2jOuKV2F+ppcZXmZ8telYvufT/HoG1/sJdcIwLVEqECcclAhkmZkhVGoKToEC"
			"UwlAE/SDqAoIAi0AuunlCoDZbqWal7Tu6QkcEq4P4drkMgJUmW+YXma8Ps+ysq/QpjnotiCA"
			"NgJSjq/Rq818A5vSlCNjGTKhDtUKsB3BJirzbgYT0Cxe3d5Rw0vLUnr18uWn1it1fULgY0bV"
			"BktHs9tovJ/xvmbeIo55idIEUkil/QIQUioUkHIyRDvUWgDa5ru0tO+BRWoxVZGWfH0OaE0Q"
			"8E/daoJhP/P8Dp86d212VTj8ZfhQoa5PoO6YVuOrlOXPIyCLYxLz/dRKACZBvya93q6u8txt"
			"WWwhMJVCirxcFKgTANfBqzK6qvVA1duvokp9mfcTCSSQN/t0F7fM6yefIOJtCvilYwoT0E68"
			"tjYvFBVGiHF07AUtZjMGYwYM1Os45X99Xi8AruabruB6BCgLhho38JmZOiSohvtaEpOYP60w"
			"iCwAWG4S81rLRaAmRT3vJgrAlCYFcRwVzHenOek+ALWQU+/ESQpG2WRrvbnXVgAUI4Eys+vH"
			"DlCMXVTDm5Ggznn09Q3YzJ+1Gaiq3RoBlJr9bdOWqVEA2kK/fTNJg1Ll6M2YhNDLsxVj9TzB"
			"LJOl9KZ7V90p/5MCov0C4GvzS4cRVZRw41a7kv3NwfbNw3oUmFb705SD0TRP22Z9QDNytzAB"
			"03kWw8EJywSomTUA8p05bCGwWwBKKJRJ0E1H1fmjkMA4hqay7XEAQM/gKbcKbO2vMrOuyVgV"
			"jKow+KD/bjOfECBLIwgRgBX7KDXxhzRcKaoVgGmhXzMxDPs4OgqxvBwXpsAO1xsxAa550FOw"
			"BMoze7Ug6LA6TdRhevKn9J675sJFjaqAVE2C8S/MMwBVuPflM4/9JwQYj5eQpj3wyPKeW/T/"
			"1sXwCsD00G9IiiVsbZ3Bysoassw4f1LKYj8f0wowSGD8ANv2l7uFbfupBaDcnCsP8epJIKry"
			"3TEBnb5uMknVlvvMji9Pn8mwBWVWkhJI4g6kYABBvh193klOgPJ3jNohd4MJmAb6NSMJpOzj"
			"YP8UgLUCAWwk0Iw36/NtLTAoYUbnCKTMAJBSfj477DMBfvtvGO+D/SoDqz6H/rfzsaG9ziTM"
			"S6PBihoVpO4eSu6qa5c7vguPAMwG/YYY4cjS5Xy3TwCF168Zbuy9Zrj5N2Qq0zQRXQHwDcBo"
			"pgAG6vW0MH3ehvFNJsEVNB8q+EzAvEQIMB6dhN7xRG1+xdC083gT84HaPYKmh34dhVCC26tf"
			"BqCZ4Wpe+d/ngGWZzA/kh8z762X+rQCZa57p4zdpZCEcOg+37PmP6mCRT+MXSUrwKUQaFpqu"
			"BKC8N+K0TrvHBEwP/cUZUVu/SHEJ+/scy8tpSfLtfgB1LeGWp3sAbR/AbgH4xwPKmmlDcLkV"
			"UD63Ib6q5c2DQC7TfY7eIgWBUmBv5zyyLAIP1WdoKKWgjIHoDYdIe+jX1G6PoBbQX9wlBMAS"
			"7qxdxsrK50UHiwrT/4rJWiBsE1DuADKbQKqwKuPrnDW/lrpTv4DquIDrNzRreZ2zZzN/UUJw"
			"dHBO7RdEKRhlYJyB6X0QHTPQhvlAm40iW0J/cU4IAt7B6upXLSh2nTO//a2aCzNbx5xX4diO"
			"584KNoz3OYRNz1Q2HfVIURYE28dYhOevSUpgNFwBwMEoBWMMjDFQxpRD2PbDEw5N/+VQJ64v"
			"FWUcR4fPYDD4GTg3GkapaQoCrgkwWm+gPw+R9jKtcphx8OyKr07TqrYQbGZVRwft/ABZ8R/K"
			"Tmc9QizCJ6AUGI2WkMZdMEpAc+ZzzsGYdgStHU79rPLeaBabKaC/CKVEbXlOzuDmzSvFRxp1"
			"RVUdsaq2+rTcp9kGMZq0vz7fqoCYZ9SHC+n6sN9lEvTPawIIAXY2n4DMQlCqmM8CblCAUlDd"
			"JGzPKgBzrg72RSGEgBIKHnbx+WfPe+C0TXNrEiMnMdtlSH1HTZ1T53PumqC/Wqaqj0UwXwiC"
			"o/0zAOGgjObaz8A4B2U831q+DYJX48z84cgmcCCEIGABDg6fxs5OhKWlMdLU3jPfQL95bmlV"
			"mvb6q86fe+1jiM8xNL5AnSDUC8Y0zUfA/C9CCCgF9vfOIkt7he3nAUfAA3Ae5H4AbaH9fgHx"
			"I8AM0O94gqCUIQzO4sMPngNjddBfV8mTNbz90WQ+XA32x21ChCYk0Ofz0u7WVUDwwu4HQQAe"
			"BAoFGAMlygS0ZZVNjZ+MqaMJXoBqklKCIOhife15HB+/WXzw0V4yVm4GGipruo0C1a+D1iNA"
			"2eFzw22nzzC/2ZzUMbw8M3lxJoAx4OjwFIZHpyzo5+BBoISAK5NA6HSjgjZN/eHIZnAwYwKE"
			"qM6KgD+Bd649jxdefNtZRFmGfzu9W5nmXE4UAMDfFJvExEmH6+n7BGSRpOtm885TyNIuwjCH"
			"/jBAGIbgQQDGVZOwWfub1bUsAPNCv32bEDDK0Ol0sbv9PHZ23sfysvr6p/5kSjm+1vaqEOh/"
			"VyDqINrnA/g8fp+9bzrqyrARwKZ5tJ8Q4PDgDAaHZ5S3zxmCgCMIQ4RhiEC3AvImYBMvmmiq"
			"1cGToL90RVRzkDEOQp7Ehx88V2lC+fvpq23uSddNcD3ZL2jnGNaFua0GV1BnIT1Curt1BVIE"
			"oJQi4BxBoJkf5AjAJsD/ZKa2/nBkG+gvv0Q+WMEoorCLrY1vYXPjI5w6fYA0Rf5pODu+Obft"
			"vhkXqFZuHROqMK/nGzQzclrm28K4SCIE2Nu5hKO9C6CUFnY/jCKEUYggDFUnkP0lkSmhX9Ps"
			"+wS2QQxCCsdlqXcVN67/BkYjNa6fJPor4NXDRQS3K7gePepaE9OPBPo8f7fjZxEefrXOgCTp"
			"Ymv91yBEWDT7oihCFIWIwghBwFULQAvADNCvacp1AdOFqk4houxXGGBw8CI++fgmnvnyR6V+"
			"AXttAOD3oI0Gy+Ladb5se9zUSmjScluj7Wuz48nivPw62vjiKYyOToIHivlhGCrt7xgEYPko"
			"4DRrAHw08cOR00J/JSo1zZdedxlfrH4PFy/dQqczLjaRaiMAdsW7s4J0uH+1TzvfoM6ztw/A"
			"b3IWRZQCB/vnsLd9pWj2BUGAqNNBp9tBJ+ogCEMENvzPCP1FmdVb80N/KbqFAmEYot99HNfe"
			"+k0MhxRpiuJwob1pIofvep7JHj5m+/L0DQhNux6/jigF0iTE2udfhe70CYIgh351hFGIMAzy"
			"tr9q/89dblPgrNBfiWmhQBiFgHgB713/FgDjB7j+QF2lzysA07b/7XQu1d2flvRsn88/+QaS"
			"0RKoVVcdrf3dDsIoykcA2czQ78bgdcHzQn8pWd4i0N2Y3W4Xx8cv45OPN/DEk58hSfQYgZ2m"
			"agL0v9sX4PP+fWbAd16eIzidACyCtAm89dmvY3BwFoxRh/lddLodhQABVx+YzpvYTk4zld9+"
			"PsCcaFP0DAYBRCaQZSexduuvg9L/hKuPrSFJqp9S00KgodbXHKxj/LQ9gG6njh2Wpu67LE7z"
			"CQHu3H4GhzsXQSkB48rpizpRwfxOp6N6/3j+dXE22+QPHwsnrAuYPrQ2Vd49zDmHjCSEFFha"
			"OoNbn34fQvwUl6/cKYRAxTdpm5BgkgAAvn5/P7PrhMOlRTJ/bfVZ7Gw8DoCD5QgZdSJ0u110"
			"e+qIoghBEBRj/57cJpdXc7/dtPAZod8lRhkEEeBSOYRZN4MQF/D5J99Hmv4ZLl9ZK8zBNALQ"
			"pP3+jqFqq8Fu/vnKWyRVmC9Vsy4IDfN7vb5ifqeDKNLa73P85uPJ5H6ABXm59kARZQw8kIhE"
			"BCEERHYOqzdfRZL8D1y6/EUB+T4/x20i1mm/ry/AbsrVmQGfr7FIKjP/CUAGBfM7UYRur4te"
			"v4duv6tMQBTlEz/oQqFfU0kA7gb0l1ITgCB3CBmHDPLJmkJCiPPYXHsVIvs5zp5bBQ/K2Ouz"
			"u24/wLS+gG+ET+cF+OF/rjogQJYFWLv1HPZ3LoKAG+Z3Ouj2euj1++j1++h2u+h0IvBgsV6/"
			"S83TwhcE/QTlDgvtxXLOISGh1vAJAOewvf4qtreu4Ymn3kKnE5fzyfPQjLE7hJoQwaf9dWYC"
			"WLzm6+cej5bwxc3nMDg8A0IpOGcIAuXwdXtd9Jb66PcV9He6HYRBCF7M+l0s9GuqNwELhn6X"
			"tDPDwYFI3ZOSQAIYHH0TH95YwaWrr2N5ZQ+MlTtdXBvd1h9wEcDH/EWTemaK/d0LWF99Bsl4"
			"qfD2C9jv99Dr99Dv99Hr99DpdhGGYT7aN3uHT5tUC98fwJu6JouyR0ug54UQAGTwZax9voy9"
			"5Xdw7sJnCMMEgBEEu4mo79sQPskBFMIgiE6zSNLCOhquYGfzMextXYaUARijeVNPdfF2e130"
			"Cug3zOec50u/Fuv1u+QXgLsE/d44+YihitgBcvOgvpp9GcODE/h07ypOnr2Ok6fXK1PKbGYD"
			"9Sjghulr+38RZISTYmfrCnbWH8d4uFJAPucMQRii04nQ7faU9vd66PZ6Cvbzvv6C+XP29U+i"
			"meYEtqN2GekFjYxZvZA0Hz/IF0EMB1/CzsZJHB99gOUTt9DrH9QOHPkYbztztt+grxdBuidT"
			"CGBn6zL2tq9ieHQSUqrFG2pYN0AYhYhyb197/J1uD51OVAz01DO/5bNMEXdB+wPUpG6VhdlU"
			"Wnu7NBcA1fTJF0IMGUbDE9g8voog+gIrpz9Bp3Nc5NKk1U1Nunm1X/deasbvbD6G8WAZIgtz"
			"W88KZ8/u21cCoCA/iiIEoeno8cO+qqtJNDFGvjxfU4s5gdNTG+ivpNH92wQg4MX4AbMGkjjn"
			"GA0fw3h4Dpu3L4GybSyduIX+8ma+Ne3da7+Xn9UgR5KE2Nu5jP2di4iHfYgsAqUkX7hhBnXC"
			"MCwGdbpF/74a4AkC1QtI83dWhVRKXcCDV29xf+A8hU2TthqXUar2Bi/WwBNQxsEYLwaTgiDA"
			"eBRiPD6H3Y3L2NvaRdTbxPLJWwjDIQCz88gi+u01szXES0lweHAGe1tXMBosI0s6yESgVkQF"
			"et1eDvm6Xz/X/KiT9+3nQsG4ntzZNL2r5XPOEGPO/QFqUreEfvdS36GUQhKp7lFSrIfTlRoE"
			"gbKXowDxOEQcr2B4cB6Dg6cAcoxObwOd3h6WljchiQQhohhj0N5/E0lpoF11FlGkSYj9vcsY"
			"Hi9jcHgakBRCBCBQY/NBQHMhZeA8UNO3IzWTp9OJcsZHamZPPq2bM25W9cw4r7811eQ/x/cC"
			"PGUsUHS1CVDbn+Rr4vMmlJ4eHUVh8T3deBwhSfpIkhMYHZ3F4CDB9voYjyyaAAAEsElEQVQI"
			"lA7R6e5CQoKHAywvbwFEfW4GRACSAEQCkkBKpnoI0wBHh2cxGi6DQCAeLSNNQ0jBIMFAoZ6J"
			"c6Lm5TPF9EJAQzWBs5jHlzM+CAKEYQCWL+kq3nFCXcxYha0Ca+cDzPwIs2h/TTLFfNscqKXQ"
			"xSzZXMvicZwLQYwkTZDEifrCdtpHJjIMDi+oMQchsLOmN1fMu6GB3EZUEUn95WUTgLC8eZof"
			"enWuYnyYI5OBfT2HPwzzpVwBB6NqTX+xwdOkSlhIm78+xoz7A9QUMSf016bI+woKx5DpbtQg"
			"r+QESaIYn6Rxfp4iTRNkSYo0U1/VyITIxx3Ml7ebdtrUsKz236HFnIZid45iqVaIMGd+EIYI"
			"owBBEKqVPPn8fePh1/Xrt6irWWhCWQsxAYuE/qZAStW284RovyArBCFNU2RZmn9QMf+wYn5k"
			"aYo0/9p2lmUFGtg7h9tyYCapEgd9aLFKhzMzfq8ZzfPuXZa3WNQWLmb1LqWs/g3vMfRrWoAA"
			"LB76mzLTDJFEWg5iBh5yiCxDmGZIM6G+qJVl+de11CdW9Zc2s0wLgP7imDIJTkGgyDe7yPfg"
			"0TN1lRDkc/N5AF66VhDPqFm1U9+ub3rduwv9muYWgLsJ/Y25uYIACck4RCARZAL6g4qZBf8i"
			"Z7xmvpAApFTbrFf5X9h/0zVNiy1ZlDDw0nWxcxf1efblt21+97sP/ZrmEoB7Bf2Nsa0JkpJK"
			"UAlIJgHB1UcVcpuvztUn5UrMz7Hf5j/RT1GYAKC8OSMtnMHCk6ekZDKa3ulBgH5N83827h5B"
			"f5usi1YDtDAo6NWTTmTh+ec2vwb+if4lplfTZq46nK1aZ2HkfYR+TTMLwP2C/knl6Lwrmshg"
			"vP78vq8F4G+OEud/8u56dYEPAvSbxbcSvN1HB5wy7jduzZhEmYu59sWqxpqV+fe5CtP8s7NU"
			"5mOl7YdFHzzod7V/ise5P3QfoZ8ASNMMg+EIcZIcUSEF9Pq9NvSgQ3+buNOFNMR6yKBfR4vj"
			"GPv7BxgNR7epEEoAwmDylOOHFfpVmgeE+fe5ChmVODoeYH1jG4Ph6A2aJjGklOiGbELSR9A/"
			"Nz0AXj+VMe6sb+LW7TXE4/S/0v2Do4+OjwfIRIal7oRJwo+gv0Wiew397aOGLMP29i6uv/cx"
			"7qxvfTqMsz+hh/sHf7C+uY29vQNQCJxcCsCdFSiPoL9tovsB/ZMzZhTo8BT7uzt48+3rePva"
			"DRDIf3b9+vWY/eef/vzdb7349ZXdvf1XkiRFFIZY7kf5VygJhJRFZ8jUj1ub7F5C/wMgAHcL"
			"+ic4fowCAZWgYoTbt+/gF6+/hb947U1IiR9de/eDHwF5R9CNj9Z//+QS3flibeOfn9/YCi9f"
			"PI9Tp06g3+siCMKa7s1HdP/J338jpUSSphiNxtjZO8CnN1fx5tvXsXZnPZaS/IuvPPf+v7r2"
			"ropb4urf/eHvPjsYDf+QB/ylEyvLOLmyjF6/W/R5P6KHg7Isw+B4iJ3dfdxeW8fanQ1I4DUm"
			"sn/w1vWPrttxK1z94Q9/yLY3Vv/W/v7hD5Ik/hII+ea9e/RHtFCS8g0QfCoF+aOvfO39//jH"
			"f4zMjfL/ALi+hrIk6FeqAAAAAElFTkSuQmCC"
			)
		self.attractor = PyEmbeddedImage( 
			"iVBORw0KGgoAAAANSUhEUgAAAIAAAACACAYAAADDPmHLAAAABHNCSVQICAgIfAhkiAAAIABJ"
			"REFUeJztvWeQJMmV3/lzD51ZsrtaVrXW0zON0QIYDIAR0MAuFLFccu+Oa7w7o/FodnY08uyM"
			"vNs17n3gLblrRrsDj1zSVoAH7oAzBqyAHgxmMTMY3Vrr7mpRWlfKUPfBIzIis1JWdc90g3hl"
			"aZUZ4eHhEe/535+/9/y5IEXXL53eg9S+6Xnuo6VSgXKpgFsu4fkeYRDwK7q7SGoaUkoAPNd9"
			"xy2X/5snn/n1s+kyIv5y9fKZ/4Ug+D+LxbyzOD9DLjdHuVQk8D2CICAMw/e5+b+ilZAQEikl"
			"mq6jGya6ZhCGFILA++dPPffVP6yUA7h68dTvB2HwTxbnZ5ibnaSQW2BmbpFS2cPzfMKopKi+"
			"RZstScq2eUW9m7V/q0Z3qltfiycSTc92RKLtZxJLfta/rHVluibQNUGXY6BHghAEAeVy4V99"
			"+td++58CiPNnjnxdCp5fmJ9ldnqcxcV5ZubyuJ5ffbvlvoyOBUB0fIvqKxtc2EIAmp++05jf"
			"WZt0TZC1NSxTRwhJLpdjbmH+N/7b//5//bY4d/Ld68ViYXBmapTp6Vlm5nNLb9W8azSmXwrm"
			"L7NBlSs7eSax5Otye3+9S7psDdvSyefyXLs+cmE6p+2X5XJ5MLcwSy63WJf5LXCx6Q07h/5O"
			"C7dxSRvQ3/z0ynp/+1UsLXQrmQ+wWPTx/QBNk/hBsJPyzFdluVykWMwzv1i4VbdbJq30Tsu7"
			"vvlVdyL0r4wKpQBN0zENg7mFhc9Jt1zC91zKZW9ps/6rh/7lk7gtLFwpPArKfoiUAl3XKeQK"
			"T0jPK+P7Pn4QNrmwg5svHcaW0dBbcMndCv0r0PqbX6J+hEGIEAIhIF8sDOq+7xGGS+f5KwW/"
			"21P21l0vgAk5xXurjjFQ7mdffhddfvaWtOlOhP7auwVhiOt6ph4G4VLm/xJDf4EiF+Rl3tzy"
			"Loe2H6PQXQQfVi328fHjT/Lo9AOs89ZEjyKi3tJ+A+9U6F9CYYgfBOjwXwf0Dxs3ONh7jDd3"
			"v8vYmnFVzgci1We6a5bvfOR7fL/wEvddvYenLj/OzsI2HGyE7FAQ7kDor60lDIEQ9Ftwu2Ve"
			"ffuhf04scLTvJO+tPcrxHacInABcFONrKVTHS1aJ93Yf5r1dhxmc3MAnjj3J/bl7WeX3IYUA"
			"ISr29aUtuvOhv5aqBOCXCfp/3PU3/ODhnzLbMwcSxfRSGzdIAeKNdSP8f599ge/Mf5+Pv/oR"
			"Pj/9LJZpEYZhBRFiVLhroL+GElH+JYP+kcy4Yr6L+nTiy5KAAZSBC5B/J89r42+RK+Vxyy6e"
			"GznIap1kdwH011IFAX6ZoB9g58JWXnFfU2O8QDFVa3K5iM4DzABXgUvRd2BHYSuB9CkEBUzT"
			"xMBA0zQ0IZBC3nXQH9MSHaD9S+Nidxb0x7RjfquCfUFlfK/81kmwLxaMEjAMXAZuAIXouKbK"
			"bJxYx6JcxPbsSmyEMAWa0AhF2NFMoTXdfuiPqY4A3N3QD0rD7RHdDI5u5MbgzYqmr06ihoRY"
			"EPLABVSPnwaC6LiZukUg6J/rJmfkCPwAIVSwhRZ9RPTX0VN8wNAfU311dnl3voVlV369hcmW"
			"qaHGl4VRlSeAt1DM11Bjf/oaAdliBpmDQr5AqVikXHLxvEgPaEu5uPOgP6YaAbj7oT8+GIYh"
			"u2/saH8g1BqfXjO/GooBrqsY7/s+YRAqqLmlgVLvH/THlBKAux/60weFEGwsrEO4K+xrAtbO"
			"rKpo/UEYWU7DUN2uZfV3JvTHtMwh4M6FflDMlwjWBqvpn+5t2rtbUgC9uW7FOCGQkSFISIkQ"
			"soWF8A6E/pq2diYAdwH0V74JyGgOQ+ODK3rruq+TKdjIOMhS09B1HalJpIzUvxXPAN4n6K/T"
			"zvYF4C6B/sqRqGfuGtu+fFVXwobZdVi+gUgxX9Mj7V9KhFxp7/9goD+mDl/NnQ396atEZKDZ"
			"Nr9JTfuWSWtm+pEINE2iaTqarqNF8fYyGgIatyL5eadBf0ztCcDdBP3xdyEQUrCOAZyCvWw9"
			"oH+xVzFbami6RNeT+b+UK4X/Dw76Y2otAHcZ9FedlYKesJvtw1uXNQyYrkF33kGICAF0XaFA"
			"hABC1Kv07oD+mNp8LXcP9FcdizT2HeNbO69WQM9iN6anV6x+uqap8V+PBGAJAtw90B9TcwG4"
			"C6E/fTDWA/ZO7OzcYCNgcHodkmi817Sq8X8p8zusfEWXrBz6Y2osAHcx9KdPu9JnvHeyc4OQ"
			"B8WhEm6vi9RkNARo6FHvXxoUchuhv871twpZGngDq2/WHt050A8wzSyHMyf5+d5fcGXTsPLu"
			"1aJAiOoCDRT5s1+4xNWv32THu5t58Ni99Ln9ifEHUghwm6F/afVtXNNeC+oLwF0K/QEBV+Q1"
			"3hh8j9d3v02+J6+8ez5JVFAt1cPAAMgC3VDMFDn5iXOc/cgl7ju8j8euPci9pX0EYUgQBJEy"
			"2O6Df/DQX3tmqQDchdCfI88J8yyv7HmNU9vOKaYGVM//O2mfD/QADhWh8QyPw08c58gjJ9g+"
			"soVPHv0E9xX24UgHIcNEEG6b4ndroT+mlQWEdFz21l0vAC/0+WHvz3jlkdeZ7JtSfv+Aav9/"
			"TO3Md0KSOoZIBCl1LiTk4uAV/t+Nf8Km0UEevfogT4w+wmqvH6EpX4GoGzS68h6yUuivd0Zf"
			"WuLugX4pBOFcwOTIlGJWNjpZT+MXqCCPegISoOL/LGAbsAvYWKdcXI9U97iWvcG8u8CDC/dR"
			"Nktoho4ulcWp2kl050F/THq9EncL9Esknw+fw3rV4lsbX1S9dhvQTRIGVltNFOJFEP32gQyw"
			"G9gBrE4dD2uu1VHDyjBwHnoud/P35/4OZtagGJSwwhCMEB3lLLo1YWK3B/pjqhkC7g7or6VP"
			"9HyYsdFxfpp/VcX0DQI7gd6oQCNBECim3wP0kSiMaYpjBgvAReAsMAl2yeJr17+AYWvkyBEE"
			"PiKSGCmUnUCFjy/DBPk+QH9MKQF4P6G/k5PNDT5h9PcF/ZPcKIxyWp6D88AVlCDsQvVqWMrc"
			"EBiIvtc6jGKkmEPFDF5ERQhrIDXJM5c/ilaGvJ8jDAOkAE1TBiNd15Fho/llC3qfoD+mKvF8"
			"/6BffZtnkZd7Xuct49DyDD4p0tD4jcVfZ8v8kBrLfRTTfga8DoyjmFqr9gYkil4M81pU/nXg"
			"r4H3gAWUDmHAkycepm8qQ7lUolQq45ZdXNfD93zCSsRQ202nTJmCVqz77LcL+mNqYQiqRyuH"
			"/mFxk9fXvc272w4xvXoWQph4c5LPTDyDmklF06mUu7WRfARhQMEroUtJVnP4zYkv8R/FnzOR"
			"nVRlfNSwMAxsAPagFDxBouTJ6ONF5c4CN1Gh4ukIYQmPnbqfdROr8bUALQiiuMBQtTu1Wqhd"
			"6J8Rs/zbB/+E6dUzfOrg0zw4d4BVbp96dmht0FlB7wfQ/t5vfe13XbdMrlBuo/jyob8YFjmS"
			"PcFfrv8x33rkRS5svkTBKqqXHsKpTeeYLy2wd3xXFGkbMT4Kw6qlQlDk7OJF/qT4PFcKwxww"
			"71Eh26FgaHIDx3tP48sI82OlbwYlDBMoxvahmJtDocU7qCjhGRI0qEw54PFTD7DpxjqkEGi6"
			"jmEYiG7Jq7/9DmZosr64FsuwKj6DVgtKR0rj/MHT3+DSzivkrBxHd53g9U1vMaPNsXp2FV3l"
			"TFXU8ZK6lsl8x5TMzs5z5er1ZSBAh8wfDSd4e80h3tryLiPrxhUzXNS0K6YQCOBvPvQLxJzk"
			"i9c/iW1Yyvau6SCTBZkz5TneKR3mp86rjGXGwYOpYICy70aZLzTWmqv50pVP81+2fY9ABEkI"
			"uB7d61r02QCsQa0AmqVqIUjtM++6vo2hm2uRUqLrOoZpYFkW01sXOP7wGY4/fIY3Rg/ysaNP"
			"8Oi1B9ECNR1sZCk8bp3m33/6T5kdmEvWLAawmM3xkwdf4ee73+CRkw/wyNgD3DezD13qna9S"
			"boM6QIDOe/931v6AP3r0P3Fy2xkWsznV2+uZY2MK4MquYSaLU+wc3YaOpgwrQnC1cIO/MH7I"
			"n2af51D2GDlyqq4A8mGe+4v30iWzhIFSCnu8btZNrOXkQJQYMz0mx/rZHArqPapXC1FddnBi"
			"PQ+eugdNKFewaZo4to2TzXDoiycYHRoHYLZrjiM7T3Bs8BRyStJf7MXwjeQNRoz7hfMO/+bX"
			"/z3FbLF+tFIAvuEzvPk6b+14j9P959HmNVYvrEJDIySMglLrM6MVi9II0KYALA/6/+yJbzPf"
			"vdCa8WnyYGTLGMPmDbZeGeKCdpkXur7HC+v+gsuZYTy8xFoXUHHoDC1sZB0DBGFAHKjZG3TT"
			"PdPF+VWXVd21ill6OliPJKyfWsMTJ+5HC1VMgGka2I6Nk8mg9Wu89BuvUdar39181wJH9p7g"
			"na2HmAvnWD3XT8bLEBDwk+6/4Y+//K36U85aCgATplZNczB7BOOIwXZti4L+ZJSsEoR2WLS8"
			"IWAZqNM318No17gaZ/3Up9U98nBu3UV+/9FvUKKsrimTDBtxDxbJNUfkCfYVdqBJrQLT2PBA"
			"cT+jY+McWnc8EZh2SEJPvpsnjj2I5gs0XcMwDCzbIuM4ZLIZbm4ZZ8FabFjFdP8MP/zIy/z8"
			"wBt87I0nme2Z5c3H3m1vtXIMHNeAQ8AVmMrPUBQFDMvCMHS0MI5OXr7RqQ0BWF7FYRji5J3E"
			"NBuPrz6NX4ALFFHr9UpQMsvqWHqa1uAz7NxgZnqWbrMLyzLRDQPTNJFC8Jn8xwlvwuENx5O6"
			"WjxytpDhw0ceRPcFUtMwdB3LMnEcByebIZPJcGn3cJMXkHzyToEfPveSQq5Sco+6FDP+Borx"
			"l6L3YsCCPk8un8fyfQLbxjRBF3oy82jj0Wpv3F48wDLn/AMzqxTT0y89FgSPBMpjJ0yJpJfH"
			"18W293Svj3/HAgXkzTyj5QnMQK8kSTYMvRK+/anFj7E4meN8/6XmSCBABIInjj1Ad95BSolh"
			"6JiWie04ZDIZMtkMWlbj9H3n69cR1x8pt1W/42OkngESxo8AR1HGrFJ0PJqGZhYdFt1F/Kx6"
			"6DjwVUoZrVTqnFEdBYW2SwKFAIZrNC6koww2BktfSKOeXgv9NZ9zmcuUSiU8zyUMw4h5BrZt"
			"093VxRfmnmVVsa/pM4lQ8NzBJ+lf6FZDSYQktuOQzSroz2QcZrsWmVg9WX1xzNyY0ZGSuuQT"
			"5yYqokzMEhgFfgS8iJqKRuN/uq16QSOfy1HIFSgWS7ium6xTbIuWPngTAVi5wccqWc2rSffi"
			"WDdoxvh6ApA6N5odo1QuK6ucryrUNIlpGjiOzapsP18f+yI9xe6GGv/Hjj1G73y2okco5ttk"
			"YuZnHSzb5szWmt6f7u3x89T2/vhcWlA84BfACzRkfPxejIJBsVikVCrilsv4nhdlKWknnX99"
			"RjQQgFtj63eKdus64pfgkljm6sG+SJ2rJwDASN8YC+TUCl7PJwjUdEk3DEzLJpNx2JhZz98e"
			"/QJmKKufXoOHLu5lzWSvCgPXNQzTxLYtnIwa851MBtu2Eabk3fuPJNemGV2vx6cFo/Z3AQX3"
			"ZeozPk0uuK4yO6tVykGySnmZ1FZQaLtUOxnJuE7rQSY2ybqpT9xLmg0BdXSCUMK0OYfnRS8o"
			"8CtDgWkYWI6Nk3EYzAzym5d+k4yuq+st2H/2ADuubEVKLen5toWTdchm1bjvOA6mZTGWmeDm"
			"xhHV/ka9vnYoSPd8j2RW41KtCzQhp2wS+AFBBPtxz2/N/sbMrMOelUN/TLn1OTXOtaLYThDP"
			"ENK/a4UgttSlESImDS73XcOPBcDzCQJlF1ARvSb5QoZzF3cxceYTZF7+u5ABefp+ePVvc2Pk"
			"XorlPkIymJaFnXHIZhLmW7aJYegc23RK3a8W5lv1+ljxLae+p2c5LUj3E509JKwCxcZUaz6u"
			"qbPu2RVCf0xaqCWafD2Ke3nMeKivNaejd9PDQXzjWBgCuNk7RnnUq6BA4KuesrAo+dafP8p7"
			"B/cBuQg1PwazfQRHP8rJ6c0oJ4BLb+80n/zE2/SvDrFsG8uyMEwDTdcJtJBDO46pe/ss1e7T"
			"v32qlcGY2X7qf5EkkVWrdxxGyagq/pEOGVWneNOg0M7rrb64d6EH6UsCK2WPrzcFi/P2xN9r"
			"FacwdU0a9mMFMiUc85kFZvV5urwMvucR+D6Xh03+7JtfYmR0E8qva6MGX+CVf4TyBi1GxyVz"
			"c2t48S+f5amPnOKTz+VSawEl1/URrg/drJ7C1rY1/t+I8fHvGBXaQYAQinoBEfTUeB0FNPQP"
			"tGbmkoCQ5dPS6++b2sv//KP/gWvaDU4PnmO8b4rJjdOEWkiohwk6xHAI9V9k7Se+XYMZw83s"
			"GOsLA/iez6Gjq3n+hX+I78ftM6hWMsooSbJJe6jC0OHnrx9gcvosv/HVHKYVEAQBZ3ouEGiB"
			"6rn1hLS2h3up3+mhLT5fZqkApLE9ABlKBhZW4ZRt0Kn4R2IXdFv8aFCsTlBoZ9Tw9kKgazo7"
			"5rawPjfAvpM7mCpMM+PPcXP9KFMbZxndN8HixkUKQbkxAjQzqMDSoUHCSM8I3sJeTp3ZzHf+"
			"+n+qaZlVc2G9JxLE3fTkqe386X+6zD/4H3N0dQne3XEoYWAt3NfO81sJQowGqfbLQNK32EO2"
			"4DA4tQHT1ehezGL7FhnNRrOSlUnKIQQtbYBNTtcNCm2XGkF/5ZBIBEHXdbJkoAiZMxabj20k"
			"+L5PvqvIa88eZGLNVOMxtJ4wkDpfIwQ3141y9vAgr3zvHzdotV2/sVWaZTwuCS5eHuSv/vIC"
			"H/nyTS7uuax6bT2oj3u3R2vGR/9lQbL92mbsosmGmbXoriBbzCLD2LyrrH2apmGYBrqhoxt6"
			"aoFqPT9A+8xcRkRQLTW+mRBCravTNUzLxHLtyrRM18t4nofpmqwe62Ni9VTiI2ikVDUbGmK+"
			"OVD8yd/nZz/4ly3abdU5FnNzaXrRV9/czokN1/D/cVBfmas31qchP/1JeTMHbq7igdN7qiKJ"
			"4sWnlZ4u1A4fpmViWRamaaLrKlOpFLUxAu1Bf0zLFoCm9UawJIVKq2IYBoGtYuU0TZln45Rr"
			"nudhYCTTv1oEgMa29FqEMIC3vgrP/7s2vH5pZSL+3ch2C+Ay/Z3H4PcuwD/7KcyzfManjulu"
			"5MyJUtBUfXRNJabQUlZJ28Z2bOXw0tXS9cbjcKt3sEwBaAn9qe+qtxuqkwqhmG+rhMue5+G5"
			"HnZgL3UOpRldz7lSezzm349+tw3mQzU30syOG1LLuWiu+i8+C589ArsnlSKYVupqlbxWghCC"
			"s2ghUh3FMA3134g3e9SRUsMwVAiaaVlYERIYhl7jCu58HF/BEND6ZnGCBnQBArTIymZZViXh"
			"ou/59AQ91W7iZopfvalhoJ5Eu7YLf3R/G21vZq9tZrd1wbeRb24l2D+5VJPvFAEAM29UUNGM"
			"3M2WbWFalhICQ0fX9Cg8TkM3YgFR+YoSAegM+mPqWABaQX/Vz6hhUqpkyqEWousaQRBgBAZB"
			"EOB7PlbZrB4C2pkGRnzpn+9l/cwA66b6Wbz+Yd5rZ0+AqpCidK+vJwTpTwBo7HzxIbbO2Fzc"
			"P8zI9gnymcim4NdU1YYOQEhlLYFlKb+Dk3EiA1QE81KixUkqNC3KV6BV6QjN+NCMOhKAtqE/"
			"dbBqrhqGhFJCCGEYEoQBvuGTLWcSs2i9nk7y3XB11k4PsGFqDf2zarqkeyoCd+rmo208RVq6"
			"audz9XSAdBm1xnzm8GYeGRhhzZF+8t0lpgZnmVk/z/WdI8yunickTAShkRBECODk7cj5pGOY"
			"JpZt4WQcHMfBME0MXa8ohHHGk3RwaN3e35Sqy75v6wIEVCxWAQF/3fUSckHwUPFDSKkl9vF0"
			"L4+UvK58lvVTaxicWEfXYga7ZCARSUCEIUGYXB9pVwBqJ+31umitxSaeFQRM5x3yeQPHLpFZ"
			"tMmeW8/mcxvY/9YOZlcvcnPzGKODk0yumiaQQTK81VYPOEUHoQk8w1OBJ6apPtFYnw4vr6yR"
			"aJGXqAUXqqhtAeiw3qZXSCTndlzgxD1n+O7R79N9rFtZYiOPmFU2GZxdT99sD2snVmOXdHRX"
			"r6RlE1oyRYphcXZhkJnZeElvo8eqHYhre3urcSeqxdeZnOtjszOeaOuaxMIiO5Nlw9QavEMe"
			"89lFxgamuD40yujAJJ50qwNkBZzadp6La69y78Qe1og1FXjX0+no2430WUbfbEsAlgP9rYp0"
			"LXSBBP/+gNkdcyoY4gxQAMd32DI1SNesg5M3lfNDF8m4pyVTJpW5U6fkWQhRJgzT7sKYYqNO"
			"/PZr8bke84PUtbUaKcwvZtTYHfVaPdLIQyAIAgLPxy7bDFzvZ++VbUx2z3Fyx3mubrlOkIlQ"
			"YQ5mmIVuODB2D2i0EdjR8uV3VLaDIWAF0F9DISHYYcILB3gctZDzMMxenuOV3W+ChP5cL5sm"
			"NrI618e6wiqc0EGPpkgVLVnX6ektsXHjWW7cuD+qNF7PlWa4m/rt1vz3q1qY/K9n5A/ZuGa6"
			"or3bGbsiBCAIg5CSX2RazDFhzHBp3VUm1kwzs2aWIAhgEuV4jEcWoKgVKrOiwFdu7OYpaFYG"
			"/TG1FIBbCf2V0yGIOFQcks7VDzyNCow8DAzDjD3HzLY5RCiwXYst80NsLWxiMFxPn+zFNK1K"
			"79u/+ww3btxL4iaMe348CHs13+P/cWRG2jNTOwQkzO/NFujtXkRKo4IATsahaJaZNKY503ue"
			"y73DjK4aJ+gJEufjDdSi07T9P3p+V7p4JS8Vy+ARGkZ7iNBB36wt2lQAbgf0g4I56dUJgYkF"
			"YiNq2dZ54AgwAaEWUjCKnFl9gTNcQPM1htwN7C3vZqs+yDZtCx966Aov/7yIH/RRH+4bIUDM"
			"/PT/WhddzAibbVsvomkhnhkw0TvD+TXDXNh4mdH+UfLdJQU+NgrZ8qjtaK6jonxrR6eInKKN"
			"53nRSmMXz/fxfR9N0whl7Z5EK4f+mG7buoDm8iFwcqk1A7UUm+L3AFtQq3WPoNbvRXFzvvC5"
			"al7nqnUdEQjWhH0Mde9APjOHf/wrMH4g4mGZzhCg9n/K3mwXYOsVZp95l0Pbx7i+aZLFVTlC"
			"J0zCt01Uqppi1O6LqN7fYleS0A/xPQ/XU1ZSPxoGliLA8qG/XtGGAnBboD9F/fPNw7Mr6K0D"
			"HwK2o4TgNOrlGkm5kJBxMcN433vw5ffg2X8Hf/AzGL+XhMmdIkA6aE/Avkvwv38TNs1yJUOy"
			"t2DMeBvoQjH7FHASFXsSl2lBTtmq6AB+NP6reL9OlMLG1OhV1xWAzqG/kyKq25tlY2nBeogQ"
			"C0IG+CgKFQ6ilnnXXh/zdnwzTK1Crf6sp/3XCkGa6WlBicd9YNGGVXOK0TGzMyinohNddgwl"
			"pDMs2XmsFYlAqi1p4g0pU5t6xzuV3irFL023aF1AU5Gpgv6YbNeujsmIkbmRYSv2uq0BPoXK"
			"/+PXKafBjoldCN9HGRfyJCswiqiBOP6fDkOudS2mf5fh2iAc2lnt/dNQCal84D+jspHMoxhf"
			"783GwlxHzZC+bBjdvVLmi8anljbzdkM/QDlwyZFXvJlFbdk2RTI9cutfByTWwnrufECWBR/W"
			"bZ54+LskCw1jQYg/hZr/JaoNROlpoOLShoEZHj6/IQGJmIl+1OZJFOPrjfORDCFRCakeB7ai"
			"UCQAURI45QYPtEJqxnyo3Ty6WcllQr+Hz4JYZFRMcLl7mIvZywz332AqO60yddReXEC9rAxK"
			"mapdW9jsZgI2Tq9H5mD7tsNcuraJ0bHtKAbX2vbd1P9aD1S10ceyXB575BDMGxxcFIRGWK0+"
			"lKi/zDzu8QYqDd29wPqoXICSvRlgAuYmFjAKOl10qUdJ+1BuA/THtPydQ2vKxsP3YrjImDbB"
			"DX2Mc2suMG5OcXntVVzdVS8OlhrbaquM0dtF9apM+83aProZz3URQvDkoz/mp69+ltm5fpI5"
			"ftoDmB7naxmvxhfHKfCRRw5jamXwDdZeG2Cse6J6IUu91AoxiGwHDqCmtVCdEMIGhiDcHPLj"
			"+1/FnrMYvLiBTeNDbC0Msa28laFwEC1tELpF0B9Te5tHN4H+MXeS8/0XmWSaUxvOMt41yUz3"
			"rOoRaU8e1M+8WUuxJXcROIdCic+hhCAtMHHenxTkGr7G6ulepTRJQTZT5PPP/ZBjZ3Zz7MTu"
			"qEFpJTDtDKoNMoDNQyMc2HeOjFVERjF4Oy9sZWzPxNIJRHr8jqt5EtXr49vVUjq4BSh2lbj4"
			"0BUuhlcgACtvsXF6PTunt7Ftdgs75rcx4K/GEmbLVDHtMB/aSRTZouf93rP/mgV9sfphUspz"
			"2xTn75lFZeq6hgrXj9O/1lLtWCthcGwjRkkDQ7lXTcPAMARPPnqeLYNjvPHuDqZneiJlKxaG"
			"WoUjJOOU+dD+swyuHUFK5a+PAzCGZjYgCoLQSrl8gzptFMBalK5SoLHNI03xkBFRySlxeegq"
			"lzdfrQjQtmtbGFzcyM7ZbWzNb2ZHYWsllnA5OSKWuS4gKSszUikziyiG+bRfVXpxxygqueM4"
			"S9OztYMcwObxDZW9/SzTwLJsTMtACMmuHWUGN5xgYkoycjNLztU4Ot4Pkw6sXWBrzzyDXQsI"
			"WWZ1/xyWUa54G2Nzr2EaZLQu+qf6mO6aSRTBenl+0u9Ajz7pqKd2qM5QeXnnVS6Lq7w+/ib2"
			"Gzb/1v1XSKkhRZgEkqYlYQkvqg80twS20PqrcgB0oaR9kcqim4aCEMN8CdXTr6JmAbEnN2Xk"
			"adiuGgQwXYP+uR6kJtENFVxhOza2bSE1FYVkWi62U2TN6inOHrgAXzkOl/pgwwzGsTUMfvMe"
			"4mVXUkZhWFGolorItTEMg92j23lr6GCCAKUGba19/trkGJ3YeGT0bqaB48A50AuCYndReUSj"
			"sDEBBHEoXhsdsbEAtKn1ZwoOrEJJqYFy6DgoK1gtusZ593Io2/iVqByqS2k9AAATYElEQVQk"
			"O3e3SxZVgb2DkxuwXANpSYwok5ft2DiOg27ohEGA63rous7F/Vd55b97W/XIB/NQhPMfHsaY"
			"Nznwg91o6Oi6hmGYmHYUpROtDDYMg/sW7+Gt0kHVhnore+KX02gG0wkiiOi95FBGphPRdxOk"
			"rpNbzEXoZGKGhrqtpqWMR7WVLW1KB1St9QdhyMDUKoY3Xa8O37JVA8mjGBw7QGZIcvTlSXpE"
			"MwpQL7mFQG4e26DgOoquNU0VOWs7qtciBJ7rcnLbOb73lZero3QiRe7U0xcIgpDHfnpAReTY"
			"VpQRzMF2MpUYve3eFnrmu5nPLCRqRL1X1erttsqbZKLQ5SSJL6TGwrgwv4Bt21i2j0BFXksh"
			"67zX+i+wyZYx7VWg+XU4GEbFY3PpdVTemzGSvDft9Pa0Qa4RCbDLFqvme5FSKX8qhj4KsTYN"
			"TNNCSMHZ7ks8//W/UEkV6k3dXDjz5EU23lzHvus7VWxetDzcsm1sy0QIiRkYbJ3ewrE1JxKj"
			"UG27NRSj2oH5WkGInosrqERRo6gOlDYtC8iUHBYXFvE9H8IwiSIyAmTYXuawpQbLTgw+IfQs"
			"NEi3Ep3HRukFl1G9uVUWjJhi6HwEle27ntk3asyW8SFMT1dh09G4nYRPK5/9iBznj/7Wnynn"
			"SqMZSqRw/ezLb3B1cESlhbMsTNPCjlbkmJaJYZgcmL+n2mFY0yY8lF7TgT8ADTV8hsDLwA9Q"
			"zDeoi5S6p1PIFygWCpXcSEnyiLTkNX7hbSazb2wjtIpWcxiPYvar8u62otgL+DRQL86zBl43"
			"TWxINPYoUsgwjYpyNBKM8/tf/78pO+X6zK+dw3vwvc+9xNXMTaUUVpZpaWhSxQDuE7swF436"
			"CBAbs36Mcgl3YuWNleNhWupFpqtTLpVwy7ELOSBYki+o+UuvFoBl2Po1SzafpsV2+7RhqBmF"
			"KEfP54F9LDWyxE2KTK92yaR7IVO1tbsRhYnphs60McsffOUb5Hpy7W8iHU2//vMz3+GMcy4K"
			"0fIJQiU9QggGnH6GZgcba/MxI3+IGv4s2u8AHo0XL8ckwCpZql2BH6XDCVTTO4grTARguW7e"
			"TNj6xXawacZqtx/9U7oKBGkmWELVu3l8CMvXK+vn4nHfMHRm7QX+n0/+R2YGZuuP+c0ohFCG"
			"vHDv97hRHKVcKqteFii8l1Jj7+yuxkogJLOAl4G3aOzpTJNaetCWIc127ShbfeQ2VgsuUiWW"
			"owNUUQs3r4Suxa7W92ln3Jdw4NI+PvaLR3j6Xz/OuoMDyVqMRhTApvH1lYWV8fhvGAZ5vcw3"
			"vvDHXNt0o7H+0IoCKDpF/nTf84wujlMqFnHLrkpLj+A+d5/q5c3qjxS8rpey7PjWloYhYVXU"
			"SvEFEKAHat5fmy1Evev24Gb58QDR6d757uY6QKwRN6ou6hUPnNnP7sub0EJB33AXH/u/HuZj"
			"f/AYmWmn/vUmdBW66FvoVtu6xosrDQN0yQsf/yturL/ZmbGlHnkw1TfDn239NtOLM5Gy5RGE"
			"AYPmetZPr0tmPbUkwfB0Hrx4L08fe4wHv7uPJ3//YfSC3nhsjxXIeublNPmwYM1X3l+cKEL9"
			"bN+q2xY4N6tuoDRA92yXeqBGBRudE6AFGs+9+1F2XBusjOOGYWDpFltPb+TXfucZHvrzA4kl"
			"LHXttvEhjFCrpHbTDR2pS1589vsceeB4O4/WHnlwc/UYf7z+eWYKM5RLJXzPR5cGe6d3LYVr"
			"qZiw6/o2Pv32U+wcHsLxLHRdY/OhDXz6d58iM+U0FoLYR9WCuopZ4szo1Z7C9vcUaJ4okgY8"
			"Td1oa3mIf/Kdf8jjP38YEYqlaBCiHjR29qTu3JPr5qMHH6ZvPpPaiCGxvDnZDL308tBr9/Kl"
			"3/k021/fUtm3R2hC7e4t1cbOerR8+ifPvMbBR49wy8mH4YHr/Hn3X7CYz+O6LkEQsCu/vWql"
			"jxLMLTx95HEeOLsHp2yp4ckwlLXOMlk7sZov/N6z9A331heCVr4PCYav05/vQ8i4+0fZRER7"
			"JuCYOo8HSPsZIslb7ffzlUOfZevFIV567udMDc0sHcdk9ffN44McOLMHp2xEyZ2V6dWy1Lq4"
			"eBoHEPgB9oLFmm/3c+nwJt79W8cohy59blel52umzhvPHuLNT7zb/tN3QpFR6szAeb6fe4kv"
			"FD6JaZps9gfRCzqe5rExv46Hrh2gbzqD5/qghanE1fFSbw2paWTdLr74jWf5q3/0ErND89WK"
			"ZKPcitG73ze8k+03NtHv9yDs2Hchl7WjSJvrAhqdDUFQYeA9o7tY8x/6eOe+Y7z3uaP4tp+M"
			"jzK5bMvNQR46cw9aIJOcvFZsunWiDBgKMhES3/dxy2XMosXeixbr/3AN092zOIYdbeJgcvAz"
			"J3n9M2+3/eDLpgBeH3ybzKUMz4RPYmsmT008geka7J3fjuuWKBhFZNSN45Sz8aJPw1D5BsMQ"
			"Mn6Gr/zR53jpi68x/Oj1JJV8PbuCgLUzA+y/spM10/3KLmFFayOjPYqkVJnRW8weq6hhPEAr"
			"6I8pTgMTG2B6Sz18+K0HGDqzjje+dJCR/eOV0GmREzxy5kMM3Vijtl+JoLuS+iTjkMk42I4T"
			"2d1NEAoByuUyplXCKBqYBYP+iR6CbIAQkmv3jfH655fJ/GX40PHhJxtfwRtxedp7kmfkk3iB"
			"R1ErgBmqncV1AyFAj6yJcdIH0zTRpKbsTZ6HWTL5tRc/xcvzr3HquXPVcRUR452izUPnD7B+"
			"clWl02hG7PNQH8NIcgmkd1trRe3HA9Q5XVmeHc3BLduK4tlh89wgq/5DH+fuucQ7XzpKSSvz"
			"2JkPsWlsXcVvrZhvVbx2mWwmcrxEa+3iISAIlU3fNTENg6JhUC6X8T0fIaDLyLBpZJBrG260"
			"9dArpuhdnLUu8vDMh+gyswDomkIyKWWUDynZZcSy7YrXLs7v73kehqmj5TU+86OnsRctDn35"
			"eCWmwvAN7ru8l6HRdThlU2ULM7TU7iW26ix28r60JlvW1jvaYl1A67NxGhjD0AlDi3gPPSkl"
			"WkHjvpN7WTc8wEIpz8BsD4ERVNKemaaJZduVHPxOJpPAv6F0A4EgDAOl5Lk6RmTsKZdd5QQB"
			"7hnrZd+f7Obthw7zs6dfJ2fkmj7BsimyQK6e6uexCw+ww92McMAVbkUXEVISWoFK+qAp+K/o"
			"NYZZSe9GGOIHPnpJ5QCSUvLsL54iyIYcFSfZOr2J+67vIZOzIKSyOjqOdbCiDSxih5VlR+ls"
			"Na2iCzTmWkJLBKBd6K8cqeQB0pW9RxBltooMM7qOUdQplcp4GY8g8EEIDF3HtC2caAOmTMbB"
			"tu2Kz12mEiOESGSgMoypIA0d04yygUdwKaTgE0c/wv6Te/jRUz/j6MMnb9mqmtj30D3TxQPv"
			"7Wfd5GqymoPneLhaGV3XELoeBY+IKOmTTLR/w6gwX5OKQSEhWhBEcC0rz/qplz/OvcEeMlMW"
			"buDim5GbVyaBLpZtVbKFxSnsLdtG15P31i61jgdoo654Tz+hG1GeOy1ilh5NfwyMYhm3XFab"
			"LAuBrhvRdE+N/bZtp9bZa1XLotUcN1ROmVjnMPQoEXTy8TyPVcVefv27n2b/z3bz06+8xtiO"
			"ieWN87EhRgetKLn37T1sP6W8jpqmEYpQLeGOTK9SysQBFSllcfIIXdfRZLKjuBJqFbAhTBEl"
			"viD6L9mc0yhkC5T1Er4XKZNRosh4OImZXhlaoiXzceKMdlnYYF1APWrhVZKSUERbqMokQWSs"
			"HFpmGddTW5wIQNdVqJVlW6nkh3pVz49vqyybqbw4gVCKlAFhEDEistF70UvfPLaRL/+bT3N6"
			"70Xe/K33KDXyBDYiDTRXY99bu9h2bCP2otqASqaZmsrkEfd000qSOGqarGjmVev8I6tdbai3"
			"+i4rs6qyaeC7HkTDSeV9Vd5ZksU8Tp7RLvTH1DwsvAn01y0uBELTlAtVJClcDF1F6HiepxIk"
			"QEWRSSd5iHt+s9bHY2soIyeIFmXk8NV1vm9iVdLPBRw4uocNFwf4zj/7CaVMG5sXxIGq0/D4"
			"X97PxitrEoFOzXbilG4q7tCO4gZjQVZzffUsouEzVeX9QTEfoTKCabqOF6XTS+9gko5J0I2E"
			"8cvdOm5FMYGNqIIGUiClRiB9dENTW5ykIDOGyHhcrIWuxs1YmilboGDVNMOKAUpGymZeKxJk"
			"PGVFjLN8L2k0aro6h1rSPQZBQT1DHAOgoouSUDM70sIrQmBWM6XdDF7xcxsYCITKPKLreJ5a"
			"MYwUaJG9RI86jKYl+xPXe2+N31013b78ADFcEyJNgRZoVcEKMo5lr6RAq3kI0f6dpZQEBGho"
			"yt4QBXHEPfb82qu4tqcY76AMLgUSI5WJirc7gwrDKgM6TKyeZsvUhkquY9tOmO448RgcRQrV"
			"heKaJ2jyQFIq37EuVDIsXYs7TKDehZQVQdRig08Tq1+7765FTOAKYIDIO6UJxXSp8t5Xna+b"
			"9qzVbeufrCiiIjGNxvGBozvGqwvH29UVUEGr51Br+heoSuQw3jsFukA3kiSOat9AJ9lJxIgV"
			"v1oo7vyZklyAglBqaHpQpR9IIRCy9a7kndAt2TGkPqXguRGjm9x0OXIdG2DSyZcDPeDSrqtL"
			"C2uooNXTwBtUnEwVCmHBWaTolOkiW7WJVCajgkTjeXcrKO6EpBAQpYWR6XidyjtsfY9OWNhk"
			"CFhp72+3iqU9ZSV3rt1OJUeBkc1jSwumffgNgjRCGTKfybGuNJB4Ks3EpGsYRgMHzMoRrX4v"
			"b/1mWpYQkQ8novYnjB2QWC4Xl/GiGpaONOera6+rLV6WaROa6J5SQ1laaU0lh1yqfS+f+aLx"
			"qVYVtUd1qmgQE7iiPriisitTaZZecqVnuPNrYwphpGccXyTRGYKod966sbJSb/0fy6yjzRK3"
			"fN/ADwr665FPwPGdp5oXahbQGsJk1zQFVLx94CcJnOII4WpaOfSvvGyjKhoozyuvOXWP2yK6"
			"y+/908wwvmqiedk26hrJjOH7Pl4qi2dsfm7YzlsG/W03c1knl06+l02i6l9bZVveefnMBxi2"
			"blDMllYWGCpgxp5XKdz8ZDfSpatvll19gx9ND96yEs3trh3QnQT9ccXn1l+oe8tO6fzAJTw/"
			"2uLGr5fE8e6D/phuyRBwp0F/TJ+58CxPvfPk8qoTVAJXH7/0YJTJszqZsxoGOmn2nQP9Md26"
			"bePuIOiPf3SLLL916Cs8cOIevvvR7zO8K4oYamPRheHp7Bvexc6RTWRwCDJBtMuJ+izVAVZA"
			"HwD0x7RiAbgTob/2bjtmt/Dbz/8mr29/m59/7U1cs4HqH1206+Y27rm8A6sU2fZNqbZqEbVY"
			"J6oTMSy397couyzqPCZwGfe4Q6G/6lRkorWFxWNH72fw9Fre+fhRTj97gQoUCPXZMLWOPde2"
			"smaqPwpbi4Na9IrLOnH2dKI/33nQH9PKt427A6E/PiQqTiEN01RBlOsX1/Lci0+y5eAQr/2D"
			"t8lpeZyyzYMX9rNxfA0iFEhNJNlGUgtVVNxdEn0rO4i+bf9x3x/oj2llO4fe4dAfb12rYvUC"
			"HD+AaOvavcM7WPs7qxjNTtB3uRujpBFqCs61OP7OMCtu4Dj40raT4E4h707oTyeh1pejzNwN"
			"0A8JAui6rvL6gbLpR4kj9KJO70Q3ZbOMJ33CIKj4EOKFHHHwpR37/6NMYVrslpXNGnHnQr/n"
			"efhBgB4GcdIDlk5pmt3lDob+ytHIo6ZpWuW3TEcrR7kEyqUyrutWBKB6pZLy+1fCvQwDXUsC"
			"PJdFHyD0C8DzfPKFImXXXdSDMKgEG/htSMDdAP1pimME1Hetss1cHF5lmialUgnfVT1CSlHZ"
			"qDkd56cyhRqV8LUV9f4WZZdFbQqjEFAul5mbm6dYKN7Qg0jqTUNSKDVfk3y3QP+S4kJlDwuC"
			"AEMalRXFWpROxrIsfF8FrCrE0Cv78+pxqHdNWPfdqvVrMmRhPs/Y+BT5QvGg7rlqJaJjai0E"
			"4O6B/kbXpCOGpEiSSgXRBk1hZVVTyu9fibrtbMFF67Y3PXjLS8QkwzKjYxNcuzFCueR9X5+b"
			"X7yga3KnYRp0OTqLhfqL0+826K9XFhK9IF5foEstSgCVRBNXcu7GMXpSa1Rd03u1bt+tgP72"
			"i5qaz9TkDKfOXGR0bPKyH2ov6gtz8/+87PvPd2Uz9PZ009dlsFjw8fzE3323Qr+6pgH2CBV5"
			"G8oQGcqqmVA6hvHON/i0rliTYEiPmekZDh89xdHjpxGE/9upU6fK2l//8JWTDz94b8/M7NwT"
			"ruthmSbdWStaVycIwpD2Q7za7f3vJ/Q3LiVIEKGyF3Eq3PqW7dVzu6C/xZCkGB8igyI3bozy"
			"9ntHeOudw4Qhf3j85Lk/hMgQdPrC2D/t65LTN0fG/4+145PmxvVr6e/vJZtxMIzWmxP8ij4o"
			"qj9rC8MQ1/MoFktMz85z+ep1Dh89xcjoWDkMxb/Ye8/Zf3n8pCpbxdW/87Uv7skXC9/UDf3R"
			"3p5u+nq6yWSdZS87+hV9MOT7PvlcgemZOW6MjDEyOk4I72iB//eOnLpQFSO3hKtf+9rXtKnx"
			"61+am1v4quuWdyLEQ+9f039Ft5TC8CCCy2Egvr13/9nvvvDC0txj/z845sVouorYCQAAAABJ"
			"RU5ErkJggg=="
			)
		self.close = PyEmbeddedImage( 
			"iVBORw0KGgoAAAANSUhEUgAAADAAAAAwCAYAAABXAvmHAAAABHNCSVQICAgIfAhkiAAABkJJ"
			"REFUaIHtmWlsFVUYhp/L3drbVrGCgYIggiC0QQo0rVoWI4ImFAuUH6KBQE3URHFJVCLQUjAK"
			"JoAQt18GE9zKYgSNkWpSl0BxKbhWbKlGKCgFQmlpe9fxxzfTOzN37n7/kPAm82fOnXPe95z3"
			"fN93zoWruHKxHVAy9BwChlyp5LXnMDA0WSK2FMmvAuD6AljxEmTlQiiYfE8OJxx4HY5+qb1p"
			"AhYAnYl2kawAHfkRULMXJpaC359kNyrsDug+Dy8vheYG7e0RYD5wLpEukhFgnPmafTB+OvR2"
			"J9GFGQo4s6D/Mmx6AJq/0BoSFmFPcKQw+fwCqM0EeQAbhALg9kDZ/XDiKJxpBxgJzAI+Bnpj"
			"9ZCIgMiZn1CSAfIa9CIq4MQxvYgZwH5iiIgn4FXgSUDIr9sHE6ZDXw/YUtn/0WADJQTu7KRF"
			"xBKw3UC+RiUfDEAgAE4n2O0yeykFMxWD7ODKBr8XHC5wuaG0AtqOwb8GEZZ2iiYgPPP5w6H2"
			"I/G83wcN78AbT8B3n8L4EsgfAQFvaiIcLggG4e3VsKsOTrZAYTl4cuH2BdDWrBcx00qElQDj"
			"zGvkfV7pbH0lnO+A020SNabOkd8lK8LhktyxrRoadsLF/+DP72H4zXDLNFndsgVmO83EZKdB"
			"+i6JiPP7ZJZ7L4nnff0Q1MX8062wYSGcPA7ZeUhCTYL8lpXwzR5jW18P2AbJWNm5sPo9mHqP"
			"1lqmChimvdCvwCqgFoAhI2HdXvF8bzcSKUJip4APfj8U/qr7AhxtkJXIT2Al9OS/NZEvmgHL"
			"N8j3iiKT5fZYrcStwPtmAUuBUtzZsHY3FJVDb5eJjALT5kF/D7Q0GUU0N0BxHBEa+a0W5AvL"
			"ZdycwRD0qd9rITYHSufDD5/BxbMAPuBNMFpIvJGVAwXjwNsbSSIUkqWt3gyLnjG2dbRCXSWc"
			"imKnWLYpUsnnXgf+ftO4NolQeflww2gjV5MAgaKITaIhFFRFbIoUcboN6iz2RDzbrNkNuYPB"
			"ZzFpGoIBeUyIFJAIBkRshkVPm0SoG/vUcalSHc7otikqhzX1Kvm+6ORjIDUBAyL6oPqVSBEd"
			"rbISZ9rB7hTyZtsU3qmb+dTIg4TO1DEgYrMQ2Lc13NbRCi8ugWFjoGm/8bvCclib3sxrSE8A"
			"GPcEGEX8/Ys8egzYJj+25xNE6hbSQy+i6lmxjRWm3A0v1Eu0yQB5yJQAkAgRCsGD66BgbGS7"
			"0w0P1cLQ4eBNzzZ6ZE6A0yXhd1u1hFEz/F7Y8Qi0/5pc2REHmRHgcMnsb1kBX+8mKrl/WiQ6"
			"RUt2KSB9AYYktdfYVjwHKlcZ30VLdikiPQGxMmzhDHj+XXhsOyw2Z2xdsktTROoC4hZm9ZBz"
			"jZTiK6PVTumvRGoC9OQjMqyuMPP1qQVgn1o7RcnYaaxEpACbTUJePPJWVaWBvC7OG2onCztp"
			"IrKiiVAkt9gj865egAuQA8xvh6SsNsPukPo8Vj0frarURDwcw04dx6X2N4hQ5NB/QT3G6rli"
			"PNDcCFQQCsrBYdREGFus1ufIMc9mgx2PwlcfxiAfI0kpiggpuTf6oaisAjzXyjULCrg8cOmc"
			"1FXtP2m/bgT2mAX8iKzILAJ+OHIARk0Ki3C64a+f4a2nLMjXhz0fL8MqiuyLkvusReQPg9tm"
			"y5huD1zqFPJ/DPyuAVgGeM0CNGU6EZ/ISowrliwb8MLBnRBQD0RFM5IjbyniMrQcDrfNWQY3"
			"FYpdu9SZD4s8CCwGBq4Fra5VGlUmsw0ixkyWA8roQui5AJPvgsdfk6Oez3wMTELEtLmQ5ZFa"
			"at5KmLscHBr5KjP5KqBH302sUdej3VJkeeC5XXBHpXrctMl+8HvVa5Y0CjObTayihGSfKQp0"
			"dcLGKr1ttJnvMX8e62qxEf1KNB2A0ZNgdBH0dYuNlFB65DUE/LICdt3MGz1vST6eAKOIoM5O"
			"+uiUESiyCl2dVp6PsI0eiVyvN2K1JzImwhQqI8nHvMdP9A+ORixFTJF9kDJ05DdG2CYueUje"
			"wHVADRDe2DMXQkAlkyzsNjjbYUV+ETFso0cqOzAswp0jpbInL4V/KW1yZ9T4gdxKC5Iinw7q"
			"yPz/xJ8DeckSSXQPmNGIZOwydfBQmk8DsIQEPH8VV5Fh/A9SPBbYUTXPbgAAAABJRU5ErkJg"
			"gg=="
			)		
		self.load = PyEmbeddedImage( 
			"iVBORw0KGgoAAAANSUhEUgAAADAAAAAwCAYAAABXAvmHAAAABHNCSVQICAgIfAhkiAAACE5J"
			"REFUaIHtmX1QlNcVxn+7sMsCy/Lt11iiEEuomFpNHag6Gh0VlTp+jNZYraZRJ1PGpGPEGAnG"
			"aKJx1DQ0bRyksRatbXEwiWKMgcF2xlgjNsXOFortYKvoDi4NWBaQhd3TPy67LiywuyiaP/LM"
			"3Nl573vPuc9z7z3n3vduEIOLMYAJaBrkfgYFwcBl4I+A5tFSGRhyAekqLzxiLgFjNtDOPQEt"
			"wNRHysgP6IHlwKeAnXvkXeUucApYjFpeXyksBirwJt1X+TMw/5Ew7YExwAm6iGk0Gp/ke7Q5"
			"Box+BLwBWA80AGI0GmXVqlUSFRXlU0BcXJysXLlSDAaDq84CrBooCe0AbOJRI5cPxM6cOZNL"
			"ly6h0+loavKd7hsaGoiJiaGiooL09HSAYUAh8CsgagB8AsL3gGpADAaD7NmzR0REjh496u/a"
			"d5eTJ0+K0+mUbdu2iU6nc9VfASYMFvn1QCsgqampcuHCBRERqampEZPJFLCA+Ph4qaurExGR"
			"8vJySUpKcr27A/zwQRLXAvtdHS9ZskSsVquIiHR2dsrUqVMDJu8qc+fOFRfq6upk/vz5nu9f"
			"fxDkTUCxy2lubq54IicnZ8DkXcW1DEVEHA6HbNy40fN9IRA6UPLfAD4DJCwsTI4cOdKNfFVV"
			"lSQnJ0tiYqIkJiZKUlKSJCQkiFar7ZOsVquVhIQESUpKksTERBk9erSMHTtWamtru/nOz88X"
			"vV7vsisDhgRKPgWoAWT48OFy7tw56YmWlhZpamqS5uZmaW5uFpvN5jMeTCaT1NTUiM1mc9s1"
			"NjZKW1ubl//Tp09LTEyMy/avQKK/5J8CbgKSkpIiZrPZy3lfsFgsEhkZ2aeAyMhIsVgsfvur"
			"qKiQUaNGuez/BaT6Iv80UA/IpEmT5MaNG353JqIC0ZcAV+bxF1evXpXU1FTPTe+p/gRYAJk2"
			"bZo0NDQE1JG/Am7evBmw3/r6eklLS3P5ueZJuOdOHG4ymTh+/DixsbG+ZssLRqOx3/cigsFg"
			"CNhvSEgIdrvd9RjXX9tGnU4nBw8eDHiUqqurZd26dRIUFNRvFlq6dKl7E/QHHR0dkpGR4emn"
			"sV8Bro4KCgr87mD79u0SFhYWUP5fu3atNDY2+vSflZWlbLRaIVjvnwBQx15fImw2myxYsMBN"
			"asqUKWI0GvskHRERIQsXLnQfqSdOnCjXr1/v039eXp6y1WiFNW8IsSP8EBAZL6R936cIu90u"
			"mZmZAkh4eLgUFhaK1Wr1zN1eJTo6WpqamqSsrExGjhwpgIwbN07q6+u9/JeUlEhwcLCyXfW6"
			"cOymEBHjj4A44XCtMP95tW77EJGbmyugvgXKyspEROTWrVt+p9Gqqip3jl+0aJE4nU63b7PZ"
			"fG8gpi8XPnEI79cIxmg/BBijhUP/FM50CnOe63UmKisr3UdgzyNGoPvAxYsXJTQ0VAApLi4W"
			"EZHbt29LSkqKsklJE443CKftQkF1AAIKqoWSduFkm5DhLWL16tUCSGZmZrdZqaur83mU6LmR"
			"bdmyRQBJT08Xm80m8+bNU+2HPKZGveSu8FHLAAR81CJ8aBNOtQpz17pJbNq0SYYOHSparVbO"
			"nz/vJaC/IDYajV4CLBaLREVFiV6vlxkzZqi2hnBhT7laBcVf9iug/+sNR4dqkvVLCDXBibfZ"
			"t28fAOPHj3d9EroRHx9PaWkpnZ2daDTdL+NEhODgYOLj47vVDxs2jNmzZ1NUVER5ebmq/Mm7"
			"8O3p0HIHX5d6vu9nHJ0gAs/tUc8n3gYgISEBrbb7Rq7X60lLS/PpsidSU1MpKipSD89shdlr"
			"oOV/ftn691HvdIC9TYlY8hIAn5w5456N+4HZbObAgQPqYfozsGIbtNlQq843/L8hc4t4CwB7"
			"8X6ys7MREbKzs6mvr2fDhg1cu1aLpo9pF4Tk5CfIy8sjNjYWq9XKsmXLsFgs8EQaZP1C9eN0"
			"+E0rsCs+pwPa2+DHSgTF+9m8eTNWq5XS0lIqKyt9urh8+S+YzWYKCgrYuXMn1dXVMPQxyP4N"
			"hIZDeyuBXGYHfkfpuZz0BvjdLvbu3avejUiCNW9CeCQ4eoxiUDDcscKhV7hy5QqTJ0+mo6MD"
			"oobApsPKtrU5IPIDEwDgyjChEaDVKlFPToeN78PwROjs6KM3HTw+AfY/S0fNJVU3cQ5MmA7N"
			"tgFRCfBmTtSoOxyQtx4ObVHkZ62G105A7AiwNcLdlt6LrRFGPA47TsHUpcrluWPwTpbKdLoQ"
			"/A3egQkIjYAvLbBzMZw9pOp+tAN+ehCC9GC/S/9LQAPtLWAwQnYhLNusBuDUe7BrOTT/V70b"
			"FAHhkVBTATkZUFmunl8+CiteVcQdfSyb3kR02lX7Z3fDi/kQEgYVH8PWDLj2N+X7gQnQaJXD"
			"P/0BXp0HdVdVwO0ogadXqN3S6fS7QzecDmhrhrnr4LUPIG4k/OfvkDMPLnyo+tT4Duj+BQTp"
			"ICQUfr8b3loBLU3w5DR48yykpHVt9fcBEeXjOzNh11lIngR3bsMbS+GDd9RyCuo/z/QtQG9Q"
			"x4ifPw+Hc1Rns9bAtq7Ramu+P/KeaG3uHtxOB+RvhAMvqJjuJ7h7FxBqVMG6Y1H3YH0xH4L1"
			"ah940P+cdgvul1Xdqfdg9w88gttbhLcAnR5qPodXZsGVc13B+lsVrB2BBGug8AzuXSqzhYTC"
			"pY9h6xyorYTgkN6suqERXUgUGo3KLGMmwku/hm+Ng1ZRy+hhQKOBcA188Tn8bC382wyG8K5T"
			"wN0mINrV1DtCOtrVry5EBdUXn8JnJx4O8Z4IM8E3vws3/qE2wl7QcwY6gaBBJ3Z/cOAx8D1n"
			"4F0gnED384cHDepf/6/xlcH/AUnBf9r7HCAQAAAAAElFTkSuQmCC"
			)
		self.info = PyEmbeddedImage( 
			"iVBORw0KGgoAAAANSUhEUgAAADAAAAAwCAYAAABXAvmHAAAABHNCSVQICAgIfAhkiAAAB7xJ"
			"REFUaIHt2nuMXHUVB/DPndmd7rZrtw9oKdC08mqxIBBaKEIqUtFiAvWBwfAQDRpEKKUoNdUG"
			"8A8FEgOKWGiCIooGgorKI0KoYUUIIEWkvFpqC7SFCi3Q175nxj/O3t3Z3Znd6bJgSfwmk7n3"
			"9zvz+53zu+d9hw84kmFcaySm4CBMwgTUoIi38Tpextqu+z0CY3A6bsVqtAqGK33y2IR7cDEO"
			"eP9ZDhyEa7BBJWZrc0V1o4pydQMJtB234/ihMrK7KjQai7Gg6zpQ38CBRzH9WKYexoQpjGok"
			"W0OxQMtOtmxi42peepLVT/DW5tJ1i7gTV+KF90qAk3A9ZnSPTD2cT57DrFOYdCC5OgoFCp3x"
			"3b1LhkyWbJZ8J1tf47m/s+I2nvxL6R7bsBQ3DLcAl+Jq1IL9DuaMJRz/uTjp9lY62ykWu1ZN"
			"qB1BpoZino623gJla8jVhzDPPMTvr+WpB0r3uw0XYOdwCHAtFnXfnXohZy5l7IRQjVLGFKnJ"
			"xYmvX8XmdYwez8EzqRtJW3PvLZOEESPJ53nwVm75LjveSmebcAb+824E+DEWglFjuOgGPnFm"
			"MNLZ0Z+6ppbmHdx0CY/cFU8FDpnFN66L77Zd/bdNMowczdqVXHse6/6VzjyOeXhnKAIsFp6G"
			"MRNZ8luOPImd24TN9V0pidO/5mwe+UP/+b3250dNsVa+jPBQN4p33uSqL/Hsw+nofThNuOB+"
			"yFZg/mT8EomGsSy9kyNOZNe2CuTCgF9ayc2Ly883b6dx7ziEjrbyNJ3tjGxk9qk89whbNsLB"
			"wvZWlPtJpszYWNyEjEyWi28cnHlCfTatGZjm1Rd6DL0sEtqb+dA4Fv+KvSenE98RXrAqAa6U"
			"RsjPL2LOGeysqII9KBRoGDswzehxoWoDIgk7mXQgF94QHiv4vA51fan7CjAD54MpM8JVtg7q"
			"yQLtrRw6m4lTK9Mc99nK+t8LSajc7FOZd146+FGc25eyrwCXYAQ463IaGsNXV4NCntF7seDG"
			"+O7FT8I5V3LYHNpaqluPoD39stL1vi2Sxm6UGvG+WIY604/lKz+gvU1Zj5OiWAzjzdVTm4ux"
			"D0/j+NNpnMD4fTlqLuddxbxzad418Hp9UcgzbhI7toZRMw5P4/mUpFQhL+gSgIXLmfe1eIwD"
			"IVdH0x08vYIkPYtiROHaEcFAJhsxo76BLy6O8WJhwGV7oXYEr6/nkmMjcPJnzE+na0pITwON"
			"e3H0p7ui5mCL5yKnefDXg9OOncgXFpHU7dZD0NEWqcvhc3jiPpgjao036LGBcZgJZpzA+P2q"
			"0/1i12lXg7pRhlw/ZbPMnJfejZHyqkeA6QhL+cjHyJbzrhVQKBsghxed7Uw7JnWpMDu9SEem"
			"dRNPmVG95ykWaRgT6iEJb5PvZPuWQQLWbiLfGanIuEm8uYGIzugRYCqorQvCfJWn2rqL+Qv5"
			"zPkhQG2OzS+z5OSYGy4U8pFMjt0nFaDbe6YChPo0NIa3qFYtisVIk+sb4r4215VyDGevQBh9"
			"toYxE0pH0CNAKH2SjdR2d1AooMstZjLVq9/uIkkYUd9vuAy3w3x6w4lC//iRChDxvb2ZzrYq"
			"Eq73GYkIfjv7t5NSATYi9Hf71oieexQS2lt4p7u6zPS9WIcwytfXlfrbPQPZLG+/Ga2ZQHfh"
			"nArwAiLPfWnlnvcEanJsWl1aVD2VXqQCrBc9S1Y1RdK0u97ovUS2hlV/S+8KeCy9Sblsx0Ng"
			"/TPREqk2x3mvkWSiInz8nnRkjZJ0uvSYf4fw40239+T3/2vUjYzm14YX05E/StVdbwEexirw"
			"19/w2r+rFKKrnZLWADUVfpN0pRrddLVVLJ2E7793eTrSgV65e6kAHaL3Gd2xP11Prlf1VmGD"
			"zvBcm9awcQ2b15cvWDo7eG1t0G1aEznNYPGmroEn7mVld9vxbiXqQ/+wW4+VOFSujh/eH+l1"
			"S4XCvqY2XNu3Ps7OrTFWLJZPJ5IkeqXpjofM4uoHe/dUS5GtDd9/2Ym88hxxwMeIkrIbfV1N"
			"i+jIRZdh2YIwoMFUqaMtTrizo3IuVCxGRyKlS9uO5ZDmPbcuTZknelVP9yUt5yvvEV051j3D"
			"zy6KJG+g4JYZgssdyE2PauTuZaW6vwaXl926whKLpAbddAc3XxaGV06IYnFouX+59kqSRIF0"
			"/y9Yfmk62oqvqtDgHciKponYsA+iwXT+daFO6eaZDC27ohPd3lJ9ElgsRHFy3Pye2iNbE3Xz"
			"vTex7OJSVfyyPp6nWgFglrD8ieDIuVz4UyYfSsv22DzJxMa7m8Hm812t9gz1o2jZwW3f566f"
			"lFItlHrGIQoAR4kXcYcg6t+zrmDu2RFkWpuHVtinhppk+OeKMNg1T6azrfgmbhl0mSq32xc3"
			"45TukWmzmL+AmafEW5h8Bx0dERfKucUkCWZrcl1q2MyLj3PfcpruVFIlrsXXpanNMAlAGPxC"
			"fA/ju0f3nxYv+Y7+FJOnR92avmZKVy8U4intfDuC3rMP89jdPP9o6fpF/BxLsKVapoZSeh0g"
			"mqznoKHXzJgJ0Z3ee3Jcp66yeRtvbOCNV+LT/wk9gKtUeerDhYNwBZ418Nv5Sp9XsRwnvBsm"
			"hqP4rcUROE54rfS/EqNLaFpEL/NVUYw8in+I98L/xwca/wUbx3+8rJ4S/gAAAABJRU5ErkJg"
			"gg=="
			)
		self.save = PyEmbeddedImage( 
			"iVBORw0KGgoAAAANSUhEUgAAADAAAAAwCAYAAABXAvmHAAAABHNCSVQICAgIfAhkiAAAB6dJ"
			"REFUaIHtmG1MlNkVx38PMwjDoFGjjW8U1DXBVbGRgigZY62bNVHaBBupdgV1UcO2VLfNKhOV"
			"BnzBVNQg2vKipdYWSi1rqxu6Wy1YPxh1m2gqLI4imrRmIllZGRBWmOH0w+MzzAAz84Cj2w/7"
			"T24mc++55/z/9z733nMvfA0v/Bh4BPxniPIIKAx2QGOQ/Y0FpvhpnxjkeIQE2Z8zQHtvkOMF"
			"XYAhQHuwZzxomAQcAB4D4qe0A8XAzK+G5mAYgJ8C/8U/8YHlKfALIPL1U+5HPPBPjVRycrKk"
			"pKT4JZ6cnCwrVqzwrLsFfOerIL8d6ARk/PjxcuTIEXE6nVJUVORXQFZWloiIVFRUSHR0tFbv"
			"RP38Ql8H8W8Af9IIpaSkiM1mEw0FBQV+BWzcuNFta7fbJTMz07P9H8Abr5L8IsAGiNlsluLi"
			"YhmI4QjQUFVVJRMnTtRs7MD3XgX5TUAHIHPmzJHr168PIjJSASIiNptNLBaLZucCcoJJfr9G"
			"IDU1VVpbW4ck8TICRES6urokKyvL0/43QNjLEDcBv9cc5uTk+AyuobCw0K+AzZs3B/Rx9OhR"
			"MRgMWp9PCJB+KD7qJwCVwFthYWEcP36czMxML4OioiLq6+sxGtXD1Wg00tjYSENDg89gMTEx"
			"JCYm4nK5AOjr62P27Nns3buXkJD+pOD8+fNkZGTw9OlTULfaHwD3/QnxigN8Csi4ceOktrZ2"
			"yJG6efOmGI3G4RxeQxZf/m/cuCExMTGaXQvwLT3k3wA+AyQqKsrnYtVQUVHxUuRzc3P9+m9u"
			"bpb58+dr9q3AQn/k5wAPAZk1a5bcuXMn4PcqIpKWljYi8snJydLT0xPQv91ulyVLlugSYQUk"
			"Li5OHjx4oIu8iMjjx489T1VdZcyYMdLY2Kg7RktLi5jNZq1/hS8BHwCSkZGh27GG+vp6zwB+"
			"i8FgkDNnzuj23dfXN3CWT3qSHnQfqK6uprS01JfIIaEoCpGR+hJLgyGE1tZWenp6dNlbrVaq"
			"q6s9q8SX7Qd4jFRJSUnA0XG5XJKTkyPKCNbA0qVLpaWlxa//U6dOCSChoaGSmJio9S33K2D0"
			"6NHuIIWFhX7Jb9q0qZ/U2+8KO/8ghI7yTfz9U0L2r4XIcQLI9OnTpampaUj/dXV1Eh4eLoAc"
			"PnxYSktL9QnYunWrHDhwwB20oKBgyABWq1W1MRiF94qFv7mESyL8KHdo8m9vEj7uE/4uQvGn"
			"wjffFEDmzZsnbW1tXr7v3bsnkydPFkDS0tJERD2hdQnQcvbc3FyfIurq6kRRFLX9vWKV1IdP"
			"hXMO4S8dwlyLN/lpscIf7cJfO4WaNqHWJZR/JkyYJtqgaWhvb5cFCxYIIAkJCeJwOETEK03x"
			"L2DLli1uZ3v27BkkwuVySVJSklq/7B115D9sF2q+UMtHXwq/uuX+TDCGCvs/EWp7VfI1X6i/"
			"H/cJu/8sKCFiNBrl9u3bIiKyevVqAWTatGle27kvAX5fCfLz81EUhfz8fKxWK2azmYSEBK5d"
			"uwYRY+CHVnD2gPT1d3reDTPiIGMvnPgJpP4M4t+CZ+30p14KdDlg0fchYQXOG7WUlpYyadIk"
			"ampqMJlMVFZWEhMT449eYAEAeXl5bjHbtm1j5swXDwqJKyEqViUyEF0d8N13oLsTlq+HLzsH"
			"24ioelZkwo1aysrK3FtrSUkJFoslIHnQ+S6Ul5dHbm4uIkJzc7NameTn4qTNSOr7YIqEF9nn"
			"IDzvhjcXw4SpbvK7du0iPT1dF3ndAqBfhBsP/g2KAiEGQCAkBMLN/SUsQm0fZfKuN2iTLhAa"
			"Bk8eQe9zANasWcO+fft0k4dhvpR5fk5UF6jf/8b9oITBMwfcqgOn0/ctQwmB2IUwZgL0uaDj"
			"CRx5F9o/JzExkZMnT/roqA+DdiFfOHjwYP8Wufrnwu/uC/Fv6zuFZ31b3UJrncLCVQLI1KlT"
			"AyaQI9qFfGHnzp2EhYWxc8cOemoOw8XfguMJ5ggTcXHzURR1jXpCURTu3r3L5/f+BbkpMGsB"
			"XP+IiIgIqqqqdO04gaB7BjRs2LDBPbIzZsyQq1ev+rVvamqS+Ph4dx+jwSDV1dW6YvmagRG/"
			"Tp84cYJz584BYLFYqKurY9GiRX77xMbGcunSJVJTU+EFG5vNNlIKg6BrBpxOp2RnZ7tHMT09"
			"XZ49e6Z71kTUHH/Hjh1uH+vWrXOnDMOdgWEJaG1tlVWrVrkD5+fnD4v4QJSVlcmoUWr2mpSU"
			"JPfv3391AhoaGmTu3LkCSGRkpFRWVr4UeQ0XL150Z57R0dFy5cqV4Au4cOGC+/1Sz2IdLmw2"
			"m3txm0wmOX36dPAEHDt2zP1SZrFY5OHDh0Elr6Gtrc2diQKye/fukQnQnv96e3u9FmtGRsaw"
			"F+tIMHBxd3R0iIjIoUOH9AnYvn27dHR0yMqVK4O2WIeL8vJyr8Vtt9v1XymXL18uixcvFkDG"
			"jh0rZ8+efa3kNVy+fFmioqIE1Cf9tWvXDilAGSDgl9qfKVOmUF5ezrJly+ju7kYG5gavGCaT"
			"iYaGBtavXz/wsDsJbNb+eOZCXjlkZ2cn2dnZdHV1vVqmfhAeHo7DMejC5JU9eApwAu7XJofD"
			"MVTn/wfoexH7Gq8J/wPD4oC/uoxCbAAAAABJRU5ErkJggg=="
			)
		self.web = PyEmbeddedImage( 
			"iVBORw0KGgoAAAANSUhEUgAAADAAAAAwCAYAAABXAvmHAAAABHNCSVQICAgIfAhkiAAACqtJ"
			"REFUaIHtmnlwVfUVxz/3viUvLy8kGJAlCfsWEBBwIRIFKtaliA64wShaEWVciopVqp3BsbVq"
			"i8pUWxCqo9alMhZUqrggWBQwIgKCEDZB1rCGJC/rW27/+L7lvpcX8lDsMtMzcyf3/e79nd85"
			"53d+37PcwP84GaeIT1tgADAY6At0AHyAFVmjHjgIbAW+Ar4GvjtFa39v8gHXAQuBQ0jYdK8q"
			"YClwO9D+PyH43UDZSQrd3HUAeBTI/3cIPw7YdIoET6XIHScrULpnIBuYCdza7BtOFwQDYDrg"
			"mgegsDeELa1QUwlbvoClr6az1mJgCrA7TdlapM7AClJZzTQthl5ucddsizYFGrvmAYsllsXi"
			"kMXioK73w7pGjtc7eR0tLrzewtuqud3YBZyXjnBmC8/PAJakZFZ8Bfz+E3jsHagohyN7oWgo"
			"jH8Qqv1QWwW11bpqKqGxDiY9Ae27QMVBGHIxPLcOLrsVHK5URnsPuLQlBU7kQp2Aj4EeCaN5"
			"HeGWP8D5V0GGGzavhmnng2XBYx9B32Kor0nNMasVfPIGPD5BfP60BvLaw1dLYe59sGNt8gx/"
			"RInPmhPS0cy4F1iEsD2iqgFn/gQemg8DR0hIwwnzpsHOr+HSyXD57VBX3bxJgo3QdQBs/VLC"
			"Olww8EI4vRNccJV25rtNYIWjM9wRBRYCFalYNudCTwJDE0Y8Prj1KejSG/zHweWBLaWwYiFk"
			"nwZj74bG+uaFB+0SwHUP6tC/Nw8OfKt5vtNgyiy5WCJ1AF4CmvhZcwpchlAgTk63LPvUzbBz"
			"M/hywOmEd+dAKAgX3gD5vSHQcGIFABrq5Gbnjobqo7DkZcj1gb8CnpoE+7aBw5k8qwSYlopd"
			"sgt5gTdRaiDq2AMeXgB7twoKP38HOveTUrOnCjbvfBZ8uVImHXK6wNdasFpxEDqdATN/Dus+"
			"ljvNeBOOH4b92+2zzgX+Bhy3DyYf4inA7Ngv0wEz3oKS0XCoHJ65HVYu1Hj3M2HbGhgxHh56"
			"Daqr7L57AjLAnQGuDJg6VEYxHRAOQa+zYdoL0P0M+HYT3Dccqo7YJ88FbrMP2HfAg3wtLzYy"
			"aiJcdR9UV0KGF0rGgjcbykrh4C69k98LCoogJw8ys2Vd0wTD1F/TKZdwe8TDNDT3w5dgzYdy"
			"TdMJo6fA3XMhLx/8VdoJh1PvxKkIeN2+C/YdGAO8HfvlzYFZK6B9t7hvmyb4WsGqd+HhK2S1"
			"KPU9DwaMEMq06wxZObKsFYY6v+LEro3wzQrY+Gki1LbtBHPWS8koEER35Z5hsHuTXYkZwCPR"
			"H/bTMt7+FiVjobBIASlK4TAEQlBbKeZFxdBjEHz2d9i0UleMs1u7EQ41RafMbCgZB6NuhBcf"
			"gl0bBKt9bfEyHJIRLrkF5t5rn3018DsgaFcgG7gg9ophwsgJEE5xKA1g/Se6P+9KmHg/XDNd"
			"vrylFL77Bsp3Qc1x4b7DJbdrUwidihSte50N7buC1wVf/EMKlJVC/+FAXXythlooHgOv/xaq"
			"j0VHi1DN8bVdgQFAx9jE/J7QYzA0JFnOMGTNnRv0u9tA8Acgu7UWKhkrdws0QH2tFHC65Psu"
			"t2KHFYZAo54bPugxRLy2r20KAsGAzkK/YfD5ouioAxgWVSAaB85MmNjnXGF9MkPTIUuU75C/"
			"tussIUNB+XRNZeS8GJDpg1Z54G2lHQ0G5I51fs3B0g4X9BLvfdsiSiUBo8MR2ZkEGhITKSpy"
			"wuMeg0mZJpkOqDwClYfhtA7Qqk3iQQZFWyus8VBQlxWOR2E7hYLKiTxZcHSfDGAmhaZgELr0"
			"S5ane/Qm6kId7U8p7ANuE8JZicwyXEojLKBtoSwcDqeJ/ynIdECbAshpA4f3aGcyfdqtmIQu"
			"KOgNHq8duWJlaFSB3NgEwxDchUNNUwN3piAQ9Gz1YvkzKaybDhmmdsYwZYjSRdCxZ8TFbErW"
			"+ZNT7txkVkv4ccrEH+s6EBW8SdaEYQrqPL6mruFwwqHdCix5+dC1f/r5T3NkGEpJqo/p7OW0"
			"TTxXhgGhEJStUiKYRFEFEpP4yTOVMSYHIK8HFj0Ps26BIT+F6S9ATQspdEvCO5ww/SJYvwxu"
			"fhzOuigRvh1OKXfbALsCMXmjCpTHJlhhhf1QijPQ6BJ8ggJVffCHnQFMMBrj0d7tgcaGxHUt"
			"JE9NQhIakzeqwJYEvnu3yDrJ0BcOCyVAaBQ9bKkgMh0yLMGkP1JsebMjqGbj53TCgZ3JxtwZ"
			"vYnGgcRitKy0Kb6DdqV1O90fKxfTVFW1YTS9UpHDobyq6qjynqwcKWAn04TtXyXPXJOswAbg"
			"cIICR/c37RaEQ5B7unbh6D7tghFJnV0ZStKycgS3jkgabTqVQmTl6LnbI6FAEHmsXC6Ul68M"
			"OPkAN9TD+qV2KSzU5gHiLnQMWI46b4q065fBqBug1hZUwiHlPe26KgE7uh8Ku0JlFezZDNvX"
			"wc71cGiPysUoZWZD2wLo0l/Za2EfyMoFp6nawLKU3GVmQ4MtzXZ7hFBb12CjsojBExQAmB9T"
			"AOCDF2D4dYlnwQprkS79pMDaJUrClryshdKNyJ37wdAxcMmkuHt0GyiXspMzAz5+BUIB++hb"
			"QCzS2Z3TF9Gsi54YMGMhnDPa1ioxoHUOvPE0PJeQoyu16H2OsLx9F7mDJ0sNrdoqxY+dG5R2"
			"741ghscr9/JXwKMfqG0TXcvtUcfi3mFyVVE9MAjtgnS0ieAH5gCPA7L6K49A/xHxBCvDC++/"
			"Am8/Q0zJ4ivhohuhzzkKQoYZSeYiaGIY8RLTsgSH29fCstdgxQJhvGHA1i9gwHAVQsGAcqDX"
			"f2sXPmr9MvtAcldiA3At0BqAYwckfPGlEAjA89Ph+QfE1OnWjLtmw9CLIUS8Fgg2attDQf0N"
			"NsbHHU7o0B1GjoWQpU4EwLqlsLsMBo+CNnmw9A149RG7bHXA9djBJoUCDShIXB0b2bxKCdaC"
			"WbB4ntxi6nMqejZ+KkXO+lnLTa0oWZE6IGzA/CfUOrlkEvgrYdMKlaU57dUBqffbZ85EbZUE"
			"aq43+iowoclofk8Jf/ZI2LgaphYLlZ79UrVBMNCUUypye3Qe7hmmfulfyqC6Ap68KbGujtNa"
			"1NyqTX7QXGvxTvQhI5GunApDR0JFlSBx0CgVOMvf1PlIl1xu+OhFuVfJOBmhW0+46VHFk0Sq"
			"ACamEh6ab+7WA8sQrPpio2WrwJmlbDUjA1yZsHw+7N8GIyaoYdUSlLoyYN92dfXCIfjFHOiQ"
			"D8vfgT9OUVSOUwB1S066Ow1wBPgncEVMicZ6FTG7N0OnvjBohDoUO9ZBblsYOLLls+DxwV9n"
			"yN8vmwwXjIN5v4K50+I5kagRuBFYcCJ26XxiGoi6YUUJo1m5MOYOpRbzfqne6NMrFQ8a6xIT"
			"MsMQjHqz4ZuVcP9IjY+aqIh/YEfymkcjwr/bknDpfiM7Hfgz9kgdJU+W4DEYgOHXwh3Pyk3c"
			"njj7QIMQ5fhBmHUblH1+orVKgcnY0oVToUCUJgEPAwXNvpGXD23ytSNR9nXV+gx1eM+JkKoa"
			"mIUCacoDe6qoHfAbVJeeivrWj7rOvX9MoVNRB+AudNBrTlLoRpTT/xro+UOEOFX/K9ETdcvO"
			"Qh8FO6I2vRMIIywvR59PvwRWAxv5/rXo/+m/hv4Fa/9JfZi5CcEAAAAASUVORK5CYII="
			)
		self.settings = PyEmbeddedImage( 
			"iVBORw0KGgoAAAANSUhEUgAAADAAAAAwCAYAAABXAvmHAAAABHNCSVQICAgIfAhkiAAACFVJ"
			"REFUaIHtmXtwVNUdxz/33n3mCQkBCZiCARE12BJUkBImaEOhhZampaMWpZ2xWBoU6gNa1JKK"
			"Oq1aHzQQi7Y4xGcREzsiCpSxI0krVrDyFOQtEsIjkGSzm91s+sfv7u7d3XuTzYYw7Uy/M2dm"
			"99xzz/n+zjm/54X/cWgXca0xwDTADXj09l+B64E7Ehi3FOjQ21ngfeA+4PLeo2YNBSgFNhtI"
			"DTQZ59SbHVhhGGtsXmAtUAw4kiHSXUwEHgVuiOlfCBwAJgBXIwK59GdBoD+Q3sm87cBw4GAS"
			"nBKCC3hCJ2O2kz1tz/YWcYAs4KNeIh5qNydDzJbguFbgCFDY6SiHC4YUwNCrYeBwyMyW/mAQ"
			"Guvh2GdwaAcc3gnB9ti3q5DruRA4l6gA3dWBT4BRcb35X4Wbbocxk6F/HjjcyKaaLOdthi/2"
			"wz/egk2r4csDsYM+BW4BdiRCqDsC3AosA/qGewYNh1sehBu+C+408HmgPQAdZuRDKypgc8hp"
			"nWuAjavh9d/K7whOIVbu7xdKgO8Df4nqmXIHzH4EMrKhtRk6gglOZYCqgTsdju+DFfNh6zrj"
			"0/PIldoOVCLXeEEyAlyHOB1XuGfOUzDjbvC2yI73FHaXMHl+IdREGaODepuk/58CrDcO6CqU"
			"6AO8i9hwwV2V8J0y8JwX5UwGmg3cqaDZ5RRsdlBVuHYK+L2wc0toZF9gqOHNscALgL8zAbKQ"
			"4wL4PVASfnJbOZTeA82NyREHcKWJRXr3T1D9DLy3Cj7eAP42GJgPhSWi2Ac/NXs7CwlD6kId"
			"sVfoCmAT8CpybdYSEvLaqbCkGnytyd13EEXfUg3PzYdTX0T6bQ74+TIovlUY+VrhnglwdI/Z"
			"LMeBEUAzxJ/Ar5AdH4c4FhWAlAxY/BqkZiZ/550psG0TLC2FFoOZH/0NWFIDo0ugrVWuZUoG"
			"5A6Dza+YzZQO7EeUWycoSAVmmi4+bS7kXQl+X3LkFVVM7Mp7IxugavCjX0P5WzBgCHgMQnmb"
			"oXAyjJ1mNePs0A+jAOOAQXFDUzKg5Mfga0mOPIDDCbvrxAOHoNmgaKYI4vcSvs2KIqdFB0wv"
			"s5qxCBgSK8BNpkMLJ8txBvymjxOCZocju6P7/D5YcbeEFIomhG0OsDvhnZVwdC8UTIDLohz/"
			"SeBJ4FtAQ6wA400Xv25q8koL+o7qVygW2zbCG09CWgakZMK5k/DYzVBRBoE2cLtgzDeNb5wC"
			"7gXWAS0QHcwNiVvA7oRho8XEJQObXeaoWQnVFtHySw/ra/hg+TxoOAZZA6W1tcPIqLTjcp3n"
			"ofAShoeD4ybvewlk5yZnedxpcPaEeNfNL1uP8/vgwW9Hn/L106BPDrT5xDfY7KErbOtMgCrE"
			"815FyPv1HSBBV3zoaw1VEy+7dT1UzIMTcdFmPIzks3Phh4vA55X+1AxIz5bNEORELWf4PQup"
			"GtSEe9L6iAJ2ElxGwemWSPSFRbKriZA3Im8kPPQm9MuF9jaZy5EiGxJBLpCBnp6aJTSRoE1R"
			"Ewv3FEXM7b5/wfK7xGRaITNH7P/JI7CrVmx+Zo6EEDfOkk3zeYgyq0oUiceAJYgSDzYT4HT4"
			"V3Oj3H8F61OwO0C1iZK++ICE1lYoLIGfPQuXjhCP2+aV+R1OmcPnkTDCSN7nAU/UnG69aWB+"
			"AvXhX431okiazVwCdzo0HIHKX0BdtTVxu1N2fcYCmScUDCqKkPV5zedXVYl6m86azXraSoDP"
			"wr/OfAlnjsMll4ldDkHTwJkKW9ZC5QI4dcya/NBRUFYBV40HT1O0wnaE8nkLaHbRo4BpCHPU"
			"SoA9gA9w0uaFA5/ApVfoAnSAzSmuv+InsOFF68VBYqjbfiP60ZJwnh6Bzd6ZPu2FaCsUwmHg"
			"4/C/D98WZQa5Ss1nYOGN0HIeFlXBqInxM/QbBItflxDZ7pTMrbsI3f+P1luN2ALmJzAUiGQs"
			"W9+Bk4chs7+YydVLRFEfegNUBQqKoawwYqfHTYc5T0t1ouV894mH4HDB7n/KDYhHAPgAok/A"
			"hiQyu5DcU9DcCO/9WWyxosKJg+BKBU2B9qBYoYBP+u58WnY+awC0NiVPHuT+Vz9jTFuDegO5"
			"IfshOqEJIo4svnh1aCd8vRQys8TMrXsOmhpBU2HlfRJFPrAGxs/QE/1ueG4zuFJhxwewarGx"
			"RPM7YB6St6xBT2hi3VQBsA2zXLloJvzyFYld1jwBf62QEymaCbPKwenSbXgPoWkQ7ID7i2F/"
			"WBUbkXS3Pna4mZ99DavMbM5TUDof/P6ISUzto6eCPdx1iHj0ZXPh7Urjk/uBx81eMatK/Bv5"
			"YBGv4Ns3Qe5wyL8G2v0SuAV8nVfiEiavCvmXl8KaKK61wE+xcBhmApxGTmZS3JOOoNQ0cwbD"
			"iLHiGy4EeZtd6qkvlUNVufFJAzAVOGP1qlVhqxb5YvIV/f9mIA9QCQahrga8HigoEoVrTzLd"
			"VFRISZe66LK5olcR1AM/QArKlrASIAhsBL4H/BEJtY8D08MjdtWKk8nJk+DM4ZYT6upEFEV2"
			"3JUGwQD8rQoev91YjQvhVeAPnU/WdbCcjTE6lSLv80Bm1KiCiVAyG64pllRQs4kgxrhHUYV8"
			"oA3qD4vwG1bB59ut1u5AwuZHEceVlABmuBJYjlSOo5GeJd8K8r8mlYyMbFH0dj80NsCxPUL4"
			"8+2Jhhd7ka+gSQRSnUMF7kS8YW99cloP9LvQxGMxqQcEm5DSeWx/AHiYi/ARXgXe7IKkH6nl"
			"NOhk6xCjMBuxaiN1wqHx7yMVwouCNODDLgSoRaoI/bD+iL0BCY1n9DJfU9iQkuQKROHMBOgK"
			"mV0PsUYyVsgKdmAYEhDmI05wH1LL/D+s8B+3MerH/E1AJwAAAABJRU5ErkJggg=="
			)

		self.icons = (self.group, self.emitter, self.particle, self.attractor, self.save, self.info, self.load, self.close, self.web, self.settings)
			
	def GetData(self, id):
		return self.icons[id].GetData()
		
	def GetImage(self, id):
		return self.icons[id].GetImage()
		
	def GetBitmap(self, id):
		return self.icons[id].GetBitmap()

# The UI class is the user interface built by wxFormBuilder
class UI ( wx.Panel ):
	
	def __init__( self, parent ):
		wx.Panel.__init__ ( self, parent, id = wx.ID_ANY, pos = wx.DefaultPosition, size = wx.Size( 800,600 ), style = wx.TAB_TRAVERSAL )
		
		box_TopLevel = wx.BoxSizer( wx.VERTICAL )
		
		self.bar_Toolbar = wx.ToolBar( self, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, wx.TB_HORIZONTAL ) 
		self.bar_Toolbar.Realize() 
		
		box_TopLevel.Add( self.bar_Toolbar, 0, wx.EXPAND, 5 )
		
		box_Main = wx.BoxSizer( wx.HORIZONTAL )
		
		self.pnl_CreateFigure = wx.Panel( self, wx.ID_ANY, wx.DefaultPosition, wx.Size( -1,-1 ), wx.SIMPLE_BORDER|wx.TAB_TRAVERSAL )
		self.pnl_CreateFigure.SetBackgroundColour( wx.Colour( 240, 240, 240 ) )
		self.pnl_CreateFigure.SetToolTipString( u"The external name of the new particle figure" )
		
		box_CreateFigure = wx.BoxSizer( wx.VERTICAL )
		
		self.lbl_Figure = wx.StaticText( self.pnl_CreateFigure, wx.ID_ANY, u"FIGURE", wx.DefaultPosition, wx.DefaultSize, wx.ALIGN_CENTRE )
		self.lbl_Figure.Wrap( -1 )
		box_CreateFigure.Add( self.lbl_Figure, 0, wx.ALL|wx.EXPAND, 5 )
		
		self.m_staticline1 = wx.StaticLine( self.pnl_CreateFigure, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, wx.LI_HORIZONTAL )
		box_CreateFigure.Add( self.m_staticline1, 0, wx.EXPAND |wx.ALL, 5 )
		
		self.lbl_FigureName = wx.StaticText( self.pnl_CreateFigure, wx.ID_ANY, u"Figure name:", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.lbl_FigureName.Wrap( -1 )
		box_CreateFigure.Add( self.lbl_FigureName, 0, wx.ALL, 5 )
		
		self.ctl_FigureName = wx.TextCtrl( self.pnl_CreateFigure, wx.ID_ANY, u"PPFig", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.ctl_FigureName.SetToolTipString( u"This is the name of the figure that will be created. The emitter will also be created using this name." )
		
		box_CreateFigure.Add( self.ctl_FigureName, 0, wx.ALL|wx.EXPAND, 5 )
		
		self.lbl_NoOfParticles = wx.StaticText( self.pnl_CreateFigure, wx.ID_ANY, u"No. of particles:", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.lbl_NoOfParticles.Wrap( -1 )
		box_CreateFigure.Add( self.lbl_NoOfParticles, 0, wx.ALL, 5 )
		
		self.ctl_NoOfParticles = wx.SpinCtrl( self.pnl_CreateFigure, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, wx.SP_ARROW_KEYS, 1, 5000, 30 )
		self.ctl_NoOfParticles.SetToolTipString( u"When you choose how many particles are in your scene,\nyou should keep in mind:\n\nIf you need to have a steady steam of particles coming from\nthe emitter. For example, if you have 3 particles spawned\nevery frame and a particle's lifespan is 30 frames, you need\n90 particles. If you only have one particle spawning every\nframe and its lifespan is 30 frames, then you only need 30\nparticles. (\"Rate of Emission\" times \"Particle Lifespan\")\n\nThe more particles you have in the scene, the longer it will\ntake to simulate, the more memory will be used and the\nmore disk space it will take to save your scene.\n\nYou can't change the number of particles later, so be sure\nto give yourself a margin of error for added flexibility" )
		self.ctl_NoOfParticles.SetMaxSize( wx.Size( 100,-1 ) )
		
		box_CreateFigure.Add( self.ctl_NoOfParticles, 0, wx.ALL|wx.EXPAND, 5 )
		
		self.lbl_Shape = wx.StaticText( self.pnl_CreateFigure, wx.ID_ANY, u"Particle shape:", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.lbl_Shape.Wrap( -1 )
		box_CreateFigure.Add( self.lbl_Shape, 0, wx.ALL, 5 )
		
		ctl_ShapeChoices = [ u"Custom", u"Ball", u"Torus", u"Pill", u"Sprite", u"Cone" ]
		self.ctl_Shape = wx.Choice( self.pnl_CreateFigure, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, ctl_ShapeChoices, 0 )
		self.ctl_Shape.SetSelection( 1 )
		self.ctl_Shape.SetToolTipString( u"The shape you choose here will be used by each one\nof the particles in the figure. Keep in mind:\n\nHow many particles are in the scene. A dense geometry\ncan slow down your machine if you have a large number\nof particles in the scene.\n\nWhether or not you want to use procedurals. Rounded\nshapes are very effective for procedural materials, while\nhard-edged shapes are better suited for transparency and\ntexture maps.\n\nHow disjointed the effect should be. Tapered and oblong\nshapes are useful for jumbled, frothy systems." )
		
		box_CreateFigure.Add( self.ctl_Shape, 0, wx.ALL|wx.EXPAND, 5 )
		
		
		box_CreateFigure.AddSpacer( ( 0, 0), 1, wx.EXPAND, 5 )
		
		self.img_Group = wx.StaticBitmap( self.pnl_CreateFigure, wx.ID_ANY, wx.NullBitmap, wx.DefaultPosition, wx.Size( 128,128 ), 0 )
		box_CreateFigure.Add( self.img_Group, 0, wx.ALIGN_CENTER_HORIZONTAL|wx.ALL, 5 )
		
		self.ctl_CreateFigure = wx.Button( self.pnl_CreateFigure, wx.ID_ANY, u"CREATE", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.ctl_CreateFigure.SetToolTipString( u"Add this particle figure to the Poser scene" )
		
		box_CreateFigure.Add( self.ctl_CreateFigure, 0, wx.ALL|wx.EXPAND, 5 )
		
		
		self.pnl_CreateFigure.SetSizer( box_CreateFigure )
		self.pnl_CreateFigure.Layout()
		box_CreateFigure.Fit( self.pnl_CreateFigure )
		box_Main.Add( self.pnl_CreateFigure, 0, wx.ALL|wx.EXPAND, 10 )
		
		self.pnl_AddAttractor = wx.Panel( self, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, wx.SIMPLE_BORDER|wx.TAB_TRAVERSAL )
		self.pnl_AddAttractor.SetBackgroundColour( wx.Colour( 240, 240, 240 ) )
		
		box_AddAttractor = wx.BoxSizer( wx.VERTICAL )
		
		self.lbl_Attractor = wx.StaticText( self.pnl_AddAttractor, wx.ID_ANY, u"ATTRACTOR", wx.DefaultPosition, wx.DefaultSize, wx.ALIGN_CENTRE )
		self.lbl_Attractor.Wrap( -1 )
		box_AddAttractor.Add( self.lbl_Attractor, 0, wx.ALL|wx.EXPAND, 5 )
		
		self.m_staticline2 = wx.StaticLine( self.pnl_AddAttractor, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, wx.LI_HORIZONTAL )
		box_AddAttractor.Add( self.m_staticline2, 0, wx.EXPAND |wx.ALL, 5 )
		
		self.lbl_AttractorName = wx.StaticText( self.pnl_AddAttractor, wx.ID_ANY, u"Attractor name:", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.lbl_AttractorName.Wrap( -1 )
		box_AddAttractor.Add( self.lbl_AttractorName, 0, wx.ALL, 5 )
		
		self.ctl_AttractorName = wx.TextCtrl( self.pnl_AddAttractor, wx.ID_ANY, u"Attractor", wx.DefaultPosition, wx.DefaultSize, 0 )
		box_AddAttractor.Add( self.ctl_AttractorName, 0, wx.ALL|wx.EXPAND, 5 )
		
		box_AttrStrength = wx.BoxSizer( wx.HORIZONTAL )
		
		self.lbl_AttrStrength = wx.StaticText( self.pnl_AddAttractor, wx.ID_ANY, u"Strength", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.lbl_AttrStrength.Wrap( -1 )
		box_AttrStrength.Add( self.lbl_AttrStrength, 1, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5 )
		
		self.ctl_AttrStrength = wx.SpinCtrl( self.pnl_AddAttractor, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.Size( 100,-1 ), wx.SP_ARROW_KEYS, -1000000, 1000000, 1000 )
		self.ctl_AttrStrength.SetToolTipString( u"An attractor's Strength controls how much it pulls on particles.\nA negative Strength value will push particles away." )
		
		box_AttrStrength.Add( self.ctl_AttrStrength, 0, wx.ALL, 5 )
		
		
		box_AddAttractor.Add( box_AttrStrength, 0, wx.EXPAND, 5 )
		
		box_AttrRange = wx.BoxSizer( wx.HORIZONTAL )
		
		self.lbl_AttrRange = wx.StaticText( self.pnl_AddAttractor, wx.ID_ANY, u"Range", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.lbl_AttrRange.Wrap( -1 )
		box_AttrRange.Add( self.lbl_AttrRange, 1, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5 )
		
		self.ctl_AttrRange = wx.SpinCtrl( self.pnl_AddAttractor, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.Size( 100,-1 ), wx.SP_ARROW_KEYS, 0, 1000000, 1000 )
		self.ctl_AttrRange.SetToolTipString( u"An attractor's Range determines how far away particles can\nbe and still be affected by the attractor." )
		
		box_AttrRange.Add( self.ctl_AttrRange, 0, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5 )
		
		
		box_AddAttractor.Add( box_AttrRange, 0, wx.EXPAND, 5 )
		
		box_AttrFalloff = wx.BoxSizer( wx.HORIZONTAL )
		
		self.lbl_AttrFalloff = wx.StaticText( self.pnl_AddAttractor, wx.ID_ANY, u"Falloff", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.lbl_AttrFalloff.Wrap( -1 )
		box_AttrFalloff.Add( self.lbl_AttrFalloff, 1, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5 )
		
		self.ctl_AttrFalloff = wx.SpinCtrl( self.pnl_AddAttractor, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.Size( 100,-1 ), wx.SP_ARROW_KEYS, -1000, 1000, 100 )
		self.ctl_AttrFalloff.SetToolTipString( u"An attractor's Falloff controls how the attractor's Strength\nchanges as particles are closer or further from the center." )
		
		box_AttrFalloff.Add( self.ctl_AttrFalloff, 0, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5 )
		
		
		box_AddAttractor.Add( box_AttrFalloff, 0, wx.EXPAND, 5 )
		
		self.lbl_AttrHints = wx.StaticText( self.pnl_AddAttractor, wx.ID_ANY, u"Values are decimal values x1000. So 1000 = a Poser value of 1.0", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.lbl_AttrHints.Wrap( 150 )
		box_AddAttractor.Add( self.lbl_AttrHints, 0, wx.ALL, 5 )
		
		
		box_AddAttractor.AddSpacer( ( 0, 0), 1, wx.EXPAND, 5 )
		
		self.img_Attractor = wx.StaticBitmap( self.pnl_AddAttractor, wx.ID_ANY, wx.NullBitmap, wx.DefaultPosition, wx.Size( 128,128 ), 0 )
		box_AddAttractor.Add( self.img_Attractor, 0, wx.ALIGN_CENTER_HORIZONTAL|wx.ALL, 5 )
		
		self.ctl_AddAttractor = wx.Button( self.pnl_AddAttractor, wx.ID_ANY, u"ADD", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.ctl_AddAttractor.SetToolTipString( u"Attractors draw particles towards them or push particles away." )
		
		box_AddAttractor.Add( self.ctl_AddAttractor, 0, wx.ALL|wx.EXPAND, 5 )
		
		
		self.pnl_AddAttractor.SetSizer( box_AddAttractor )
		self.pnl_AddAttractor.Layout()
		box_AddAttractor.Fit( self.pnl_AddAttractor )
		box_Main.Add( self.pnl_AddAttractor, 0, wx.BOTTOM|wx.EXPAND|wx.TOP, 10 )
		
		box_Action = wx.BoxSizer( wx.VERTICAL )
		
		self.pnl_Configure = wx.Panel( self, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, wx.SIMPLE_BORDER|wx.TAB_TRAVERSAL )
		self.pnl_Configure.SetBackgroundColour( wx.Colour( 240, 240, 240 ) )
		self.pnl_Configure.SetToolTipString( u"Change those the properties of the selected actor whic affect its behaviour in the Particles simulation." )
		
		box_Configure = wx.BoxSizer( wx.VERTICAL )
		
		self.lbl_Configure = wx.StaticText( self.pnl_Configure, wx.ID_ANY, u"CONFIGURE", wx.DefaultPosition, wx.DefaultSize, wx.ALIGN_CENTRE )
		self.lbl_Configure.Wrap( -1 )
		box_Configure.Add( self.lbl_Configure, 0, wx.ALL|wx.EXPAND, 5 )
		
		self.m_staticline3 = wx.StaticLine( self.pnl_Configure, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, wx.LI_HORIZONTAL )
		box_Configure.Add( self.m_staticline3, 0, wx.EXPAND |wx.ALL, 5 )
		
		box_Selection = wx.FlexGridSizer( 0, 2, 0, 0 )
		box_Selection.AddGrowableCol( 1 )
		box_Selection.SetFlexibleDirection( wx.HORIZONTAL )
		box_Selection.SetNonFlexibleGrowMode( wx.FLEX_GROWMODE_SPECIFIED )
		
		self.lbl_CurrentActor = wx.StaticText( self.pnl_Configure, wx.ID_ANY, u"Selected actor:", wx.DefaultPosition, wx.Size( 100,-1 ), 0 )
		self.lbl_CurrentActor.Wrap( -1 )
		box_Selection.Add( self.lbl_CurrentActor, 0, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5 )
		
		self.txt_SelectedActor = wx.TextCtrl( self.pnl_Configure, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, wx.TE_READONLY )
		self.txt_SelectedActor.SetToolTipString( u"The Poser scene's currently selected actor" )
		
		box_Selection.Add( self.txt_SelectedActor, 1, wx.ALIGN_CENTER_VERTICAL|wx.ALL|wx.EXPAND, 5 )
		
		self.lbl_ActorType = wx.StaticText( self.pnl_Configure, wx.ID_ANY, u"Type:", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.lbl_ActorType.Wrap( -1 )
		box_Selection.Add( self.lbl_ActorType, 0, wx.ALL, 5 )
		
		self.txt_ActorType = wx.TextCtrl( self.pnl_Configure, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, wx.TE_READONLY )
		self.txt_ActorType.SetToolTipString( u"The Particle type of the selected actor. Set to non valid if the actor is not a special type used by this script." )
		
		box_Selection.Add( self.txt_ActorType, 1, wx.ALIGN_CENTER_VERTICAL|wx.ALL|wx.EXPAND, 5 )
		
		
		box_Configure.Add( box_Selection, 1, wx.EXPAND, 5 )
		
		bSizer17 = wx.BoxSizer( wx.HORIZONTAL )
		
		self.ctl_Configure = wx.Button( self.pnl_Configure, wx.ID_ANY, u"CONFIGURE", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.ctl_Configure.Enable( False )
		
		bSizer17.Add( self.ctl_Configure, 1, wx.ALIGN_CENTER_HORIZONTAL|wx.ALIGN_RIGHT|wx.ALL|wx.EXPAND, 5 )
		
		self.ctl_DeleteKeyframes = wx.Button( self.pnl_Configure, wx.ID_ANY, u"Delete ALL keyframes", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.ctl_DeleteKeyframes.Enable( False )
		self.ctl_DeleteKeyframes.SetToolTipString( u"Deletes ALL keyframes for the selected actor" )
		
		bSizer17.Add( self.ctl_DeleteKeyframes, 0, wx.ALL, 5 )
		
		
		box_Configure.Add( bSizer17, 0, wx.EXPAND, 5 )
		
		
		self.pnl_Configure.SetSizer( box_Configure )
		self.pnl_Configure.Layout()
		box_Configure.Fit( self.pnl_Configure )
		box_Action.Add( self.pnl_Configure, 0, wx.EXPAND|wx.LEFT|wx.RIGHT|wx.TOP, 10 )
		
		box_Run = wx.BoxSizer( wx.HORIZONTAL )
		
		self.pnl_Simulate = wx.Panel( self, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, wx.SIMPLE_BORDER|wx.TAB_TRAVERSAL )
		self.pnl_Simulate.SetBackgroundColour( wx.Colour( 240, 240, 240 ) )
		
		box_Simulate = wx.BoxSizer( wx.VERTICAL )
		
		self.lbl_Simulate = wx.StaticText( self.pnl_Simulate, wx.ID_ANY, u"SIMULATE", wx.DefaultPosition, wx.DefaultSize, wx.ALIGN_CENTRE )
		self.lbl_Simulate.Wrap( -1 )
		box_Simulate.Add( self.lbl_Simulate, 0, wx.ALL|wx.EXPAND, 5 )
		
		self.m_staticline5 = wx.StaticLine( self.pnl_Simulate, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, wx.LI_HORIZONTAL )
		box_Simulate.Add( self.m_staticline5, 0, wx.EXPAND |wx.ALL, 5 )
		
		box_SimulateControls = wx.GridSizer( 0, 2, 0, 0 )
		
		self.lbl_InPlace = wx.StaticText( self.pnl_Simulate, wx.ID_ANY, u"Simulate in place:", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.lbl_InPlace.Wrap( -1 )
		box_SimulateControls.Add( self.lbl_InPlace, 0, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5 )
		
		self.ctl_InPlace = wx.CheckBox( self.pnl_Simulate, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, 0 )
		self.ctl_InPlace.SetToolTipString( u"This option causes the simulation to run on the start frame X number of times, and then proceed. You can use\nthis option to make the particle figure more fully formed than it would be otherwise. Typically, if you use this\noption, you should set it to be the same as the particle figure's Lifespan, so that the particle figure has gone\nthrough one complete cycle." )
		
		box_SimulateControls.Add( self.ctl_InPlace, 0, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5 )
		
		self.lbl_RandomRot = wx.StaticText( self.pnl_Simulate, wx.ID_ANY, u"Random rotate:", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.lbl_RandomRot.Wrap( -1 )
		box_SimulateControls.Add( self.lbl_RandomRot, 0, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5 )
		
		self.ctl_RandomRot = wx.CheckBox( self.pnl_Simulate, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, 0 )
		self.ctl_RandomRot.SetValue(True) 
		self.ctl_RandomRot.SetToolTipString( u"Apply random rotation to emitted particles" )
		
		box_SimulateControls.Add( self.ctl_RandomRot, 0, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5 )
		
		self.lbl_Frames = wx.StaticText( self.pnl_Simulate, wx.ID_ANY, u"No. of frames:", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.lbl_Frames.Wrap( -1 )
		box_SimulateControls.Add( self.lbl_Frames, 0, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5 )
		
		self.ctl_Frames = wx.SpinCtrl( self.pnl_Simulate, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, wx.SP_ARROW_KEYS, 0, 100000, 0 )
		self.ctl_Frames.SetMinSize( wx.Size( 100,-1 ) )
		
		box_SimulateControls.Add( self.ctl_Frames, 0, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5 )
		
		
		box_Simulate.Add( box_SimulateControls, 0, wx.EXPAND, 5 )
		
		ctl_RunForChoices = [ u"Selected particle figure", u"All particle figures", u"These particle figures:" ]
		self.ctl_RunFor = wx.RadioBox( self.pnl_Simulate, wx.ID_ANY, u"Run simulation for", wx.DefaultPosition, wx.DefaultSize, ctl_RunForChoices, 1, wx.RA_SPECIFY_COLS )
		self.ctl_RunFor.SetSelection( 0 )
		self.ctl_RunFor.SetBackgroundColour( wx.Colour( 240, 240, 240 ) )
		
		box_Simulate.Add( self.ctl_RunFor, 0, wx.ALL|wx.EXPAND, 5 )
		
		ctl_TheseFiguresChoices = []
		self.ctl_TheseFigures = wx.ListBox( self.pnl_Simulate, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, ctl_TheseFiguresChoices, wx.LB_ALWAYS_SB|wx.LB_EXTENDED|wx.LB_MULTIPLE|wx.LB_SORT )
		box_Simulate.Add( self.ctl_TheseFigures, 1, wx.ALL|wx.EXPAND, 5 )
		
		box_SimulateButtons = wx.BoxSizer( wx.HORIZONTAL )
		
		self.ctl_Simulate = wx.Button( self.pnl_Simulate, wx.ID_ANY, u"SIMULATE", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.ctl_Simulate.SetToolTipString( u"Run the simulation" )
		
		box_SimulateButtons.Add( self.ctl_Simulate, 1, wx.ALL|wx.EXPAND, 5 )
		
		self.ctl_Refresh = wx.Button( self.pnl_Simulate, wx.ID_ANY, u"Refresh", wx.DefaultPosition, wx.DefaultSize, wx.BU_EXACTFIT )
		self.ctl_Refresh.SetToolTipString( u"Refreshes the list of figures to reflect any changes to the scene" )
		
		box_SimulateButtons.Add( self.ctl_Refresh, 0, wx.ALL, 5 )
		
		
		box_Simulate.Add( box_SimulateButtons, 0, wx.EXPAND, 5 )
		
		
		self.pnl_Simulate.SetSizer( box_Simulate )
		self.pnl_Simulate.Layout()
		box_Simulate.Fit( self.pnl_Simulate )
		box_Run.Add( self.pnl_Simulate, 0, wx.BOTTOM|wx.EXPAND|wx.LEFT|wx.TOP, 10 )
		
		box_Extra = wx.BoxSizer( wx.VERTICAL )
		
		self.pnl_Wacros = wx.Panel( self, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, wx.SIMPLE_BORDER|wx.TAB_TRAVERSAL )
		self.pnl_Wacros.SetBackgroundColour( wx.Colour( 240, 240, 240 ) )
		self.pnl_Wacros.SetToolTipString( u"The currently selected material. Using these option from within the Material Room is recommended (but not essential)." )
		
		box_Wacros = wx.BoxSizer( wx.VERTICAL )
		
		self.lbl_Materials = wx.StaticText( self.pnl_Wacros, wx.ID_ANY, u"MATERIALS", wx.DefaultPosition, wx.DefaultSize, wx.ALIGN_CENTRE )
		self.lbl_Materials.Wrap( -1 )
		box_Wacros.Add( self.lbl_Materials, 0, wx.ALL|wx.EXPAND, 5 )
		
		self.m_staticline51 = wx.StaticLine( self.pnl_Wacros, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, wx.LI_HORIZONTAL )
		box_Wacros.Add( self.m_staticline51, 0, wx.EXPAND |wx.ALL, 5 )
		
		box_CurrentMat = wx.BoxSizer( wx.HORIZONTAL )
		
		self.lbl_CurrentMaterial = wx.StaticText( self.pnl_Wacros, wx.ID_ANY, u"Current mat:", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.lbl_CurrentMaterial.Wrap( -1 )
		box_CurrentMat.Add( self.lbl_CurrentMaterial, 0, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5 )
		
		self.ctl_CurrentMat = wx.TextCtrl( self.pnl_Wacros, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, wx.TE_CENTRE )
		self.ctl_CurrentMat.Enable( False )
		
		box_CurrentMat.Add( self.ctl_CurrentMat, 1, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5 )
		
		
		box_Wacros.Add( box_CurrentMat, 0, wx.EXPAND, 5 )
		
		box_Buttons = wx.GridSizer( 0, 2, 0, 0 )
		
		self.cmd_WacroAddLife = wx.Button( self.pnl_Wacros, wx.ID_ANY, u"+ Life node", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.cmd_WacroAddLife.SetToolTipString( u"The PRT3_Life node refects how far along its lifespan the particle is. The PRT3_Life node will be black (0.0) when the particle first spawns and white (1.0) when the particle is about to die.\n\nEvery material can only have one PRT3_Life node. The life node's value is updated whenever you copy or simulate materials. " )
		
		box_Buttons.Add( self.cmd_WacroAddLife, 0, wx.ALL, 5 )
		
		self.cmd_WacroRandomNode = wx.Button( self.pnl_Wacros, wx.ID_ANY, u"+ Random", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.cmd_WacroRandomNode.SetToolTipString( u"The PRT3_Random node adds a random color to your material.\n\nPRT3_Random nodes are updated whenever you copy or simulate materials. A new random color is picked whenever a particle spawns. " )
		
		box_Buttons.Add( self.cmd_WacroRandomNode, 0, wx.ALL, 5 )
		
		self.cmd_WacroColourByLife = wx.Button( self.pnl_Wacros, wx.ID_ANY, u"Life colour", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.cmd_WacroColourByLife.SetToolTipString( u"This wacro creates a ColorRamp node and attaches a PRT3_Life node to it.\n\nYou can use this setup to make a particle change color over the course of its life. The particle will spawn at with the ColorRamp's value at Color1 and die with the value at Color4. " )
		
		box_Buttons.Add( self.cmd_WacroColourByLife, 0, wx.ALL, 5 )
		
		self.cmd_WacroColourByRandom = wx.Button( self.pnl_Wacros, wx.ID_ANY, u"Random col.", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.cmd_WacroColourByRandom.SetToolTipString( u"This wacro creates a ColorRamp node and attaches a new PRT3_Random node to it.\n\nThis makes particles a random color from the ColorRamp. Remember, the PRT3_Random node's value will change whenever the particle spawns or you copy materials. " )
		
		box_Buttons.Add( self.cmd_WacroColourByRandom, 0, wx.ALL, 5 )
		
		self.cmd_WacroCopy = wx.Button( self.pnl_Wacros, wx.ID_ANY, u"Copy to all", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.cmd_WacroCopy.SetToolTipString( u"After you've setup the material how you want it, use the Copy to Other Particles wacro to spread your material to the rest of the particle figure.\n\nThe special PRT3_Life and PRT3_Random nodes will update for each particle when you do this. Copying materials will wipe out material keyframes, so use caution if you're animating. " )
		
		box_Buttons.Add( self.cmd_WacroCopy, 0, wx.ALL, 5 )
		
		self.cmd_WacroSimulate = wx.Button( self.pnl_Wacros, wx.ID_ANY, u"Simulate", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.cmd_WacroSimulate.SetToolTipString( u"If you're making an animation and want your materials to animate, you will need to run the Simulate Materials wacro.\n\nThis wacro updates the special PRT3_Life and PRT3_Random nodes for every frame of your animation. (PRT3_Random nodes are only updated when the particle spawns.)\n\nYou may run into some display issues if you make major changes to your material after first simulating and then copying. If you run into any problems, save your scene and then reopen it.\n\nMaterial simulations run much faster than normal simulations, though complex materials may slow them down. " )
		
		box_Buttons.Add( self.cmd_WacroSimulate, 0, wx.ALL, 5 )
		
		self.cmd_SpriteFix = wx.Button( self.pnl_Wacros, wx.ID_ANY, u"Fix Sprites", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.cmd_SpriteFix.SetToolTipString( u"Makes all Sprite particles point towards the camera" )
		
		box_Buttons.Add( self.cmd_SpriteFix, 0, wx.ALL, 5 )
		
		self.m_button19 = wx.Button( self.pnl_Wacros, wx.ID_ANY, u"Unused", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.m_button19.Enable( False )
		
		box_Buttons.Add( self.m_button19, 0, wx.ALL, 5 )
		
		
		box_Wacros.Add( box_Buttons, 1, wx.EXPAND, 5 )
		
		
		self.pnl_Wacros.SetSizer( box_Wacros )
		self.pnl_Wacros.Layout()
		box_Wacros.Fit( self.pnl_Wacros )
		box_Extra.Add( self.pnl_Wacros, 1, wx.EXPAND |wx.ALL, 10 )
		
		self.pnl_Info = wx.Panel( self, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, wx.TAB_TRAVERSAL )
		self.pnl_Info.SetBackgroundColour( wx.SystemSettings.GetColour( wx.SYS_COLOUR_APPWORKSPACE ) )
		
		box_Info = wx.BoxSizer( wx.HORIZONTAL )
		
		self.lbl_Info = wx.StaticText( self.pnl_Info, wx.ID_ANY, u"This project is based on code released by Poseworks under the Mozilla Public License 1.1.\nIt has been extensively modified by Snarlygribbly for use in Poser Pro 2014 and later versions.\nHave fun!", wx.DefaultPosition, wx.DefaultSize, wx.ALIGN_CENTRE )
		self.lbl_Info.Wrap( 300 )
		self.lbl_Info.SetForegroundColour( wx.SystemSettings.GetColour( wx.SYS_COLOUR_HIGHLIGHTTEXT ) )
		self.lbl_Info.SetMinSize( wx.Size( -1,125 ) )
		
		box_Info.Add( self.lbl_Info, 1, wx.ALIGN_CENTER_VERTICAL|wx.BOTTOM|wx.TOP, 5 )
		
		
		self.pnl_Info.SetSizer( box_Info )
		self.pnl_Info.Layout()
		box_Info.Fit( self.pnl_Info )
		box_Extra.Add( self.pnl_Info, 0, wx.EXPAND|wx.LEFT|wx.RIGHT, 10 )
		
		
		box_Run.Add( box_Extra, 1, wx.EXPAND, 5 )
		
		
		box_Action.Add( box_Run, 1, wx.EXPAND, 5 )
		
		
		box_Main.Add( box_Action, 1, wx.EXPAND, 5 )
		
		
		box_TopLevel.Add( box_Main, 1, wx.EXPAND, 5 )
		
		self.m_staticline7 = wx.StaticLine( self, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, wx.LI_HORIZONTAL )
		box_TopLevel.Add( self.m_staticline7, 0, wx.EXPAND |wx.ALL, 5 )
		
		self.ctl_StatusBar = wx.TextCtrl( self, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, wx.TE_READONLY )
		box_TopLevel.Add( self.ctl_StatusBar, 0, wx.ALL|wx.EXPAND, 5 )
		
		
		self.SetSizer( box_TopLevel )
		self.Layout()
		
		# Connect Events
		self.Bind( wx.aui.EVT_AUI_PANE_CLOSE, self.Close )
		self.ctl_CreateFigure.Bind( wx.EVT_BUTTON, self.CreateFigure )
		self.ctl_AddAttractor.Bind( wx.EVT_BUTTON, self.AddAttractor )
		self.ctl_Configure.Bind( wx.EVT_BUTTON, self.ConfigureActor )
		self.ctl_DeleteKeyframes.Bind( wx.EVT_BUTTON, self.DeleteKeyframes )
		self.ctl_Simulate.Bind( wx.EVT_BUTTON, self.Simulate )
		self.ctl_Refresh.Bind( wx.EVT_BUTTON, self.PopulateFigureList )
		self.cmd_WacroAddLife.Bind( wx.EVT_BUTTON, self.WacroAddLifeNode )
		self.cmd_WacroRandomNode.Bind( wx.EVT_BUTTON, self.WacroAddRandomNode )
		self.cmd_WacroColourByLife.Bind( wx.EVT_BUTTON, self.WacroColourByLife )
		self.cmd_WacroColourByRandom.Bind( wx.EVT_BUTTON, self.WacroColourByRandom )
		self.cmd_WacroCopy.Bind( wx.EVT_BUTTON, self.WacroCopyToAll )
		self.cmd_WacroSimulate.Bind( wx.EVT_BUTTON, self.WacroSimulate )
		self.cmd_SpriteFix.Bind( wx.EVT_BUTTON, self.SpriteFix )
	
	def __del__( self ):
		pass
	
	
	# Virtual event handlers, overide them in your derived class
	def Close( self, event ):
		event.Skip()
	
	def CreateFigure( self, event ):
		event.Skip()
	
	def AddAttractor( self, event ):
		event.Skip()
	
	def ConfigureActor( self, event ):
		event.Skip()
	
	def DeleteKeyframes( self, event ):
		event.Skip()
	
	def Simulate( self, event ):
		event.Skip()
	
	def PopulateFigureList( self, event ):
		event.Skip()
	
	def WacroAddLifeNode( self, event ):
		event.Skip()
	
	def WacroAddRandomNode( self, event ):
		event.Skip()
	
	def WacroColourByLife( self, event ):
		event.Skip()
	
	def WacroColourByRandom( self, event ):
		event.Skip()
	
	def WacroCopyToAll( self, event ):
		event.Skip()
	
	def WacroSimulate( self, event ):
		event.Skip()
	
	def SpriteFix( self, event ):
		event.Skip()
				
# The Particles class is our core application code
class Particles (UI):
	
	def __init__(self, parent, main, cfg):
	
		# Horrible klunky fudge
		global ME
		ME = self
	
		# Initialise the user interface
		UI.__init__(self, parent)
		self.style = self.GetWindowStyle()
		self.parent = parent
		self.main = main
		self.config = cfg

		# Class variables
		self.scene = poser.Scene()
		self.lastActor = None
		self.CurrentActor = None
		self.CurrentMaterial = None
		
		# Tidy up the UI a bit :-)
		for ctl in (self.lbl_Figure, self.lbl_Attractor, self.lbl_Configure, self.lbl_Simulate, self.lbl_Materials):
			font = ctl.GetFont()
			font.SetWeight(wx.FONTWEIGHT_BOLD)
			ctl.SetFont(font)
		
		# Add the icons
		icons = MyIcons()
		self.img_Group.SetBitmap(icons.group.GetBitmap())
		self.img_Attractor.SetBitmap(icons.attractor.GetBitmap())
		
		# Set up the toolbar
		self.bar_Toolbar.SetToolBitmapSize((50, 50))
		self.bar_Toolbar.AddLabelTool( 1, u"Import", icons.load.GetBitmap(), wx.NullBitmap, wx.ITEM_NORMAL, u"Import presets", u"Import presets", None ) 		
		self.bar_Toolbar.AddLabelTool( 2, u"Export", icons.save.GetBitmap(), wx.NullBitmap, wx.ITEM_NORMAL, u"Export presets", u"Export presets", None ) 		
		self.bar_Toolbar.AddLabelTool( 3, u"Settings", icons.settings.GetBitmap(), wx.NullBitmap, wx.ITEM_NORMAL, u"Script settings", u"Script settings", None ) 		
		self.bar_Toolbar.AddLabelTool( 4, u"Website", icons.web.GetBitmap(), wx.NullBitmap, wx.ITEM_NORMAL, u"Go to Poseworks website",u"Go to Poseworks website", None ) 		
		self.bar_Toolbar.AddLabelTool( 5, u"About", icons.info.GetBitmap(), wx.NullBitmap, wx.ITEM_NORMAL, u"About %s" % APPNAME, u"About %s" % APPNAME, None ) 		
		self.bar_Toolbar.AddLabelTool( 6, u"Close", icons.close.GetBitmap(), wx.NullBitmap, wx.ITEM_NORMAL, u"Close %s" % APPNAME, u"Close %s" % APPNAME, None ) 		
		self.bar_Toolbar.Realize()
		
		# ... and bind them to our methods
		self.Bind( wx.EVT_TOOL, self.Import, id = 1 )
		self.Bind( wx.EVT_TOOL, self.Export, id = 2 )
		self.Bind( wx.EVT_TOOL, self.Settings, id = 3 )
		self.Bind( wx.EVT_TOOL, self.Web, id = 4 )
		self.Bind( wx.EVT_TOOL, self.About, id = 5 )
		self.Bind( wx.EVT_TOOL, self.Close, id = 6 )
		
		# Populate figure list
		self.PopulateFigureList()
		
		# Initialise the timer
		self.timer = wx.Timer(self)
		self.Bind(wx.EVT_TIMER, self.Update, self.timer)
		self.timer.Start(500)

	def Close(self, event):
	
		# Get our pane object
		man = self.main.manager
		pane = man.GetPane(self)
		
		# Write the presets config file to disk
		try:
			self.config.set('PRT3', 'xpos', pane.floating_pos[0]) 
			self.config.set('PRT3', 'ypos', pane.floating_pos[1]) 
			self.config.set('PRT3', 'xwidth', pane.floating_size[0]) 
			self.config.set('PRT3', 'yheight', pane.floating_size[1]) 
			self.main.SaveConfig()
		except:
			ErrorBox('Unable to open configuration file for writing')	
		
		# Close the pane
		pane.Hide()
		man.Update()
		man.ClosePane(pane)
			
	def About( self, event ):
		
		about = wx.AboutDialogInfo()
		
		about.SetName(APPNAME)
		about.SetVersion(BUILD)
		about.AddDeveloper('Snarlygribbly, Poseworks, Wimvdb')
		about.AddDocWriter('Wimvdb')
		about.SetWebSite('http://3d.snarlygribbly.org')
		about.SetCopyright('(c) 2015 Snarlygribbly')
		about.SetArtists(['Biscuits', 'wimvdb'])

		wx.AboutBox(about)
		
	def AddAttractor(self, event):
		
		# Load the prop ...
		self.scene.LoadLibraryProp(os.path.join(DIR_RESOURCES, "attractor.pp2"))
		
		# Configure the new attractor
		actor = self.scene.CurrentActor()
		if self.Type(actor) == ATTRACTOR_TYPE:
			actor.SetName(self.ctl_AttractorName.GetValue())
			actor.Parameter(IPARM_ATTRACTION).SetValue(self.ctl_AttrStrength.GetValue() / 1000.0)
			actor.Parameter(IPARM_RANGE).SetValue(self.ctl_AttrRange.GetValue() / 1000.0)
			actor.Parameter(IPARM_FALLOFF).SetValue(self.ctl_AttrFalloff.GetValue() / 1000.0)
		
		# Inform the user
		self.StatusBar('New attractor added to scene')
		
	def ConfigureActor(self, event):
		type = self.Type(self.CurrentActor)
		
		if type == 1:
			self.EditEmitter(self.CurrentActor)
		elif type == 2:
			self.EditFigure(self.CurrentActor)
		elif type == 3:
			self.EditFigure(self.CurrentActor.ItsFigure().ParentActor())
		elif type == 4:
			self.EditAttractor(self.CurrentActor)
		else:
			ErrorBox('Function not yet available for this object type - %s' % type)
		
	def CreateFigure(self, event):
		name = self.ctl_FigureName.GetValue()
		count = self.ctl_NoOfParticles.GetValue()
		shape = self.ctl_Shape.GetSelection()
		
		# Deal with custom particles
		if shape == 0: shape = self.FindGeometry()		
		
		system = createSystem(name, [("Particle Group", count, shape)])
		self.StatusBar('New particle figure added to scene')
		self.ctl_TheseFigures.Append(name)
	
	def DeleteKeyframes(self, event):
	
		f = self.scene.Frame()
		a = self.CurrentActor
	
		try:
			self.scene.SetFrame(1)
		except:
			return
		
		while True:
			kf = a.NextKeyFrame()			
			if kf:
				a.DeleteKeyFrame(kf)
				self.scene.SetFrame(kf)
			else:
				break
		
		self.StatusBar("All keyframes deleted for %s actor" % a.Name())
		self.scene.SetFrame(f)
		self.scene.DrawAll()
	
	def EditAttractor(self, event):
		data = (
			(PARM_ATTRACTION,(PARM_ATTRACTION, 'float', (-10000.00, 10000.00), self.CurrentActor.Parameter(PARM_ATTRACTION).Value(), '', None)),	
			(PARM_RANGE,(PARM_RANGE, 'float', (-10000.00, 10000.00), self.CurrentActor.Parameter(PARM_RANGE).Value(), '', None)),	
			(PARM_FALLOFF,(PARM_FALLOFF, 'float', (-1.00, 1.00), self.CurrentActor.Parameter(PARM_FALLOFF).Value(), '', None)),	
			)
		
		dlg = EZDialog('Edit attractor', data)
		
		if dlg.ShowModal():
			for name, value in dlg.Values().items():
				parm = self.CurrentActor.Parameter(name)
				if parm:
					parm.SetValue(value)
				
		dlg.Destroy()
	
	def EditEmitter(self, actor):
		
		data = (
			(PARM_EMISSIONANGLE,(PARM_EMISSIONANGLE, 'float', (-360.00, 360.00), self.CurrentActor.Parameter(PARM_EMISSIONANGLE).Value(), 'The Angle of Emission property changes how wide the spread of spawn vectors is. A low Angle of Emission keeps the particles focused like a laser, while a wide angle makes the particles spread out like a dandelion head.', None)),	
			)
		
		dlg = EZDialog('Edit emitter', data)
		
		if dlg.ShowModal():
			for name, value in dlg.Values().items():
				parm = self.CurrentActor.Parameter(name)
				if parm:
					parm.SetValue(value)
				
		dlg.Destroy()

	def EditFigure(self, actor):
		p = actor.Parameter

		# Tooltip definitions
		STRING_STARTEND_TIP = """The Start Frame is the first frame of your animation that particles will be spawned. For example, if you set
the Start Frame to 10, no particles will appear until frame 10.

The End Frame is the last frame of your animation that particles can be spawned. If you set the
End Frame to 30, new particles will stop appearing after frame 30. (Any existing particles will continue
to age and move.)

Setting the End Frame to -1 will automatically use the last frame in the animation."""	

		STRING_TRANSFORM_TIP = """Transform Controls change the initial state of each particle.

Initial Scale sets how large the particle is when it is spawned.

Initial Rotation controls the angle of the particle on each axis
when it is spawned.

Initial Offset controls the distance between the particle and the
emitter when it is spawned. Remember that particles come
from the emitter radially, so setting Initial Offset X to 1.0
will create a circle of particles around the emitter with a radius
of 1.0."""

		STRING_VELOCITY_TIP = """Velocity causes particles to move in a direction. For instance,
Velocity Y causes particles to move up along their spawn vectors.

Velocity is measured in Poser units per second, so a Velocity Y of
1.0 will move particles up 1.0 units per second. In Poser, there is
a default of 30 frames per second. """

		STRING_ORBIT_TIP = """Orbit causes the particles to move around an axis, like the
planets orbit around the sun.

If Orbit Y is 15, particles will Y-Rotate 15 degrees per frame
around wherever the emitter was when the particle spawned."""

		STRING_SPIN_TIP = """Spin causes each particle to rotate on its own axis.

If Spin Y is 15, particles will Y-Rotate 15 degrees per frame.

If Spin X is -15, particles will X-Rotate -15 degrees per frame."""

		STRING_GROW_TIP = """The Grow properties cause particles to change in scale.

If Grow Y is 2, particles will increase in Y-Scale by 2% every
frame. If Grow Y is -2, particles will shrink vertically by 2%
every frame."""
	
		data = (
			('label1',('--- SIMULATION ---', 'label', None, None, '', None) ),
			(PARM_EMISSION,(PARM_EMISSION, 'int', (-10000.00, 10000.00), p(PARM_EMISSION).Value(), 'Rate of Emission is how many particles spawn per frame.', None)),	
			(PARM_RECYCLE,(PARM_RECYCLE, 'bool', (None), p(PARM_RECYCLE).Value(), 'Recycle Particles allows dead particles to spawn again', None)),	
			(PARM_LIFESPAN,('Lifespan', 'int', (-10000.00, 10000.00), p(PARM_LIFESPAN).Value(), 'Lifespan sets the number of frames that a particle will live after it is spawned.', None)),	
			(PARM_START,(PARM_START, 'int', (-10000.00, 10000.00), p(PARM_START).Value(), STRING_STARTEND_TIP, None)),	
			(PARM_END,(PARM_END, 'int', (-10000.00, 10000.00), p(PARM_END).Value(), STRING_STARTEND_TIP	, None)),	
			(PARM_GRAVITY,(PARM_GRAVITY, 'float', (-10.00, 10.00), p(PARM_GRAVITY).Value(), '', 0.05)),	
			('label1',('--- TRANSFORM ---', 'label', None, None, '', None) ),
			(PARM_PSCALE,(PARM_PSCALE, 'float', (-10000.00, 10000.00), p(PARM_PSCALE).Value(), STRING_TRANSFORM_TIP, None)),	
			(PARM_PXROTATE,(PARM_PXROTATE, 'float', (-10000.00, 10000.00), p(PARM_PXROTATE).Value(), STRING_TRANSFORM_TIP, 0.10)),	
			(PARM_PYROTATE,(PARM_PYROTATE, 'float', (-10000.00, 10000.00), p(PARM_PYROTATE).Value(), STRING_TRANSFORM_TIP, 0.10)),	
			(PARM_PZROTATE,(PARM_PZROTATE, 'float', (-10000.00, 10000.00), p(PARM_PZROTATE).Value(), STRING_TRANSFORM_TIP, 0.10)),	
			(PARM_PXTRAN,(PARM_PXTRAN, 'float', (-10000.00, 10000.00), p(PARM_PXTRAN).Value(), STRING_TRANSFORM_TIP, 0.10)),	
			(PARM_PYTRAN,(PARM_PYTRAN, 'float', (-10000.00, 10000.00), p(PARM_PYTRAN).Value(), STRING_TRANSFORM_TIP, 0.10)),	
			(PARM_PZTRAN,(PARM_PZTRAN, 'float', (-10000.00, 10000.00), p(PARM_PZTRAN).Value(), STRING_TRANSFORM_TIP, 0.10)),	
			('label1',('--- VELOCITY ---', 'label', None, None, '', None) ),
			(PARM_VELOCITYX,(PARM_VELOCITYX, 'float', (-10000.00, 10000.00), p(PARM_VELOCITYX).Value(), STRING_VELOCITY_TIP, 0.10)),	
			(PARM_VELOCITYY,(PARM_VELOCITYY, 'float', (-10000.00, 10000.00), p(PARM_VELOCITYY).Value(), STRING_VELOCITY_TIP, 0.10)),	
			(PARM_VELOCITYZ,(PARM_VELOCITYZ, 'float', (-10000.00, 10000.00), p(PARM_VELOCITYZ).Value(), STRING_VELOCITY_TIP, 0.10)),	
			(PARM_ACCELERATIONX,(PARM_ACCELERATIONX, 'float', (-10000.00, 10000.00), p(PARM_ACCELERATIONX).Value(), STRING_VELOCITY_TIP, 0.10)),	
			(PARM_ACCELERATIONY,(PARM_ACCELERATIONY, 'float', (-10000.00, 10000.00), p(PARM_ACCELERATIONY).Value(), STRING_VELOCITY_TIP, 0.10)),	
			(PARM_ACCELERATIONZ,(PARM_ACCELERATIONZ, 'float', (-10000.00, 10000.00), p(PARM_ACCELERATIONZ).Value(), STRING_VELOCITY_TIP, 0.10)),	
			(PARM_VELOCITYVAR,(PARM_VELOCITYVAR, 'float', (-10000.00, 10000.00), p(PARM_VELOCITYVAR).Value(), STRING_VELOCITY_TIP, 0.10)),	
			('label1',('--- ORBIT ---', 'label', None, None, '', None) ),
			(PARM_ORBITX,(PARM_ORBITX, 'float', (-10000.00, 10000.00), p(PARM_ORBITX).Value(), STRING_ORBIT_TIP, 0.20)),	
			(PARM_ORBITY,(PARM_ORBITY, 'float', (-10000.00, 10000.00), p(PARM_ORBITY).Value(), STRING_ORBIT_TIP, 0.20)),	
			(PARM_ORBITZ,(PARM_ORBITZ, 'float', (-10000.00, 10000.00), p(PARM_ORBITZ).Value(), STRING_ORBIT_TIP, 0.20)),	
			(PARM_ORBITAX,(PARM_ORBITAX, 'float', (-10000.00, 10000.00), p(PARM_ORBITAX).Value(), STRING_ORBIT_TIP, 0.10)),	
			(PARM_ORBITAY,(PARM_ORBITAY, 'float', (-10000.00, 10000.00), p(PARM_ORBITAY).Value(), STRING_ORBIT_TIP, 0.10)),	
			(PARM_ORBITAZ,(PARM_ORBITAZ, 'float', (-10000.00, 10000.00), p(PARM_ORBITAZ).Value(), STRING_ORBIT_TIP, 0.10)),	
			(PARM_ORBITVAR,(PARM_ORBITVAR, 'float', (-10000.00, 10000.00), p(PARM_ORBITVAR).Value(), STRING_ORBIT_TIP, 0.10)),	
			('label1',('--- SPIN ---', 'label', None, None, '', None) ),
			(PARM_SPINX,(PARM_SPINX, 'float', (-10000.00, 10000.00), p(PARM_SPINX).Value(), STRING_SPIN_TIP, 1.00)),	
			(PARM_SPINY,(PARM_SPINY, 'float', (-10000.00, 10000.00), p(PARM_SPINY).Value(), STRING_SPIN_TIP, 1.00)),	
			(PARM_SPINZ,(PARM_SPINZ, 'float', (-10000.00, 10000.00), p(PARM_SPINZ).Value(), STRING_SPIN_TIP, 1.00)),	
			(PARM_SPINAX,(PARM_SPINAX, 'float', (-10000.00, 10000.00), p(PARM_SPINAX).Value(), STRING_SPIN_TIP, 0.10)),	
			(PARM_SPINAY,(PARM_SPINAY, 'float', (-10000.00, 10000.00), p(PARM_SPINAY).Value(), STRING_SPIN_TIP, 0.10)),	
			(PARM_SPINAZ,(PARM_SPINAZ, 'float', (-10000.00, 10000.00), p(PARM_SPINAZ).Value(), STRING_SPIN_TIP, 0.10)),	
			(PARM_SPINVAR,(PARM_SPINVAR, 'float', (-10000.00, 10000.00), p(PARM_SPINVAR).Value(), STRING_SPIN_TIP, 0.10)),	
			('label1',('--- GROW ---', 'label', None, None, '', None) ),
			(PARM_GROWX,(PARM_GROWX, 'float', (-10000.00, 10000.00), p(PARM_GROWX).Value(), STRING_GROW_TIP, 0.10)),	
			(PARM_GROWY,(PARM_GROWY, 'float', (-10000.00, 10000.00), p(PARM_GROWY).Value(), STRING_GROW_TIP, 0.10)),	
			(PARM_GROWZ,(PARM_GROWZ, 'float', (-10000.00, 10000.00), p(PARM_GROWZ).Value(), STRING_GROW_TIP, 0.10)),	
			(PARM_GROWAX,(PARM_GROWAX, 'float', (-10000.00, 10000.00), p(PARM_GROWAX).Value(), STRING_GROW_TIP, 0.10)),	
			(PARM_GROWAY,(PARM_GROWAY, 'float', (-10000.00, 10000.00), p(PARM_GROWAY).Value(), STRING_GROW_TIP, 0.10)),	
			(PARM_GROWAZ,(PARM_GROWAZ, 'float', (-10000.00, 10000.00), p(PARM_GROWAZ).Value(), STRING_GROW_TIP, 0.10)),	
			(PARM_GROWVAR,(PARM_GROWVAR, 'float', (-10000.00, 10000.00), p(PARM_GROWVAR).Value(), STRING_GROW_TIP, 0.10)),	
			)
		
		dlg = EZDialog('Edit group', data)
		
		if dlg.ShowModal():
			for name, value in dlg.Values().items():
				parm = actor.Parameter(name)
				if parm:
					parm.SetValue(value)
				
		dlg.Destroy()		
	
	def Export(self, event):
		
		fig = self.scene.CurrentFigure()
		if fig:
			if isParticleFigure(fig):
				path = self.SelectFile(folder=self.config.get('PRT3', 'PresetFolder'), type=wx.SAVE)
				if path:
					savePreset(fig, path)
					self.config.set('PRT3', 'PresetFolder', os.path.dirname(path))
		else:
			ErrorBox('No particle figure selected')
		
	def FindGeometry(self):
		path = self.SelectFile(folder=self.config.get('PRT3', 'GeomFolder'), filetype="*.obj")
		if path:
			self.config.set('PRT3', 'GeomFolder', os.path.dirname(path))
		return path
	
	def Import(self, event):
		
		path = self.SelectFile(folder=self.config.get('PRT3', 'PresetFolder'))
		
		if path:
			sysData = readPreset(path)
			createSystem(sysData["name"], [(sysData["name"], sysData["count"], sysData["shape"])])
			self.scene.LoadLibraryPose(path)
			figure = self.scene.CurrentFigure()
			self.Copy(figure.ParentActor(), figure)
			self.config.set('PRT3', 'PresetFolder', os.path.dirname(path))
			
		self.PopulateFigureList()
		self.StatusBar('Preset %s imported' % sysData["name"])
		
	def IsValidFigure(self, fig):
		if fig:
			return isParticleFigure(fig)
		else:
			return False
	
	def IsValidParticle(self, actor):
		if actor:
			if actor.IsBodyPart():
				parm = actor.Parameter(PARM_TYPE)
				if parm and (parm.Value() == PARTICLE_TYPE or parm.Value() == GROUP_TYPE):
					return parm

		return None
	
	def NotYet(self, event):
		ErrorBox("Not implemented yet")
	
	def PopulateFigureList(self, event=None):
		self.ctl_TheseFigures.Clear()
		for s in getSystems():
			self.ctl_TheseFigures.Append(s.Name())
		self.StatusBar('Figure list refreshed')

	def SelectFile(self, title="Choose a file", type=wx.OPEN, folder="", filetype="*.pz2"):
		
		self.Hide()
		
		try:
			fdlg = wx.FileDialog(None,
				title,
				defaultDir=folder,
				defaultFile='',
				wildcard=filetype,
				style=type,
				pos=wx.DefaultPosition)
				
			if fdlg.ShowModal() == wx.ID_OK:			# User chose a file	
				path = fdlg.GetPath()					# get the path to the file	
			else:
				path = None
				
			fdlg.Destroy()
			
		except:
			pass
			
		finally:		
			self.Show()
		
		return path
	
	def Settings (self, event):
		data = (
			('remember_pos',('Save UI position', 'bool', (None), GetConfig(self.config, 'remember_pos', 'boolean'), 'Remember the position and size of the user interface', None)),
			)
			
		dlg = EZDialog('Edit settings', data)
		
		if dlg.ShowModal():
			for name, value in dlg.Values().items():
				self.config.set('PRT3', name, value)
				
		dlg.Destroy()
		
	def Simulate(self, event):
	
		# Hack for avoiding random rotations
		global USE_RANDOM
		USE_RANDOM = self.ctl_RandomRot.IsChecked()
	
		# Get the current figure ...
		fig = self.scene.CurrentFigure()
		
		# ... and make sure it is a particle figure 
		if fig and self.IsValidFigure(fig):
			
			opt = self.ctl_RunFor.GetSelection()
			
			# Current figure only
			if opt == 0:						
				simulate(buildSystemFrom(fig), inplace=self.ctl_InPlace.IsChecked(), update=self.StatusBar)
			
			# All figures
			elif opt == 1:						 
				for f in self.scene.Figures():
					if self.IsValidFigure(f): simulate(buildSystemFrom(f), inplace=self.ctl_InPlace.IsChecked(), update=self.StatusBar)
					self.StatusBar('Simulation complete')
			
			# Selected figures
			else:
				
				ctl = self.ctl_TheseFigures
				remove = []
				
				# Get each selected figure
				for i in ctl.GetSelections():
					itm = ctl.GetString(i)
					
					# If it doesn't exist anymore, add it to the remove list
					try:
						fig = self.scene.Figure(itm)
					except:
						remove.append(itm)
						continue
					
					# Simulate
					self.StatusBar("Simulating for %s" % fig.Name())
					simulate(buildSystemFrom(fig), inplace=self.ctl_InPlace.IsChecked(), update=self.StatusBar)

				# Remove any missing figures
				for itm in remove:
					s = ctl.FindString(itm)
					ctl.Delete(s)
					self.StatusBar("%s removed from list" % itm)

		else:
			ErrorBox("No particle figure selected.")

	def SpriteFix(self, event):
		figure = poser.Scene().CurrentFigure()
		camera = poser.Scene().CurrentCamera()

		if figure and camera:
			i = 0
			go = 1
			count = 0
			while go:
				try:
					particle = figure.Actor("Particle_%i" % i)
					shape = particle.Parameter(PARM_SHAPE)
				except:
					break
				if shape == SHAPE_SPRITE:
					particle.PointAt(camera)
					count += 1
				i += 1
		self.StatusBar('%s sprites updated' % count)
		
	def StatusBar(self, msg):
		self.ctl_StatusBar.SetValue(msg)
		self.scene.ProcessSomeEvents(10)
		
	def Type(self, actor):
		parm = actor.Parameter(PARM_TYPE)
		if parm:
			return int(parm.Value())
		else:
			return None
	
	def Update(self, event):
	
		# Get the current material
		self.CurrentMaterial = self.scene.CurrentMaterial()
		if self.CurrentMaterial:
			self.ctl_CurrentMat.SetValue(self.CurrentMaterial.Name())
		else:
			self.ctl_CurrentMat.SetValue('No Material selected')
		
		# Get the current actor
		actor = self.scene.CurrentActor()
		
		# Record lastActor
		try:
			self.lastActor = actor.InternalName()
		except:
			try:
				actor = self.scene.ActorByInternalName(lastActor)
			except:
				actor = None
		
		# Validate actor		
		if actor:
			if actor.IsCamera() or actor.IsLight():
				actor = self.scene.CurrentFigure()
				if actor:
					actor = actor.ParentActor()
				else:
					actor = None
					
		# Update the UI
		if actor:
		
			# Get parent's name if applicable
			if actor.IsBodyPart():
				parent = " (of %s)" % actor.ItsFigure().Name()
			else:
				parent = ""
				
			self.txt_SelectedActor.SetValue("%s %s" % (actor.Name(), parent))
				
			type = self.Type(actor)
			if type:
				self.txt_ActorType.SetValue(('System', 'Emitter', 'Group', 'Particle', 'Attractor', 'Deflector')[type])
				self.ctl_Configure.Enable()
				if type in (1, 2):
					self.ctl_DeleteKeyframes.Enable()
				else:
					self.ctl_DeleteKeyframes.Disable()
					
			else:
				self.txt_ActorType.SetValue('Not valid')
				self.ctl_Configure.Disable()
				self.ctl_DeleteKeyframes.Disable()
		else:
			self.txt_SelectedActor.SetValue('Not valid')
			self.txt_ActorType.SetValue('Not valid')	
			self.ctl_Configure.Disable()
			self.ctl_DeleteKeyframes.Disable()
				
		self.CurrentActor = actor
		
		# Prevent further processing of this event
		event.Skip(False)
	
	def WacroAddLifeNode(self, event):
		actor = self.CurrentActor
		parm = self.IsValidParticle(actor)
		if parm:
			if findLifeNode(actor):
				ErrorBox('Material already has a PRT3_Life node.')
			else:
				addLifeNode(actor, parm.Value())
				try:
					self.scene.CurrentMaterial().UpdatePreview()
				except:
					pass
					
				self.StatusBar('Life node added to particle %s' % actor.Name())
		else:
			ErrorBox("No particle figure selected.")
	
	def WacroAddRandomNode(self, event):
		actor = self.CurrentActor
		parm = self.IsValidParticle(actor)
		if parm:
			addRandomNode(actor)
			self.StatusBar('Random node added to particle %s' % actor.Name())
		else:
			ErrorBox("No particle figure selected.")		
	
	def WacroColourByLife(self, event):
		actor = self.CurrentActor
		parm = self.IsValidParticle(actor)
		if parm:
			addColorByLife(actor)
			self.StatusBar('Colour by Life nodes added to particle %s' % actor.Name())
		else:
			ErrorBox("No particle figure selected.")			

	def WacroColourByRandom(self, event):
		actor = self.CurrentActor
		parm = self.IsValidParticle(actor)
		if parm:
			addColorByRandom(actor)
			self.StatusBar('Colour by Random nodes added to particle %s' % actor.Name())
		else:
			ErrorBox("No particle figure selected.")
			
	def WacroSimulate(self, event):
		self.timer.Stop()
		actor = self.CurrentActor
		parm = self.IsValidParticle(actor)
		if parm:
			nodesimulate(self.scene.CurrentFigure(), self.StatusBar)
			self.StatusBar('Node simulation complete')
		else:
			ErrorBox("No particle figure selected.")
		self.timer.Start()
			
	def WacroCopyToAll(self, event):
		self.timer.Stop()
		act = self.CurrentActor
		if self.IsValidParticle(act):
			figure = act.ItsFigure()
			self.Copy(act, figure)
			self.StatusBar('Copying complete')
			self.scene.DrawAll()
		else:
			ErrorBox('Selected actor is not a particle')
		self.timer.Start()
	
	def Web(self, event):
		url = "http://www.poseworks.com/particles3/"
		webbrowser.open(url,new=2)

	def Copy(self, act, fig):

		# Get the actors to whom the materials will be applied
		actors = [x for x in fig.Actors() if (x.Name() != act.Name() and ValidActor(x))]
		
		# Get each actor in turn
		for actor in actors:

			# Process each target actor
			for material in act.Materials():
				m = material.Name()
				self.StatusBar('Copying material %s' % m)
				
				# Get the PoserMat object for the source material
				mat = poserMat(material, POSER_VERSION, useExt=True, getMaps=False)
				
				self.StatusBar('Copying material %s to actor %s' % (m, actor.Name()))
		
				target = actor.Material(m)
				
				if target:
					to_mat = poserMat(target, POSER_VERSION, useExt=True, getMaps=False)
					
					# Clear the target PoserSurface
					to_mat.ClearPoserSurface(severe=True)
					to_mat.RemoveOrphans()
					
					# Copy the PS input values
					self.CopyTheLot(mat.root, to_mat.root)
					
					# Copy all nodes
					xref = {}
					for old in mat.AllNodes():
						if old == mat.tree.Node(0):
							new = to_mat.tree.Node(0)
						else:
							new = to_mat.tree.CreateNode(old.Type())
						x,y = old.Location()
						new.SetLocation(x,y)
						new.SetName(old.Name())
						new.SetInputsCollapsed(old.InputsCollapsed())
						new.SetPreviewCollapsed(old.PreviewCollapsed())
						self.CopyTheLot(old, new)
						xref[old.InternalName()] = (old,new)
						
					# Connect'em up
					for k, v in xref.items():
						old, new = v
						for idx, inp in enumerate(old.Inputs()):
							i = inp.InNode()
							if i:
								to_mat.tree.AttachTreeNodes(new, inp.InternalName(), xref[i.InternalName()][1])
								
						# Process texture maps
						if old in mat.Maps():
							idx = mat.Maps().index(old)
							map = to_mat.Maps()[idx]
							if map:
								self.CopyTheLot(map, new)
		
			material.ShaderTree().UpdatePreview()
					
	def CopyTheLot(self, fromNode, toNode):
		for inp in fromNode.Inputs():
			to_input = toNode.InputByInternalName(inp.InternalName())
			if to_input:
				self.SetInputValue(to_input, inp.Value())
				to_input.SetAnimated(inp.Animated())
	
	def SetInputValue(self, input, value):
		if input:
			inputType = input.Type()
			if inputType in [0, 4, 5, 6]:
				try: input.SetFloat(value)
				except: pass
			elif inputType in [1, 2]:
				try: 
					r, g, b = value
					input.SetColor(r, g, b)
				except: pass
			elif inputType == 3:
				try: input.SetString(value)
				except: pass
				
# Main class manages the whole show
class Main(object):
	
	def __init__(self):
	
		# Read the configuraton file
		self.config = ConfigParser.RawConfigParser()
		self.cfgfile = os.path.join(poser.PrefsLocation(), 'prt3' + os.extsep + 'cfg')
		self.config.read(self.cfgfile)
		if not self.config.has_section('PRT3'): self.config.add_section('PRT3')
		if not self.config.has_option('PRT3', 'PresetFolder'): self.config.set('PRT3', 'PresetFolder', '')
		if not self.config.has_option('PRT3', 'GeomFolder'): self.config.set('PRT3', 'GeomFolder', '')
		if not self.config.has_option('PRT3', 'remember_pos'): self.config.set('PRT3', 'remember_pos', False)
		if not self.config.has_option('PRT3', 'xpos'): self.config.set('PRT3', 'xpos', 0)
		if not self.config.has_option('PRT3', 'ypos'): self.config.set('PRT3', 'ypos', 0)
		if not self.config.has_option('PRT3', 'xwidth'): self.config.set('PRT3', 'xwidth', APPSIZE[0])
		if not self.config.has_option('PRT3', 'yheight'): self.config.set('PRT3', 'yheight', APPSIZE[1])
	
		self.manager = poser.WxAuiManager()
		self.root = self.manager.GetManagedWindow()
		self.pane = self.InitialisePane(wx.aui.AuiPaneInfo())
		
		# Launch the UI
		self.prt3 = Particles(self.root, self, self.config)
		self.manager.AddPane(self.prt3, self.pane)
		self.manager.GetPane(self.prt3).Show()
		self.manager.Update()
		
	def AlreadyRunning(self):
		# Returns True if script is already running, else False
		
		for pane in self.manager.GetAllPanes():
			if pane.caption == APPCAPTION:
				return True
		return False
		
	def InitialisePane(self, pane):
		# Provides the default configuration for our AuiPaneInfo
		
		# Get user preferences
		if GetConfig(self.config, 'remember_pos', 'boolean') == True:
			mysize = (GetConfig(self.config, 'xwidth','int'), GetConfig(self.config, 'yheight', 'int'))
			mypos = (GetConfig(self.config, 'xpos', 'int'), GetConfig(self.config, 'ypos', 'int'))
		else:
			mysize = APPSIZE
			mypos = (0,0)
			
		pane.Caption(APPCAPTION)
		pane.CaptionVisible(False)
		pane.Dockable(False)
		pane.Name(APPNAME)
		pane.FloatingSize(mysize)
		pane.MinSize(APPSIZE)
		pane.BestSize(APPSIZE)
		pane.DestroyOnClose(True)		
		pane.Gripper(False)
		pane.CloseButton(False)
		pane.Float()
		pane.Hide()
		pane.FloatingPosition(mypos)
		pane.Layer(0)
		pane.Position(0)
		pane.Row(1)
		pane.Direction(4)		
		
		return pane
		
	def SaveConfig(self):
		cfgfile = open(self.cfgfile, 'wb')
		self.config.write(cfgfile)

# Here we go ...
app = Main()